/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_bemf.c
*
* @brief This file provides bldc sensorless control functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "gpio_drv.h"
#include "bldc_keys.h"
#include "bldc_pwm.h"
#include "bldc_bemf.h"
#include "bldc_startup.h"
#include "bldc_parameters_define.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Storage data to buffer periodicity.
*
* @param[in] buffer: pointer to BUFFER_TYPE structure
* @param[in] data: the data to storage in buffer every cycle
* @return none
*/
void StorageDataCyclic(BUFFER_TYPE *buffer, float data)
{
    *(buffer->array + buffer->bufferIndex) = data;
    buffer->bufferIndex++;
    if (buffer->bufferIndex >= BUFFER_SIZE)
    {
        buffer->bufferIndex = 0;
    }
}

/*!
* @brief Median filter buffer data and return average value.
*
* @param[in] buffer: pointer to BUFFER_TYPE structure
* @param[in] n: the data numbers to be filtered
* @return average value of buffer data
*/
float Filter_AverageCalc(BUFFER_TYPE *buffer, float n)
{
    float max = 0.0f, min = 0.0f, tempValue = 0.0f;
    uint16_t j = 0;

    max = min = tempValue = *buffer->array;

    for (j = 1; j < n; j++)
    {
        if (max < *(buffer->array + j))
        {
            max = *(buffer->array + j);
        }
        else if (min > *(buffer->array + j))
        {
            min = *(buffer->array + j);
        }
        else
        {
        }

        tempValue += *(buffer->array + j);
    }

    buffer->maxValue = max;
    buffer->minValue = min;

    tempValue -= (max + min);

    if (n > 2)
    {
        tempValue /= (n - 2);
    }

    return tempValue;
}

/*!
* @brief calculate and updata the average change phase time.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @return none
*/
void ChangePhaseTimeCalc(BLDC_VARS_CTRL *bldcCtrl)
{
    StorageDataCyclic(&bldcCtrl->pBemf->commutationTime, bldcCtrl->pBemf->saveChangePhaseTime);
    bldcCtrl->pBemf->aveChangePhaseTime = Filter_AverageCalc(&bldcCtrl->pBemf->commutationTime, BUFFER_SIZE);
    bldcCtrl->pBemf->maxChangePhaseTime = bldcCtrl->pBemf->commutationTime.maxValue;
    bldcCtrl->pBemf->minChangePhaseTime = bldcCtrl->pBemf->commutationTime.minValue;
}

/*!
* @brief Change phase to drive corresponding motor MOS and update afterflow time, average change phase time.
*
* @param[in] bldcVars: pointer to BLDC_VARS structure
* @return none
*/
void BldcBemfChangePhase(BLDC_VARS *bldcVars)
{
    MosDriver(bldcVars, bldcVars->pCtrl->pBemf->nextDriverVector);

    bldcVars->pCtrl->pBemf->currentDriverVector = bldcVars->pCtrl->pBemf->nextDriverVector;

    bldcVars->pCtrl->pBemf->lastChangePhaseTime = bldcVars->pCtrl->pBemf->saveChangePhaseTime;
    bldcVars->pCtrl->pBemf->saveChangePhaseTime = bldcVars->pCtrl->pBemf->changePhaseTime;
    bldcVars->pCtrl->pBemf->afterFlowTime = ((bldcVars->pCtrl->pBemf->saveChangePhaseTime + bldcVars->pCtrl->pBemf->lastChangePhaseTime)) >> 3;

    ChangePhaseTimeCalc(bldcVars->pCtrl);

    bldcVars->pCtrl->pBemf->changePhaseTime = 0;
    bldcVars->pCtrl->pBemf->overZero_Flag = 0;
    bldcVars->pCtrl->pBemf->afterFlow_Flag = 0;
    bldcVars->pCtrl->pBemf->forceChangePhaseFlag = 0;
}

/*!
* @brief update the motor1 next magnetic vector and change phase delay time, Prepare to change phase.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @return none
*/
void PrepareNextChangePhase_M1(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg, PROTECTOR_TYPE *protect)
{
    uint8_t curDriverValue1 = bldcCtrl->pBemf->currentDriverVector;
    if (bldcCfg->motorDir == FORWARD_ROTATE)
    {
        curDriverValue1++;
        if (curDriverValue1 >= (uint8_t)INVALID_VECTOR_MAX)
        {
            curDriverValue1 = (uint8_t)AH_PWM_BL_ON;
        }
    }
    else
    {
        curDriverValue1--;
        if (curDriverValue1 <= (uint8_t)INVALID_VECTOR_MIN)
        {
            curDriverValue1 = (uint8_t)CH_PWM_BL_ON;
        }
    }

    bldcCtrl->pBemf->nextDriverVector = curDriverValue1;

    if (protect->stallFlag)
    {
        bldcCtrl->pBemf->delayTime = 1;
    }
    else
    {
        bldcCtrl->pBemf->delayTime = (uint32_t)(((30.0f - LEAD_ANGLE)/60.0f) * bldcCtrl->pBemf->saveChangePhaseTime);
    }

    bldcCtrl->pBemf->overZero_Flag = 1;
}

/*!
* @brief update the motor2 next magnetic vector and change phase delay time, Prepare to change phase.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @return none
*/
void PrepareNextChangePhase_M2(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg, PROTECTOR_TYPE *protect)
{
    uint8_t curDriverValue2 = bldcCtrl->pBemf->currentDriverVector;
    if (bldcCfg->motorDir == FORWARD_ROTATE)
    {
        curDriverValue2++;
        if (curDriverValue2 >= (uint8_t)INVALID_VECTOR_MAX)
        {
            curDriverValue2 = (uint8_t)AH_PWM_BL_ON;
        }
    }
    else
    {
        curDriverValue2--;
        if (curDriverValue2 <= (uint8_t)INVALID_VECTOR_MIN)
        {
            curDriverValue2 = (uint8_t)CH_PWM_BL_ON;
        }
    }

    bldcCtrl->pBemf->nextDriverVector = curDriverValue2;

    if (protect->stallFlag)
    {
        bldcCtrl->pBemf->delayTime = 1;
    }
    else
    {
        bldcCtrl->pBemf->delayTime = (uint32_t)(((30.0f - LEAD_ANGLE_M2)/60.0f) * bldcCtrl->pBemf->saveChangePhaseTime);
    }

    bldcCtrl->pBemf->overZero_Flag = 1;
}

/*!
* @brief API to update the motor1 next magnetic vector and change phase delay time, Prepare to change phase.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @return none
*/
void PrepareNextChangePhase(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg, PROTECTOR_TYPE *protect)
{
    if (bldcCtrl == &g_bldcVarsCtrl)
    {
        PrepareNextChangePhase_M1(bldcCtrl, bldcCfg, protect);
    }
    else if (bldcCtrl == &g_bldcVarsCtrl_M2)
    {
         PrepareNextChangePhase_M2(bldcCtrl, bldcCfg, protect);
    }
    else
    {
        return;
    }
}

/*!
* @brief Check change phase interval to determine whether forced change phase is in needed.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @return none
*/
void ForceChangePhaseCheck(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg, PROTECTOR_TYPE *protect)
{
    if (bldcCtrl->pBemf->forceChangePhaseFlag == 0)
    {
        if (bldcCtrl->pBemf->changePhaseTime > bldcCfg->forceCommutateTime)
        {
            bldcCtrl->pBemf->forceChangePhaseFlag = 1;
            bldcCtrl->pBemf->forceChangePhaseNum++;
            bldcCtrl->pBemf->afterFlow_Flag = 1;

            if (bldcCtrl->pBemf->overZero_Flag == 0)
            {
                PrepareNextChangePhase(bldcCtrl, bldcCfg, protect);
            }
        }
    }
}

/*!
* @brief Force change phase interval while motor stall.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @return none
*/
void StallForceChangePhase(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg, PROTECTOR_TYPE *protect)
{
    uint16_t stallTempTime = 0;
    if ((bldcCtrl->pBemf->currentDriverVector % 2) == 0)
    {
        stallTempTime = 104;  /* 2.5ms = 104 * COMMUTATION_TIMER_PERIOD */
    }
    else
    {
        switch (bldcCtrl->pBemf->currentDriverVector)
        {
        case 1:
            stallTempTime = 625;  /* 15ms = 625 * COMMUTATION_TIMER_PERIOD */
            break;

        case 3:
            stallTempTime = 480;  /* 11.5ms = 480 * COMMUTATION_TIMER_PERIOD */
            break;

        case 5:
            stallTempTime = 354;  /* 8.5ms = 354 * COMMUTATION_TIMER_PERIOD */
            break;

        default:
            break;
        }
    }

    if (bldcCtrl->pBemf->forceChangePhaseFlag == 0)
    {
        if (bldcCtrl->pBemf->changePhaseTime > stallTempTime)
        {
            bldcCtrl->pBemf->forceChangePhaseFlag = 1;
            bldcCtrl->pBemf->forceChangePhaseNum++;
            bldcCtrl->pBemf->afterFlow_Flag = 1;

            if (bldcCtrl->pBemf->overZero_Flag == 0)
            {
                PrepareNextChangePhase(bldcCtrl, bldcCfg, protect);
            }
        }
    }
}

/*!
* @brief Check motor1 black EMF over zero point to determine when to change phase.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @return none
*/
void BemfOverZeroCheck_M1(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg, PROTECTOR_TYPE *protect)
{
    if (bldcCfg->motorDir == FORWARD_ROTATE)
    {
        switch (bldcCtrl->pBemf->currentDriverVector)
        {
        case AH_PWM_BL_ON:
            /* Phase C is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(2U)) == 0)
            {
                PrepareNextChangePhase_M1(bldcCtrl, bldcCfg, protect);
            }
            break;

        case AH_PWM_CL_ON:
            /* Phase B is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(1U)) != 0)
            {
                PrepareNextChangePhase_M1(bldcCtrl, bldcCfg, protect);
            }
            break;

        case BH_PWM_CL_ON:
            /* Phase A is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(0U)) == 0)
            {
                PrepareNextChangePhase_M1(bldcCtrl, bldcCfg, protect);
            }
            break;

        case BH_PWM_AL_ON:
            /* Phase C is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(2U)) != 0)
            {
                PrepareNextChangePhase_M1(bldcCtrl, bldcCfg, protect);
            }
            break;

        case CH_PWM_AL_ON:
            /* Phase B is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(1U)) == 0)
            {
                PrepareNextChangePhase_M1(bldcCtrl, bldcCfg, protect);
            }
            break;

        case CH_PWM_BL_ON:
            /* Phase A is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(0U)) != 0)
            {
                PrepareNextChangePhase_M1(bldcCtrl, bldcCfg, protect);
            }
            break;

        default:
            break;
        }
    }
    else
    {
        switch (bldcCtrl->pBemf->currentDriverVector)
        {
        case AH_PWM_BL_ON:
            /* Phase C is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(2U)) != 0)
            {
                PrepareNextChangePhase_M1(bldcCtrl, bldcCfg, protect);
            }
            break;

        case AH_PWM_CL_ON:
            /* Phase B is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(1U)) == 0)
            {
                PrepareNextChangePhase_M1(bldcCtrl, bldcCfg, protect);
            }
            break;

        case BH_PWM_CL_ON:
            /* Phase A is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(0U)) != 0)
            {
                PrepareNextChangePhase_M1(bldcCtrl, bldcCfg, protect);
            }
            break;

        case BH_PWM_AL_ON:
            /* Phase C is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(2U)) == 0)
            {
                PrepareNextChangePhase_M1(bldcCtrl, bldcCfg, protect);
            }
            break;

        case CH_PWM_AL_ON:
            /* Phase B is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(1U)) != 0)
            {
                PrepareNextChangePhase_M1(bldcCtrl, bldcCfg, protect);
            }
            break;

        case CH_PWM_BL_ON:
            /* Phase A is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(0U)) == 0)
            {
                PrepareNextChangePhase_M1(bldcCtrl, bldcCfg, protect);
            }
            break;

        default:
            break;
        }
    }
}

/*!
* @brief Check motor2 black EMF over zero point to determine when to change phase.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @return none
*/
void BemfOverZeroCheck_M2(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg, PROTECTOR_TYPE *protect)
{
    if (bldcCfg->motorDir == FORWARD_ROTATE)
    {
        switch (bldcCtrl->pBemf->currentDriverVector)
        {
        case AH_PWM_BL_ON:
            /* Phase C is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(14U)) == 0)
            {
                PrepareNextChangePhase_M2(bldcCtrl, bldcCfg, protect);
            }
            break;

        case AH_PWM_CL_ON:
            /* Phase B is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOB) & GPIO_PIDR_Msk(3U)) != 0)
            {
                PrepareNextChangePhase_M2(bldcCtrl, bldcCfg, protect);
            }
            break;

        case BH_PWM_CL_ON:
            /* Phase A is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOB) & GPIO_PIDR_Msk(2U)) == 0)
            {
                PrepareNextChangePhase_M2(bldcCtrl, bldcCfg, protect);
            }
            break;

        case BH_PWM_AL_ON:
            /* Phase C is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(14U)) != 0)
            {
                PrepareNextChangePhase_M2(bldcCtrl, bldcCfg, protect);
            }
            break;

        case CH_PWM_AL_ON:
            /* Phase B is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOB) & GPIO_PIDR_Msk(3U)) == 0)
            {
                PrepareNextChangePhase_M2(bldcCtrl, bldcCfg, protect);
            }
            break;

        case CH_PWM_BL_ON:
            /* Phase A is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOB) & GPIO_PIDR_Msk(2U)) != 0)
            {
                PrepareNextChangePhase_M2(bldcCtrl, bldcCfg, protect);
            }
            break;

        default:
            break;
        }
    }
    else
    {
        switch (bldcCtrl->pBemf->currentDriverVector)
        {
        case AH_PWM_BL_ON:
            /* Phase C is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(14U)) != 0)
            {
                PrepareNextChangePhase_M2(bldcCtrl, bldcCfg, protect);
            }
            break;

        case AH_PWM_CL_ON:
            /* Phase B is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOB) & GPIO_PIDR_Msk(3U)) == 0)
            {
                PrepareNextChangePhase_M2(bldcCtrl, bldcCfg, protect);
            }
            break;

        case BH_PWM_CL_ON:
            /* Phase A is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOB) & GPIO_PIDR_Msk(2U)) != 0)
            {
                PrepareNextChangePhase_M2(bldcCtrl, bldcCfg, protect);
            }
            break;

        case BH_PWM_AL_ON:
            /* Phase C is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(14U)) == 0)
            {
                PrepareNextChangePhase_M2(bldcCtrl, bldcCfg, protect);
            }
            break;

        case CH_PWM_AL_ON:
            /* Phase B is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOB) & GPIO_PIDR_Msk(3U)) != 0)
            {
                PrepareNextChangePhase_M2(bldcCtrl, bldcCfg, protect);
            }
            break;

        case CH_PWM_BL_ON:
            /* Phase A is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOB) & GPIO_PIDR_Msk(2U)) == 0)
            {
                PrepareNextChangePhase_M2(bldcCtrl, bldcCfg, protect);
            }
            break;

        default:
            break;
        }
    }
}

/*!
* @brief API to check corresponding motor black EMF over zero point to determine when to change phase.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @return none
*/
void BemfOverZeroCheck(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg, PROTECTOR_TYPE *protect)
{
    if (bldcCtrl == &g_bldcVarsCtrl)
    {
        BemfOverZeroCheck_M1(bldcCtrl, bldcCfg, protect);
    }
    else if (bldcCtrl == &g_bldcVarsCtrl_M2)
    {
        BemfOverZeroCheck_M2(bldcCtrl, bldcCfg, protect);
    }
    else
    {
        return;
    }
}

/*!
* @brief Correct motor1 bemf over zero point time when detect over zero error happened.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @return none
*/
void OverZeroErrorCorrect_M1(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg)
{
    if (bldcCfg->motorDir == FORWARD_ROTATE)
    {
        switch (bldcCtrl->pBemf->currentDriverVector)
        {
        case AH_PWM_BL_ON:
            /* Phase C is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(2U)) != 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case AH_PWM_CL_ON:
            /* Phase B is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(1U)) == 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case BH_PWM_CL_ON:
            /* Phase A is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(0U)) != 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case BH_PWM_AL_ON:
            /* Phase C is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(2U)) == 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case CH_PWM_AL_ON:
            /* Phase B is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(1U)) != 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case CH_PWM_BL_ON:
            /* Phase A is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(0U)) == 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        default:
            break;
        }
    }
    else
    {
        switch (bldcCtrl->pBemf->currentDriverVector)
        {
        case AH_PWM_BL_ON:
            /* Phase C is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(2U)) == 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case AH_PWM_CL_ON:
            /* Phase B is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(1U)) != 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case BH_PWM_CL_ON:
            /* Phase A is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(0U)) == 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case BH_PWM_AL_ON:
            /* Phase C is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(2U)) != 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case CH_PWM_AL_ON:
            /* Phase B is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(1U)) == 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case CH_PWM_BL_ON:
            /* Phase A is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(0U)) != 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        default:
            break;
        }
    }
}

/*!
* @brief Correct motor2 bemf over zero point time when detect over zero error happened.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @return none
*/
void OverZeroErrorCorrect_M2(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg)
{
    if (bldcCfg->motorDir == FORWARD_ROTATE)
    {
        switch (bldcCtrl->pBemf->currentDriverVector)
        {
        case AH_PWM_BL_ON:
            /* Phase C is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(14U)) != 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case AH_PWM_CL_ON:
            /* Phase B is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOB) & GPIO_PIDR_Msk(3U)) == 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case BH_PWM_CL_ON:
            /* Phase A is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOB) & GPIO_PIDR_Msk(2U)) != 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case BH_PWM_AL_ON:
            /* Phase C is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(14U)) == 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case CH_PWM_AL_ON:
            /* Phase B is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOB) & GPIO_PIDR_Msk(3U)) != 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case CH_PWM_BL_ON:
            /* Phase A is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOB) & GPIO_PIDR_Msk(2U)) == 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        default:
            break;
        }
    }
    else
    {
        switch (bldcCtrl->pBemf->currentDriverVector)
        {
        case AH_PWM_BL_ON:
            /* Phase C is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(14U)) == 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case AH_PWM_CL_ON:
            /* Phase B is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOB) & GPIO_PIDR_Msk(3U)) != 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case BH_PWM_CL_ON:
            /* Phase A is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOB) & GPIO_PIDR_Msk(2U)) == 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case BH_PWM_AL_ON:
            /* Phase C is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOC) & GPIO_PIDR_Msk(14U)) != 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case CH_PWM_AL_ON:
            /* Phase B is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOB) & GPIO_PIDR_Msk(3U)) == 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        case CH_PWM_BL_ON:
            /* Phase A is the undriven phase */
            if ((GPIO_DRV_ReadPins(GPIOB) & GPIO_PIDR_Msk(2U)) != 0)
            {
                bldcCtrl->pBemf->delayTime++;
            }
            break;

        default:
            break;
        }
    }
}

/*!
* @brief API to correct corresponding motor bemf over zero point time when detect over zero error happened.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @return none
*/
void OverZeroErrorCorrect(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg)
{
    if (bldcCtrl == &g_bldcVarsCtrl)
    {
        OverZeroErrorCorrect_M1(bldcCtrl, bldcCfg);
    }
    else if (bldcCtrl == &g_bldcVarsCtrl_M2)
    {
        OverZeroErrorCorrect_M2(bldcCtrl, bldcCfg);
    }
    else
    {
        return;
    }
}

/*!
* @brief Bemf over zero point detect and change phase function.
*
* @param[in] bldcVars: pointer to BLDC_VARS structure
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @return none
*/
void BemfCheckFun(BLDC_VARS *bldcVars, PROTECTOR_TYPE *protect)
{
    if (protect->stallFlag)
    {
        StallForceChangePhase(bldcVars->pCtrl, bldcVars->pCfg, protect);
    }
    else
    {
        ForceChangePhaseCheck(bldcVars->pCtrl, bldcVars->pCfg, protect);
    }

    if (bldcVars->pCtrl->pBemf->afterFlow_Flag == 0)
    {
        if (bldcVars->pCtrl->pBemf->changePhaseTime > bldcVars->pCtrl->pBemf->afterFlowTime)
        {
            bldcVars->pCtrl->pBemf->afterFlow_Flag = 1;
        }
    }
    else
    {
        if (bldcVars->pCtrl->pBemf->overZero_Flag == 0)
        {
            if (!protect->stallFlag)
            {
                BemfOverZeroCheck(bldcVars->pCtrl, bldcVars->pCfg, protect);
            }
        }
        else
        {
            if (bldcVars->pCtrl->pBemf->delayTime > 0)
            {
                OverZeroErrorCorrect(bldcVars->pCtrl, bldcVars->pCfg);
                bldcVars->pCtrl->pBemf->delayTime--;
            }

            if ((bldcVars->pCtrl->pBemf->delayTime == 0) || (bldcVars->pCtrl->pBemf->forceChangePhaseFlag == 1))
            {
                DisableInterrupts;
                BldcBemfChangePhase(bldcVars);
                EnableInterrupts;
            }
        }
    }
}

/* =============================================  EOF  ============================================== */
