/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_startup.c
*
* @brief This file provides bldc sensorless startup functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "bldc_keys.h"
#include "bldc_pwm.h"
#include "bldc_startup.h"
#include "bldc_parameters_define.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Get hall status level while startup.
*
* @param[in] PWMx: PWM Module
* @return hallInitStatus
*/
uint8_t GetHallInitStatus(PWM_Type *PWMx)
{
    uint16_t tempStatus;
    uint8_t hallInitStatus;
    uint8_t tempA, tempB, tempC;

    tempStatus = PWM_DRV_GetAllChannelInputStatus(PWM_INDEX(PWMx)) >> 16;
    tempA = tempStatus & 0x1U;
    tempB = (tempStatus & 0x2U) >> 1;
    tempC = (tempStatus & 0x4U) >> 2;
    hallInitStatus = (tempA << 2) + (tempB << 1) + tempC;

    return (hallInitStatus);
}

/*!
* @brief StartUp function in bldc hall control.
*
* @param[in] bldcVars: pointer to BLDC_VARS structure
* @param[in] command: pointer to SPEED_RAMP_TYPE structure
* @return none
*/
void BLDC_Hall_Startup(BLDC_VARS *bldcVars, SPEED_RAMP_TYPE *command)
{
    uint8_t flowState = 0;

    if (Get_McStatus() == START)
    {
        flowState = GetHallInitStatus(PWM0);
        if (bldcVars->pCtrl->setDuty <= 0.25f)
        {
            bldcVars->pCtrl->setDuty += bldcVars->pCfg->startupStepDuty;
        }
        else
        {
            bldcVars->pCtrl->setDuty = 0.25f;
            bldcVars->pCtrl->startupStatus = STARTUP_FAIL;
        }
        UpDatePwmDuty(bldcVars, DUTY_PERIOD(bldcVars->pCtrl->setDuty));

        BLDC_Commutate_BaseSector(bldcVars->pCfg->motorDir, flowState);

        if ((bldcVars->pCfg->motorDir * command->speedFbkRpm) >= 60.0f)   /* Mechanicla frequeccy >1hz,startup successful */
        {
            bldcVars->pCtrl->startupStatus = STARTUP_READY;
        }

    }
    else
    {

    }
}

/*!
* @brief StartUp function in bldc2 hall control.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] command: pointer to SPEED_RAMP_TYPE structure
* @return none
*/
void BLDC_Hall_Startup_M2(BLDC_VARS *bldcVars, SPEED_RAMP_TYPE *command)
{
    uint8_t flowState = 0;

    if (Get_McStatus_M2() == START)
    {
        flowState = GetHallInitStatus(PWM1);
        if (bldcVars->pCtrl->setDuty <= 0.25f)
        {
            bldcVars->pCtrl->setDuty += bldcVars->pCfg->startupStepDuty;
        }
        else
        {
            bldcVars->pCtrl->setDuty = 0.25f;
            bldcVars->pCtrl->startupStatus = STARTUP_FAIL;
        }
        UpDatePwmDuty(bldcVars, DUTY_PERIOD(bldcVars->pCtrl->setDuty));

        BLDC_Commutate_BaseSector_M2(bldcVars->pCfg->motorDir, flowState);

        if ((bldcVars->pCfg->motorDir * command->speedFbkRpm) >= 60.0f)   /* Mechanicla frequeccy >1hz,startup successful */
        {
            bldcVars->pCtrl->startupStatus = STARTUP_READY;
        }

    }
    else
    {

    }
}

/*!
* @brief Pre-position function in bldc sensorless control.
*
* @param[in] bldcVars: pointer to BLDC_VARS structure
* @return none
*/
void BLDC_Preposition(BLDC_VARS *bldcVars)
 {
    BLDC_VARS_CTRL *bldcCtrl = bldcVars->pCtrl;
    BLDC_VARS_CFG *bldcCfg = bldcVars->pCfg;

    uint16_t delay = bldcCfg->bemfCfg->prepositionTimeMax;

    for (bldcCtrl->setDuty = bldcCfg->bemfCfg->prepositionDutyMin; bldcCtrl->setDuty < bldcCfg->bemfCfg->prepositionDutyMax; bldcCtrl->setDuty += bldcCfg->bemfCfg->prepositionDutyStep)
    {
        delay -= bldcCfg->bemfCfg->prepositionTimeStep;
        if (delay <= bldcCfg->bemfCfg->prepositionTimeMin)
        {
            delay = bldcCfg->bemfCfg->prepositionTimeMin;
        }

        UpDatePwmDuty(bldcVars, DUTY_PERIOD(bldcCtrl->setDuty));

        bldcCtrl->pBemf->currentDriverVector = (uint8_t)AH_PWM_BL_ON;
        MosDriver(bldcVars, bldcCtrl->pBemf->currentDriverVector);
        mdelay(delay);
    }
    bldcCtrl->pBemf->kickTimeThreshold = bldcCfg->bemfCfg->kickTimeMax;
 }

/*!
* @brief bldc openkick function in bldc sensorless control, it is usually executed after obtaining the initial rotor position.
*
* @param[in] bldcVars: pointer to BLDC_VARS structure
* @return none
*/
void BLDC_Bemf_OpenKick(BLDC_VARS *bldcVars)
{
    if (bldcVars->pCfg->motorDir != FORWARD_ROTATE)
    {
        bldcVars->pCtrl->pBemf->currentDriverVector--;
        if (bldcVars->pCtrl->pBemf->currentDriverVector <= (uint8_t)INVALID_VECTOR_MIN)
        {
            bldcVars->pCtrl->pBemf->currentDriverVector = (uint8_t)CH_PWM_BL_ON;
        }
        MosDriver(bldcVars, bldcVars->pCtrl->pBemf->currentDriverVector);
    }
    else
    {
        bldcVars->pCtrl->pBemf->currentDriverVector++;
        if (bldcVars->pCtrl->pBemf->currentDriverVector >= (uint8_t)INVALID_VECTOR_MAX)
        {
            bldcVars->pCtrl->pBemf->currentDriverVector = (uint8_t)AH_PWM_BL_ON;
        }
        MosDriver(bldcVars, bldcVars->pCtrl->pBemf->currentDriverVector);

    }

    if (bldcVars->pCtrl->setDuty <= bldcVars->pCfg->bemfCfg->kickDutyMax)
    {
        bldcVars->pCtrl->setDuty += bldcVars->pCfg->startupStepDuty;
    }
    else
    {
        bldcVars->pCtrl->setDuty = bldcVars->pCfg->bemfCfg->kickDutyMax;
    }
    UpDatePwmDuty(bldcVars, DUTY_PERIOD(bldcVars->pCtrl->setDuty));
}

/*!
* @brief drag motor dircet to RUN state, get into close loop control fast.
*
* @param[in] bldcVars: pointer to BLDC_VARS structure
* @return none
*/
void MotorDirectDrag(BLDC_VARS *bldcVars)
{
    UpDatePwmDuty(bldcVars, DUTY_PERIOD(bldcVars->pCfg->dragDuty));
    MosDriver(bldcVars, bldcVars->pCtrl->pBemf->currentDriverVector);

    bldcVars->pCtrl->pBemf->changePhaseTime = 0;
    bldcVars->pCtrl->pBemf->saveChangePhaseTime = bldcVars->pCfg->firstCommutateTime;
    bldcVars->pCtrl->pBemf->afterFlowTime = bldcVars->pCfg->firstDragTime;
    bldcVars->pCtrl->pBemf->forceChangePhaseFlag = 0;

    bldcVars->pCtrl->startupStatus = STARTUP_READY;
}

/*!
* @brief drag motor to RUN state through open kick.
*
* @param[in] bldcVars: pointer to BLDC_VARS structure
* @return none
*/
void MotorKickDrag(BLDC_VARS *bldcVars)
{
    if (bldcVars->pCtrl->pBemf->kickTimeThreshold > bldcVars->pCfg->bemfCfg->kickTimeMin)
    {
        bldcVars->pCtrl->bldcStartTime++;
        if (bldcVars->pCtrl->bldcStartTime > bldcVars->pCtrl->pBemf->kickTimeThreshold)
        {
            bldcVars->pCtrl->bldcStartTime = 0;
            if (bldcVars->pCtrl->pBemf->kickTimeThreshold > bldcVars->pCfg->bemfCfg->kickTimeMin)
            {
                bldcVars->pCtrl->pBemf->kickTimeThreshold -= bldcVars->pCfg->bemfCfg->kickTimeStep;
            }
            else
            {
                bldcVars->pCtrl->pBemf->kickTimeThreshold = bldcVars->pCfg->bemfCfg->kickTimeMin;
            }
            BLDC_Bemf_OpenKick(bldcVars);
        }
    }
    else
    {
        if ((bldcVars->pCtrl->pBemf->currentDriverVector + bldcVars->pCfg->motorDir) >= (uint8_t)INVALID_VECTOR_MAX)
        {
            bldcVars->pCtrl->pBemf->currentDriverVector = (uint8_t)AH_PWM_BL_ON;
        }
        else if ((bldcVars->pCtrl->pBemf->currentDriverVector + bldcVars->pCfg->motorDir) <= (uint8_t)INVALID_VECTOR_MIN)
        {
            bldcVars->pCtrl->pBemf->currentDriverVector = (uint8_t)CH_PWM_BL_ON;
        }
        else
        {
            bldcVars->pCtrl->pBemf->currentDriverVector += bldcVars->pCfg->motorDir;
        }

        MosDriver(bldcVars, bldcVars->pCtrl->pBemf->currentDriverVector);

        bldcVars->pCtrl->pBemf->changePhaseTime = 0;
        bldcVars->pCtrl->pBemf->saveChangePhaseTime = bldcVars->pCfg->firstCommutateTime;
        bldcVars->pCtrl->pBemf->afterFlowTime = bldcVars->pCfg->firstDragTime;
        bldcVars->pCtrl->pBemf->forceChangePhaseFlag = 0;

        bldcVars->pCtrl->startupStatus = STARTUP_READY;
    }
}

/*!
* @brief BLDC motor startup function, include bldc hall and bldc sensorless control.
*
* @param[in] bldcVars: pointer to BLDC_VARS structure
* @return none
*/
void BLDC_Startup(BLDC_VARS *bldcVars)
{
#if (defined BLDC_HALL)
    if (bldcVars->pCtrl->bldcStartTime == 0)
    {
        BLDC_Hall_Startup(bldcVars, &g_bldc_speedCmd);
    }
    if (bldcVars->pCtrl->bldcStartTime++ > 20)
    {
        bldcVars->pCtrl->bldcStartTime = 0;
    }
#elif (defined BLDC_SENSORLESS)
    if (bldcVars->pCtrl->pBemf->prepositonIndex < 3)   /* preposition */
    {
        bldcVars->pCtrl->pBemf->prepositonIndex++;
        BLDC_Preposition(bldcVars);
    }
    else        /* drag startup */
    {
        #if (defined NOKICK_STARTUP)
        MotorDirectDrag(bldcVars);
        #else
        MotorKickDrag(bldcVars);
        #endif
    }
#else
#endif
}

/*!
* @brief BLDC motor2 startup function, include bldc hall and bldc sensorless control.
*
* @param[in] bldcVars: pointer to BLDC_VARS structure
* @return none
*/
void BLDC_Startup_M2(BLDC_VARS *bldcVars)
{
#if (defined BLDC_HALL_M2)
    if (bldcVars->pCtrl->bldcStartTime == 0)
    {
        BLDC_Hall_Startup_M2(bldcVars, &g_bldc_speedCmd_M2);
    }
    if (bldcVars->pCtrl->bldcStartTime++ > 20)
    {
        bldcVars->pCtrl->bldcStartTime = 0;
    }
#elif (defined BLDC_SENSORLESS_M2)
    if (bldcVars->pCtrl->pBemf->prepositonIndex < 3)   /* preposition */
    {
        bldcVars->pCtrl->pBemf->prepositonIndex++;
        BLDC_Preposition(bldcVars);
    }
    else        /* drag startup */
    {
        #if (defined NOKICK_STARTUP_M2)
        MotorDirectDrag(bldcVars);
        #else
        MotorKickDrag(bldcVars);
        #endif
    }
#else
#endif
}

/* =============================================  EOF  ============================================== */
