/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __STARTUP_H__
#define __STARTUP_H__
/*!
* @file startup.h
*
* @brief This file provides sensorless foc open loop startup functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "param_init.h"
#include "speed_angle.h"
#include "foc_keys.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Open loop startup curve configuration parameters structure.
*/
typedef struct
{
    uint8_t curveType;

    uint8_t curveSegNum;           /*!< Segment number of open loop start curve */
    uint16_t curveTime1;            /*!< Time duration value of open loop start curve segment 1 */
    uint16_t curveTime2;            /*!< Time duration value of open loop start curve segment 2 */
    uint16_t curveTime3;            /*!< Time duration value of open loop start curve segment 3 */
    uint16_t curveTime4;            /*!< Time duration value of open loop start curve segment 4 */
    uint16_t curveTime5;            /*!< Time duration value of open loop start curve segment 5 */
    uint16_t synTime;               /*!< Synchronization times to reach openloop target speed */
    uint16_t alignTime;             /*!< Time of pole alignment */
    uint16_t alignTimeTwice;        /*!< Time of the second pole alignment */
    uint16_t slopeDuration1;        /*!< Alignment once time of current arrive to traget value in ms */
    uint16_t slopeDuration2;        /*!< Alignment twice time of current arrive to traget value in ms */

    float curStartOffset;         /*!< Open loop startup current setting value */
    float synSpd;                 /*!< Target Synchronization speed of open loop */
    float spdSmoothCur;           /*!< Speed loop integration initial value, used to smooth the speed switching process */
    float alignAngle;            /*!< Angle of pole alignment */
    float alignCur;              /*!< Motor current with pole alignment */
    float alignCurTwice;         /*!< Motor current at second pole alignment */
    float curRamp1Init;          /*!< Alignment once current add initial value */
    float curRamp2Init;          /*!< Alignment twice current add initial value */
    float curveFreqStart;        /*!< Frequency initial value of open loop start curve */
    float curveCurStart;         /*!< Current initial value of open loop start curve */
    float curveVoltStart;        /*!< Voltage initial value of open loop start curve */
    float curveFreqSet1;         /*!< Frequency set value of point 1 of open loop start curve */
    float curveFreqSet2;         /*!< Frequency set value of point 2 of open loop start curve */
    float curveFreqSet3;         /*!< Frequency set value of point 3 of open loop start curve */
    float curveFreqSet4;         /*!< Frequency set value of point 4 of open loop start curve */
    float curveFreqSet5;         /*!< Frequency set value of point 5 of open loop start curve */
    float curveCurSet1;          /*!< Current set value of point 1 of open loop start curve */
    float curveCurSet2;          /*!< Current set value of point 2 of open loop start curve */
    float curveCurSet3;          /*!< Current set value of point 3 of open loop start curve */
    float curveCurSet4;          /*!< Current set value of point 4 of open loop start curve */
    float curveCurSet5;          /*!< Current set value of point 5 of open loop start curve */
    float curveVoltSet1;         /*!< Voltage set value of point 1 of open loop start curve */
    float curveVoltSet2;         /*!< Voltage set value of point 2 of open loop start curve */
    float curveVoltSet3;         /*!< Voltage set value of point 3 of open loop start curve */
    float curveVoltSet4;         /*!< Voltage set value of point 4 of open loop start curve */
    float curveVoltSet5;         /*!< Voltage set value of point 5 of open loop start curve */
    float curStartSub;           /*!< Decrement calue of current calculation */
    float curveFreqStart1;       /*!< Frequency initial value of open loop start curve */
    float curveCurStart1;        /*!< Current initial value of open loop start curve */
    float speedDelayZone;         /*!< Close and open loop switch speed delay zone, need to be set less than MIN_SPEED_VALUE */
} START_UP_CFG;

/*!
* @brief Open loop startup curve control parameters structure.
*/
typedef struct
{
    uint8_t startUpFlag;           /*!< Open loop startup operation flag */
    uint8_t synFreReach;           /*!< Flag to reach the open loop synchronization frequency */
    uint8_t alignState;            /*!< State of the rotor alignment process */
    uint16_t alignCnt;
    uint16_t thetaCompTimes;
    uint16_t synTime;               /*!< Synchronization times to reach openloop target speed */
    uint16_t synCount;              /*!< The count to judging the openloop synchronization frequency is reached or not */
    uint16_t countStartUpST1;       /*!< Count value for open loop startup control */
    uint16_t countStartUpST1a;      /*!< Count value for open loop startup control */
    uint16_t countStartUpST2;       /*!< Count value for open loop startup control */
    uint16_t countStartUpST3;       /*!< Count value for open loop startup control */
    uint16_t countStartUpST4;       /*!< Count value for open loop startup control */
    uint16_t countStartUpST5;       /*!< Count value for open loop startup control */
    uint16_t startUpTimes;          /*!< Count value for open loop startup control */
    float elecThetaEst;          /*!< Estimation of electrical angle for open loop */
    float curStartSub;           /*!< Decrement calue of current calculation */
    float startUpElecTheta;      /*!< Electric angle of open loop control */
    float synSpd;                 /*!< Target Synchronization speed of open loop */
    float curStartOffset;         /*!< Open loop startup current setting value */
    float curAddSlope1;           /*!< Alignment once current add slope */
    float curAddSlope2;           /*!< Alignment twice current add slope */
} STARTUP_CTRL;

/*!
* @brief Open loop startup curve calculation unit time parameters structure.
*/
typedef struct
{
    START_UP_CFG *pStCfg;           /*!< Open loop configuration parameter structure */
    STARTUP_CTRL *pStCtrl;          /*!< Open loop control parameter structure */
} START_UP;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Clear startUp control counting parameter.
*
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void ClearStartUpParam(START_UP *startUp);

/*!
* @brief Motor startup control initialization.
*
* @param[in] startUp: pointer to START_UP structure
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] stc: pointer to STC_PARAM structure
* @return none
*/
void StartUp_Ctrl_Init(START_UP *startUp, STARTUP_CURVE *curve, STC_PARAM *stc);

/*!
* @brief Motor 2 startup control initialization.
*
* @param[in] startUp: pointer to START_UP structure
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] stc: pointer to STC_PARAM structure
* @return none
*/
void StartUp_Ctrl_Init_M2(START_UP *startUp, STARTUP_CURVE *curve, STC_PARAM *stc);

/*!
* @brief Tail wind and head wind processes.
         When the motor is in the HIGH_REVERSE state, it will enter the STOP state;
         when the motor is in the LOW_REVERSE state or LOW_FORWARD state, it will enter the START_BRAKE state;
         when the motor is in the STATIC state, it will enter the START state;
         when the motor is in the HIGH_FORWARD state, it will enter the RUN state;
*
* @param[in] startUpCtrl: pointer to STARTUP_CTRL structure
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] speedHandle: pointer to SPEED_RAMP_HANDLE structure
* @param[in] judgeHandle: pointer to INITSPEED_JUDGE structure
* @return status: Motor operation status
*/
MC_STATUS Tail_Head_Wind_Process(STARTUP_CTRL *startUpCtrl, FOC_VARS *focVars, SPEED_RAMP_HANDLE *speedHandle, INITSPEED_JUDGE *judgeHandle);

/*!
* @brief StartUp control task in foc process.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to START_UP structure
* @param[in] speed: pointer to ESTIMATED_SPEED structure
* @param[in] pll: pointer to PID_REGULATOR structure
* @param[in] asr: pointer to PID_REGULATOR structure
* @param[in] stCurve: pointer to STARTUP_CURVE structure
* @return none
*/
void StartUp_Ctrl_Foc(STC_PARAM *stc, START_UP *startUp, ESTIMATED_SPEED *speed, PID_REGULATOR *pll, PID_REGULATOR *asr, STARTUP_CURVE *stCurve);

/*!
* @brief StartUp control task in foc process of Motor 2.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to START_UP structure
* @param[in] speed: pointer to ESTIMATED_SPEED structure
* @param[in] pll: pointer to PID_REGULATOR structure
* @param[in] asr: pointer to PID_REGULATOR structure
* @param[in] stCurve: pointer to STARTUP_CURVE structure
* @return none
*/
void StartUp_Ctrl_Foc_M2(STC_PARAM *stc, START_UP *startUp, ESTIMATED_SPEED *speed, PID_REGULATOR *pll, PID_REGULATOR *asr, STARTUP_CURVE *stCurve);

/*!
* @brief StartUp control task in 1ms period.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to START_UP structure
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] asr: pointer to PID_REGULATOR structure
* @param[in] pidcfg: pointer to PID_CFG structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void StartUp_Ctrl_1ms(STC_PARAM *stc, START_UP *startUp, STARTUP_CURVE *curve, PID_REGULATOR *asr, PID_REGULATOR *acr, PID_CFG *pidcfg, SPEED_RAMP_HANDLE *speed);

/*!
* @brief StartUp control task in 1ms period of Motor 2.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to START_UP structure
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] asr: pointer to PID_REGULATOR structure
* @param[in] pidcfg: pointer to PID_CFG structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void StartUp_Ctrl_1ms_M2(STC_PARAM *stc, START_UP *startUp, STARTUP_CURVE *curve, PID_REGULATOR *asr, PID_REGULATOR *acr, PID_CFG *pidcfg, SPEED_RAMP_HANDLE *speed);

/*!
* @brief Pre_positioning motor to an expected angle.
*
* @param[in] focVarsCtrl: pointer to FOC_VARS structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void Angle_Prepositioning_Process(FOC_VARS *focVars, START_UP *startUp);

/*!
* @brief Pre_positioning motor 2 to an expected angle.
*
* @param[in] focVarsCtrl: pointer to FOC_VARS structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void Angle_Prepositioning_Process_M2(FOC_VARS *focVars, START_UP *startUp);

/*!
* @brief Angle alignment process to prepare for motor open loop operation.
*
* @param[in] focVarsCtrl: pointer to FOC_VARS structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void Angle_Alignment_Process(FOC_VARS *focVars, START_UP *startUp);

/*!
* @brief Angle alignment process to prepare for motor 2 open loop operation.
*
* @param[in] focVarsCtrl: pointer to FOC_VARS_CTRL structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void Angle_Alignment_Process_M2(FOC_VARS *focVars, START_UP *startUp);

/*!
* @brief Transfer the electrical angle of the motor during the startup phase.
*
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return startUpElecTheta: electric angle of startup process
*/
float Get_Sensorless_ElecAngle(STARTUP_CTRL *startUp);

/*!
* @brief StartUp control task in 1ms period of ACIM SLVC.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
//void Acim_Startup_1ms(FOC_VARS *focVars, START_UP *startUp);

/*!
* @brief Asr coefficients adaptive calculation.
*
* @param[in] none
* @return none
*/
//void Acim_AsrPiCoefUpdate(void);

/* ======================================  Functions define  ======================================== */

#endif /* __STARTUP_H__ */

/* =============================================  EOF  ============================================== */
