/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file encoder.c
*
* @brief This file contain mainly two parts, read pulse count captrued by PWM module and calculate,update the elec angle in foc control.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pwm_input.h"
#include "motorctrl_api.h"
#include "globalvar.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Get Motor1 encoder pulse count captured by PWM module.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @return none
*/
void Encoder_GetPulseCount(ENCODER_TYPE *encoderSensor)
{
    /* Get encoder paulse count */
    encoderSensor->encoderCnt = PWM0->CNT & 0xFFFF;
}

/*!
* @brief Get Motor2 encoder pulse count captured by PWM module.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @return none
*/
void Encoder_GetPulseCount_M2(ENCODER_TYPE *encoderSensor)
{
    /* Get encoder paulse count */
    encoderSensor->encoderCnt = PWM1->CNT & 0xFFFF;
}

/*!
* @brief Obtain Motor1 encoder pulse count offset after encoder ALIGN.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @param[in] stc: pointer to STC_PARAM structure.
* @return none
*/
void Encoder_GetAlignOffset(ENCODER_TYPE *encoderSensor, STC_PARAM *stc)
{
#if (defined MOTOR_ONE_TYPE)
    /* Get encoder paulse count */
    Encoder_GetPulseCount(encoderSensor);
    /* Get align point paulse count offset.
    actual align position can be expressed as: (ALIGNMENT_ANGLE_DEGREE + 90 * (2 - pFocVarsCfg->motorDir)).
    take default 270 degree as example: when motorDir is FORWARD_ROTATE(1), actual align position is 0 degree.
    when motorDir is REVERSE_ROTATE(-1), actual align position is 180 degree */
    encoderSensor->alignOffSet = (int16_t)(encoderSensor->encoderCnt - (stc->pMotor->encoderLines / stc->pMotor->polePairs)
                                  * (ALIGNMENT_ANGLE_DEGREE + 90 * (2 - stc->pFOCVars->pCfg->motorDir)) / 360.0);
    encoderSensor->encoderCntOffSet = encoderSensor->alignOffSet;
#endif  /* end of MOTOR_ONE_TYPE */
}

/*!
* @brief Obtain Motor2 encoder pulse count offset after encoder ALIGN.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @param[in] stc: pointer to STC_PARAM structure.
* @return none
*/
void Encoder_GetAlignOffset_M2(ENCODER_TYPE *encoderSensor, STC_PARAM *stc)
{
#if (defined MOTOR_TWO_TYPE)
    /* Get encoder paulse count */
    Encoder_GetPulseCount_M2(encoderSensor);
    /* Get align point paulse count offset.
    actual align position can be expressed as: (ALIGNMENT_ANGLE_DEGREE + 90 * (2 - pFocVarsCfg->motorDir)).
    take default 270 degree as example: when motorDir is FORWARD_ROTATE(1), actual align position is 0 degree.
    when motorDir is REVERSE_ROTATE(-1), actual align position is 180 degree */
    encoderSensor->alignOffSet = (int16_t)(encoderSensor->encoderCnt - (stc->pMotor->encoderLines / stc->pMotor->polePairs)
                                  * (ALIGNMENT_ANGLE_DEGREE_M2 + 90 * (2 - stc->pFOCVars->pCfg->motorDir)) / 360.0);
    encoderSensor->encoderCntOffSet = encoderSensor->alignOffSet;
#endif  /* end of MOTOR_TWO_TYPE */
}

/*!
* @brief Statistic motor turn circles according to PWM module overflow direction.
*
* @param[in] encoderModule: PWM instance of motor encoder sensor.
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @return none
*/
void Encoder_MotorCircleCalc(uint8_t encoderModule, ENCODER_TYPE *encoderSensor)
{
    /* Get cross dir in overflow moment */
    encoderSensor->encoderDir = PWM_DRV_GetQuadOverflowDir(encoderModule);
    if (encoderSensor->encoderDir != 0)
    {
        /* 144-->0, encoder circle increase */
        encoderSensor->encoderCircleCnt++;
    }
    else
    {
        /* 0-->144, encoder circle ruduce */
        encoderSensor->encoderCircleCnt--;
    }
}

/*!
* @brief Calculate encoder elec angle and update in foc control current loop.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @param[in] motor: pointer to MOTOR_PARAM structure
* @return encoder elec angle
*/
uint16_t Encoder_ElecAngleCalc(ENCODER_TYPE *encoderSensor, MOTOR_PARAM *motor)
{
#if (defined MOTOR_ONE_TYPE)
    /* Get encoder paulse count */
    Encoder_GetPulseCount(encoderSensor);
    #if (!defined POSITION_LOOP_CTRL)
    encoderSensor->encoderCircleCnt = 0;
    #endif  /* end of POSITION_LOOP_CTRL */

    /* Get mechanical position encoder count, 2.5degree per count */
    encoderSensor->encoderAngleCnt = encoderSensor->encoderCnt - encoderSensor->encoderCntOffSet
                                     + motor->encoderLines * encoderSensor->encoderCircleCnt;
    /* Turn mechanical position to FOC angle in encoder count unit */
    encoderSensor->encoderFocCnt = (encoderSensor->encoderAngleCnt + motor->encoderLines
                                   * (1 - encoderSensor->encoderCircleCnt)) % motor->encoderLines;
    /* Calculate elec Angle In Degree(FOC) */
    encoderSensor->encoderElecAngle = (uint16_t)(encoderSensor->encoderFocCnt * (65536
                                                 * motor->polePairs / motor->encoderLines));
#endif  /* end of MOTOR_ONE_TYPE */

    return encoderSensor->encoderElecAngle;
}

/*!
* @brief Calculate encoder elec angle of Motor 2 and update in foc control current loop.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @param[in] motor: pointer to MOTOR_PARAM structure
* @return encoder elec angle
*/
uint16_t Encoder_ElecAngleCalc_M2(ENCODER_TYPE *encoderSensor, MOTOR_PARAM *motor)
{
#if (defined MOTOR_TWO_TYPE)
    /* Get encoder paulse count */
    Encoder_GetPulseCount_M2(encoderSensor);
     #if (!defined POSITION_LOOP_CTRL)
    encoderSensor->encoderCircleCnt = 0;
    #endif  /* end of POSITION_LOOP_CTRL */

    /* Get mechanical position encoder count, 2.5degree per count */
    encoderSensor->encoderAngleCnt = encoderSensor->encoderCnt - encoderSensor->encoderCntOffSet
                                     + motor->encoderLines * encoderSensor->encoderCircleCnt;
    /* Turn mechanical position to FOC angle in encoder count unit */
    encoderSensor->encoderFocCnt = (encoderSensor->encoderAngleCnt + motor->encoderLines
                                   * (1 - encoderSensor->encoderCircleCnt)) % motor->encoderLines;
    /* Calculate elec Angle In Degree(FOC) */
    encoderSensor->encoderElecAngle = (uint16_t)(encoderSensor->encoderFocCnt * (65536
                                                 * motor->polePairs / motor->encoderLines));
#endif  /* end of MOTOR_TWO_TYPE */

    return encoderSensor->encoderElecAngle;
}

/*!
* @brief Used to obtain electrical angle in encoder sensor FOC.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @return elec angle
*/
uint16_t Encoder_GetElecAngle(ENCODER_TYPE *encoderSensor)
{
    return encoderSensor->encoderElecAngle;
}

/*!
* @brief Encoder M_T measure method parameters initialization.
*
* @param[in] encoderMeasure: encoder speed measure variable, pointer to ENCODER_SPEED_MEASURE struct
* @return none
*/
void Encoder_MT_Init(ENCODER_SPEED_MEASURE *encoderMeasure)
{
#if (defined MOTOR_ONE_TYPE)
    memset(encoderMeasure, 0U, sizeof(*(encoderMeasure)));
    encoderMeasure->mtSpeedCoeff = (T_METHOD_HZ * POLE_PAIR_NUM) / PULSE_PER_REVOLUTION;
#endif  /* end of MOTOR_ONE_TYPE */
}

/*!
* @brief Encoder M_T measure method parameters initialization.
*
* @param[in] encoderMeasure: encoder speed measure variable, pointer to ENCODER_SPEED_MEASURE struct
* @return none
*/
void Encoder_MT_Init_M2(ENCODER_SPEED_MEASURE *encoderMeasure)
{
#if (defined MOTOR_TWO_TYPE)
    memset(encoderMeasure, 0U, sizeof(*(encoderMeasure)));
    encoderMeasure->mtSpeedCoeff = (T_METHOD_HZ * POLE_PAIR_NUM_M2) / PULSE_PER_REVOLUTION_M2;
#endif  /* end of MOTOR_TWO_TYPE */
}

/*!
* @brief Measure encoder pulse and time.
*
* @param[in] encoderMeasure: encoder speed measure variable, pointer to ENCODER_SPEED_MEASURE struct
* @return none
*/
void Encoder_MT_Measure(ENCODER_SPEED_MEASURE *encoderMeasure)
{
#if (defined MOTOR_ONE_TYPE)
    uint32_t temp_Mcvr;
    uint32_t temp_Cnt;
    int16_t temp_Pulse = PULSE_PER_REVOLUTION / POLE_PAIR_NUM;

    /* build high frequency pulse base count */
    if (++encoderMeasure->pulseTime > 60000)
    {
        encoderMeasure->pulseTime = 0;
    }

    /* encoder pulse edge alignment */
    temp_Cnt = PWM0->CNT;
    temp_Mcvr = PWM0->MCVR;
//    temp_Cnt = PWM_DRV_GetCounter(PWM_MODULER_ENCODER);

    if (((temp_Cnt % 4) == 1) && (encoderMeasure->speedUpdateFlag))
    {
        /* get and save M and T count */
        encoderMeasure->measurePulse1 = encoderMeasure->measurePulse2;
        encoderMeasure->measureTime1 = encoderMeasure->measureTime2;
        encoderMeasure->measurePulse2 = temp_Cnt & 0xFFFF;
        encoderMeasure->measureTime2 = encoderMeasure->pulseTime;
        /* get M and T delta count */
        if (encoderMeasure->measureTime2 < encoderMeasure->measureTime1)
        {
            encoderMeasure->deltaTime = encoderMeasure->measureTime2 + (60000 - encoderMeasure->measureTime1);
        }
        else
        {
            encoderMeasure->deltaTime = encoderMeasure->measureTime2 - encoderMeasure->measureTime1;
        }

        if ((encoderMeasure->measurePulse2 - encoderMeasure->measurePulse1) > (temp_Pulse))
        {
            encoderMeasure->deltaPulse = encoderMeasure->measurePulse2 - encoderMeasure->measurePulse1 - temp_Mcvr;
        }
        else if ((encoderMeasure->measurePulse2 - encoderMeasure->measurePulse1) < -(temp_Pulse))
        {
            encoderMeasure->deltaPulse = encoderMeasure->measurePulse2 - encoderMeasure->measurePulse1 + temp_Mcvr;
        }
        else
        {
            encoderMeasure->deltaPulse = encoderMeasure->measurePulse2 - encoderMeasure->measurePulse1;
        }
        encoderMeasure->speedUpdateFlag = 0;
    }
#endif  /* end of MOTOR_ONE_TYPE */
}

/*!
* @brief Measure Motor2 encoder pulse and time.
*
* @param[in] encoderMeasure: encoder speed measure variable, pointer to ENCODER_SPEED_MEASURE struct
* @return none
*/
void Encoder_MT_Measure_M2(ENCODER_SPEED_MEASURE *encoderMeasure)
{
#if (defined MOTOR_TWO_TYPE)
    uint32_t temp_Mcvr;
    uint32_t temp_Cnt;
    int16_t temp_Pulse = PULSE_PER_REVOLUTION_M2 / POLE_PAIR_NUM_M2;

    /* build high frequency pulse base count */
    if (++encoderMeasure->pulseTime > 60000)
    {
        encoderMeasure->pulseTime = 0;
    }

    /* encoder pulse edge alignment */
    temp_Cnt = PWM1->CNT;
    temp_Mcvr = PWM1->MCVR;
//    temp_Cnt = PWM_DRV_GetCounter(PWM_MODULER_ENCODER);

    if (((temp_Cnt % 4) == 1) && (encoderMeasure->speedUpdateFlag))
    {
        /* get and save M and T count */
        encoderMeasure->measurePulse1 = encoderMeasure->measurePulse2;
        encoderMeasure->measureTime1 = encoderMeasure->measureTime2;
        encoderMeasure->measurePulse2 = temp_Cnt & 0xFFFF;
        encoderMeasure->measureTime2 = encoderMeasure->pulseTime;
        /* get M and T delta count */
        if (encoderMeasure->measureTime2 < encoderMeasure->measureTime1)
        {
            encoderMeasure->deltaTime = encoderMeasure->measureTime2 + (60000 - encoderMeasure->measureTime1);
        }
        else
        {
            encoderMeasure->deltaTime = encoderMeasure->measureTime2 - encoderMeasure->measureTime1;
        }

        if ((encoderMeasure->measurePulse2 - encoderMeasure->measurePulse1) > (temp_Pulse))
        {
            encoderMeasure->deltaPulse = encoderMeasure->measurePulse2 - encoderMeasure->measurePulse1 - temp_Mcvr;
        }
        else if ((encoderMeasure->measurePulse2 - encoderMeasure->measurePulse1) < -(temp_Pulse))
        {
            encoderMeasure->deltaPulse = encoderMeasure->measurePulse2 - encoderMeasure->measurePulse1 + temp_Mcvr;
        }
        else
        {
            encoderMeasure->deltaPulse = encoderMeasure->measurePulse2 - encoderMeasure->measurePulse1;
        }
        encoderMeasure->speedUpdateFlag = 0;
    }
#endif  /* end of MOTOR_TWO_TYPE */
}

/*!
* @brief Calculate encoder pulse elec frequency by Measured encoder pulse and time.
*
* @param[in] encoderMeasure: encoder speed measure variable, pointer to ENCODER_SPEED_MEASURE struct
* @return none
*/
void Encoder_MT_SpeedCalc(ENCODER_SPEED_MEASURE *encoderMeasure)
{
#if (defined MOTOR_ONE_TYPE)
    if (encoderMeasure->speedUpdateFlag == 0)
    {
        /* use M and T calculate M_T speed */
        if (encoderMeasure->deltaTime != 0)
        {
            encoderMeasure->measureFreq = (encoderMeasure->deltaPulse * encoderMeasure->mtSpeedCoeff) / encoderMeasure->deltaTime;
        }
        encoderMeasure->mtSpeed = Math_IQ(encoderMeasure->measureFreq * ONE_BASE_FREQ);
        encoderMeasure->mtSpeedFilter = (((int32_t)encoderMeasure->mtSpeedFilter) * 7U + ((int32_t)encoderMeasure->mtSpeed) * 1U) >> 3U;
        encoderMeasure->speedUpdateFlag = 1;
    }
#endif  /* end of MOTOR_ONE_TYPE */
}

/*!
* @brief Calculate encoder pulse elec frequency of Motor 2 by Measured encoder pulse and time.
*
* @param[in] encoderMeasure: encoder speed measure variable, pointer to ENCODER_SPEED_MEASURE struct
* @return none
*/
void Encoder_MT_SpeedCalc_M2(ENCODER_SPEED_MEASURE *encoderMeasure)
{
#if (defined MOTOR_TWO_TYPE)
    if (encoderMeasure->speedUpdateFlag == 0)
    {
        /* use M and T calculate M_T speed */
        if (encoderMeasure->deltaTime != 0)
        {
            encoderMeasure->measureFreq = (encoderMeasure->deltaPulse * encoderMeasure->mtSpeedCoeff) / encoderMeasure->deltaTime;
        }
        encoderMeasure->mtSpeed = Math_IQ(encoderMeasure->measureFreq * ONE_BASE_FREQ_M2);
        encoderMeasure->mtSpeedFilter = (((int32_t)encoderMeasure->mtSpeedFilter) * 7U + ((int32_t)encoderMeasure->mtSpeed) * 1U) >> 3U;
        encoderMeasure->speedUpdateFlag = 1;
    }
#endif  /* end of MOTOR_TWO_TYPE */
}

/* =============================================  EOF  ============================================== */
