/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file foc_isr.c
*
* @brief This file provides FOC control interrupt callback functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "gpio_drv.h"
#include "adc_drv.h"
#include "pwmdrv.h"
#include "timer_drv.h"
#include "foc_isr.h"
#include "motor_debug.h"
#include "globalvar.h"
#include "vf_control.h"
#include "task_scheduler.h"
#include "motorctrl_api.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief PWMx Fault Test CallBack Function. When hardware over-current fault occurs, PWM fault interrupt is triggered and this function is called.
*
* @param[in] instance: device instance
* @param[in] status: fault status
* @param[in] userData: user data parameter
* @return none
*/
void FOC_PWM_FaultCallback(uint8_t instance, uint32_t status, void *userData)
{
#if (defined MOTOR_ONE_TYPE)
    if (status & PWM_FDSR_FAULTDF_Msk)
    {
        Set_McStatus(FAULT);
        g_protector.ErrFlag.bits.hwFault = 1;
        PWM_DRV_ClearFaultChannelFlag(instance, PWM_FAULT_CH_0);
    }
#endif
}

/*!
* @brief PWMx Fault Test CallBack Function. When hardware over-current fault occurs, PWM fault interrupt is triggered and this function is called.
*
* @param[in] instance: device instance
* @param[in] status: fault status
* @param[in] userData: user data parameter
* @return none
*/
void FOC_PWM_FaultCallback_M2(uint8_t instance, uint32_t status, void *userData)
{
#if (defined MOTOR_TWO_TYPE)
    if (status & PWM_FDSR_FAULTDF_Msk)
    {
        Set_McStatus_M2(FAULT);
        g_protector_M2.ErrFlag.bits.hwFault = 1;
        PWM_DRV_ClearFaultChannelFlag(instance, PWM_FAULT_CH_0);
    }
#endif
}

/*!
* @brief Callback routine of TIMER0. The interruption period is 1ms. The Task scheduler update in Timer interrupt.
*        When the Timer_0 counts to 1ms, the Timer_0 interrupt is triggered and this function is called.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void Timer0_Callback(void *device, uint32_t wpara, uint32_t lpara)
{
    if (wpara & TIMER_CTRL_SR_TIF0_Msk)
    {
        SCH_Update();
    }
}

/*!
* @brief Callback routine of TIMER1. The interruption period is M_T_PERIOD. The Task scheduler update in Timer interrupt.
*        When the Timer_1 counts to M_T_PERIOD, the Timer_1 interrupt is triggered and this function is called.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void Timer1_Callback(void *device, uint32_t wpara, uint32_t lpara)
{
#if (defined MOTOR_ONE_TYPE)
    if (wpara & TIMER_CTRL_SR_TIF1_Msk)
    {
        /* query the rising edge of pulse and counting */
        #if (defined MT_SPEED_MEASURE)
        Encoder_MT_Measure(&g_encoderSpeed);
        #endif
    }
#endif
}

/*!
* @brief Callback routine of TIMER2. The interruption period is M_T_PERIOD. The Task scheduler update in Timer interrupt.
*        When the Timer_2 counts to M_T_PERIOD, the Timer_2 interrupt is triggered and this function is called.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void Timer2_Callback(void *device, uint32_t wpara, uint32_t lpara)
{
#if (defined MOTOR_TWO_TYPE)
    if (wpara & TIMER_CTRL_SR_TIF2_Msk)
    {
        /* query the rising edge of pulse and counting */
        #if (defined MT_SPEED_MEASURE_M2)
        Encoder_MT_Measure_M2(&g_encoderSpeed_M2);
        #endif
    }
#endif
}

/*!
* @brief Call back function of PWM_Hall module used by hall sensor.
*        When PWM_Hall module Hall timer overflow will trriger Hall interrupt.
*
* @param[in] instance: device instance
* @param[in] status: fault status
* @param[in] userData: user data parameter
* @return none
*/
void FOC_Hall_OverflowCallback(uint8_t instance, uint32_t status, void *userData)
{
#if (defined MOTOR_ONE_TYPE)
    uint32_t hallStatus;
    #if (defined HALL_SENSOR_CHECK_ENALBE)
    static uint8_t cnt = 0;
    #endif

//    hallStatus = PWM_DRV_GetHallStatus(PWM_MODULE_HALL_MOTOR1);
    hallStatus = GetHallStatus(PWM_MODULE_HALL_MOTOR1);
    if (status & PWM_INIT_CNTOF_Msk)
    {
        if ((defHallSensor.hallSelfLearnEnable == DISABLE) && (Get_McStatus() != IDLE) \
            && (Get_McStatus() != POWER_ON) && (Get_McStatus() != STOP) && (Get_McStatus() != FAULT))
        {
            HallSensor_Init(hallStatus, &g_hall, &g_focVars);

            #if (defined HALL_SENSOR_CHECK_ENALBE)
            if ((g_hall.pHallSensor->pwdtDatValid <= 2U) \
                && (MathFloat_Abs(g_speedCommand.speedTargetRamp) >= HALL_DISCONNECT_FREQ_THRESHOLD))
            {
                if (cnt > HALL_DISCONNECT_JUDGE)
                {
                    Set_McStatus(FAULT);
                    cnt = 0;
                    g_protector.ErrFlag.bits.hallStatus = 1U;
                }
                else
                {
                    cnt++;
                }
            }
            #endif  /* end of HALL_SENSOR_CHECK_ENALBE */
        }
        else
        {
//            cnt = 0;
        }

    }
#endif
}

/*!
* @brief Call back function of PWM_Hall module used by hall sensor.
*        When PWM_Hall module Hall timer overflow will trriger Hall interrupt.
*
* @param[in] instance: device instance
* @param[in] status: fault status
* @param[in] userData: user data parameter
* @return none
*/
void FOC_Hall_OverflowCallback_M2(uint8_t instance, uint32_t status, void *userData)
{
#if (defined MOTOR_TWO_TYPE)

    uint32_t hallStatus;
    #if (defined HALL_SENSOR_CHECK_ENALBE)
    static uint8_t cnt1 = 0;
    #endif

//    hallStatus = PWM_DRV_GetHallStatus(PWM_MODULE_HALL_MOTOR2);
    hallStatus = GetHallStatus(PWM_MODULE_HALL_MOTOR2);

    if (status & PWM_INIT_CNTOF_Msk)
    {
        if ((defHallSensor_M2.hallSelfLearnEnable == DISABLE) && (Get_McStatus_M2() != IDLE) \
            && (Get_McStatus_M2() != POWER_ON) && (Get_McStatus_M2() != STOP) && (Get_McStatus_M2() != FAULT))
        {
            HallSensor_Init(hallStatus, &g_hall_M2, &g_focVars_M2);

            #if (defined HALL_SENSOR_CHECK_ENALBE)
            if ((g_hall_M2.pHallSensor->pwdtDatValid <= 2U) \
                && (MathFloat_Abs(g_speedCommand_M2.speedTargetRamp) >= HALL_DISCONNECT_FREQ_THRESHOLD))
            {
                if (cnt1 > HALL_DISCONNECT_JUDGE)
                {
                    Set_McStatus_M2(FAULT);
                    cnt1 = 0;
                    g_protector_M2.ErrFlag.bits.hallStatus = 1U;
                }
                else
                {
                    cnt1++;
                }
            }
            #endif  /* end of HALL_SENSOR_CHECK_ENALBE */
        }
        else
        {
//            cnt1 = 0;
        }
    }
#endif
}

/*!
* @brief Call back function of PWM_Hall module used by hall sensor.
*        When PWM_Hall module input signal level change will trriger Hall interrupt.
*
* @param[in] instance: device instance
* @param[in] status: fault status
* @param[in] userData: user data parameter
* @return none
*/
void FOC_Hall_CaptureCallback(uint8_t instance, uint32_t status, void *userData)
{
#if (defined MOTOR_ONE_TYPE)
    uint8_t hallStatus = 0;
    uint32_t posWidth = 0;
    uint32_t negWidth = 0;

    if ((status & PWM_STR_CH0SF_Msk) || (status & PWM_STR_CH1SF_Msk))
    {
        /* get HALL status */
        hallStatus = GetHallStatus(PWM_MODULE_HALL_MOTOR1);
        posWidth = PWM_DRV_GetChannelCountValue(PWM_MODULE_HALL_MOTOR1, PWM_CH_0);
        negWidth = PWM_DRV_GetChannelCountValue(PWM_MODULE_HALL_MOTOR1, PWM_CH_1);

        if (defHallSensor.hallSelfLearnEnable == DISABLE)
        {
            HallSensor_HallSubCallBack(hallStatus, negWidth, posWidth, &g_hall, pFocVarsCfg);

            #if (defined HALL_SENSOR_CHECK_ENALBE)
            /* Hall sensor disconnect protect */
            if (g_hall.pHallSensor->pwdtDatValid == 2U)
            {
                if (Hall_Sensor_Change_Check(hallStatus, g_hall.pHallSensor->hallStatus, &g_protector))
                {
                    HallSensor_Init(hallStatus, &g_hall, &g_focVars);
                    Set_McStatus(FAULT);
                }
            }
            #endif  /* end of HALL_SENSOR_CHECK_ENALBE */
        }
        else
        {
            HallSensor_SelfLearn_CaptureIsr(hallStatus, negWidth, posWidth, &g_hall);
        }
    }
#endif
}

/*!
* @brief Call back function of PWM_Hall module used by hall sensor.
*        When PWM_Hall module input signal level change will trriger Hall interrupt.
*
* @param[in] instance: device instance
* @param[in] status: fault status
* @param[in] userData: user data parameter
* @return none
*/
void FOC_Hall_CaptureCallback_M2(uint8_t instance, uint32_t status, void *userData)
{
#if (defined MOTOR_TWO_TYPE)
    uint8_t hallStatus = 0;
    uint32_t posWidth = 0;
    uint32_t negWidth = 0;

    if ((status & PWM_STR_CH0SF_Msk) || (status & PWM_STR_CH1SF_Msk))
    {
        /* get HALL status */
        hallStatus = GetHallStatus(PWM_MODULE_HALL_MOTOR2);
        posWidth = PWM_DRV_GetChannelCountValue(PWM_MODULE_HALL_MOTOR2, PWM_CH_0);
        negWidth = PWM_DRV_GetChannelCountValue(PWM_MODULE_HALL_MOTOR2, PWM_CH_1);

        if (defHallSensor_M2.hallSelfLearnEnable == DISABLE)
        {
            HallSensor_HallSubCallBack(hallStatus, negWidth, posWidth, &g_hall_M2, pFocVarsCfg_M2);

            #if (defined HALL_SENSOR_CHECK_ENALBE)
            /* Hall sensor disconnect protect */
            if (g_hall_M2.pHallSensor->pwdtDatValid == 2U)
            {
                if (Hall_Sensor_Change_Check(hallStatus, g_hall_M2.pHallSensor->hallStatus, &g_protector_M2))
                {
                    HallSensor_Init(hallStatus, &g_hall_M2, &g_focVars_M2);
                    Set_McStatus_M2(FAULT);
                }
            }
            #endif  /* end of HALL_SENSOR_CHECK_ENALBE */
        }
        else
        {
            HallSensor_SelfLearn_CaptureIsr(hallStatus, negWidth, posWidth, &g_hall_M2);
        }
    }
#endif
}

/*!
* @brief Get Hall sensor status.
*
* @param[in] pwmModule: PWM module to get Hall sector
* @return hallStatus
*/
uint8_t GetHallStatus(const uint8_t pwmModule)
{
    uint16_t tempStatus;
    uint8_t hallStatus;
    uint8_t tempA, tempB, tempC;

    tempStatus = PWM_DRV_GetAllChannelInputStatus(pwmModule) >> 16;
    tempA = tempStatus & 0x1U;
    tempB = (tempStatus & 0x2U) >> 1;
    tempC = (tempStatus & 0x4U) >> 2;
    hallStatus = (tempA << 2) + (tempB << 1) + tempC;

    return (hallStatus);
}

/*!
* @brief PWM count overflow interrupt to statistic circles for encoder position servo applicaton.
*
* @param[in] instance: device instance
* @param[in] status: fault status
* @param[in] userData: user data parameter
* @return none
*/
void FOC_Encoder_Callback(uint8_t instance, uint32_t status, void *userData)
{
#if (defined MOTOR_ONE_TYPE)
    if (status & PWM_INIT_CNTOF_Msk)
    {
        #if (defined POSITION_LOOP_CTRL)
        Encoder_MotorCircleCalc(PWM_MODULE_ENCODER_MOTOR1, &g_encoderSensor);
        #endif
    }
#endif
}

/*!
* @brief PWM count overflow interrupt to statistic circles for encoder position servo applicaton.
*
* @param[in] instance: device instance
* @param[in] status: fault status
* @param[in] userData: user data parameter
* @return none
*/
void FOC_Encoder_Callback_M2(uint8_t instance, uint32_t status, void *userData)
{
#if (defined MOTOR_TWO_TYPE)
    if (status & PWM_INIT_CNTOF_Msk)
    {
        #if (defined POSITION_LOOP_CTRL_M2)
        Encoder_MotorCircleCalc(PWM_MODULE_ENCODER_MOTOR2, &g_encoderSensor_M2);
        #endif
    }
#endif
}

/*!
* @brief ADC interrupt service subfunction for short brake.
*        When motor is short circuit braking, ADC interruption wiil enter this function.
*
* @param[in] pwmModule: PWM module instance of motor
* @param[in] sBrake: pointer to SHORT_BRAKE structure
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] adc: pointer to ADC_PARAM structure
* @return none
*/
void ShortBrake_AdcProcess(uint8_t pwmModule, SHORT_BRAKE *sBrake, STC_PARAM *stc, ADC_PARAM *adc)
{
    Get_CurrentSample(adc);
    AdcResultTransmit(adc->adTrans, stc->pFOCVars->pCtrl);
    ShortBrake_AdcSubCallBack(sBrake, stc);
}

/*!
* @brief Hall sensor self-learn process.
*
* @param[in] none
* @return none
*/
void HallSelfLearnProcess(void)
{
#if (defined MOTOR_ONE_TYPE)
    if (defHallSensor.hallSelfLearnMode == 1)
    {
        if (g_hallSelfLearn.selfLearnStage != SELF_LEARN_2DEGREE_FINISHED)
        {
            HallSensor_SelfLearn(GetHallStatus(PWM_MODULE_HALL_MOTOR1), &g_hall, &g_focVars);
            g_hallSelfLearnFinishFlag = 0;
        }
        else
        {
            g_hallSelfLearnFinishFlag = 1;
        }
    }
    else if (defHallSensor.hallSelfLearnMode == 2)
    {
        if (g_hallSelfLearn.selfLearnStage != SELF_LEARN_HALL_WIDTH)
        {
            HallSensor_SelfLearn(GetHallStatus(PWM_MODULE_HALL_MOTOR1), &g_hall, &g_focVars);
            g_hallSelfLearnFinishFlag = 0;
        }
        else
        {
            g_hallSelfLearnFinishFlag = 1;
        }
    }
    else
    {
    }
#endif
}

/*!
* @brief Hall sensor self-learn process.
*
* @param[in] none
* @return none
*/
void HallSelfLearnProcess_M2(void)
{
#if (defined MOTOR_TWO_TYPE)
    if (defHallSensor_M2.hallSelfLearnMode == 1)
    {
        if (g_hallSelfLearn_M2.selfLearnStage != SELF_LEARN_2DEGREE_FINISHED)
        {
            HallSensor_SelfLearn(GetHallStatus(PWM_MODULE_HALL_MOTOR2), &g_hall_M2, &g_focVars_M2);
            g_hallSelfLearnFinishFlag_M2 = 0;
        }
        else
        {
            g_hallSelfLearnFinishFlag_M2 = 1;
        }
    }
    else if (defHallSensor_M2.hallSelfLearnMode == 2)
    {
        if (g_hallSelfLearn_M2.selfLearnStage != SELF_LEARN_HALL_WIDTH)
        {
            HallSensor_SelfLearn(GetHallStatus(PWM_MODULE_HALL_MOTOR2), &g_hall_M2, &g_focVars_M2);
            g_hallSelfLearnFinishFlag_M2 = 0;
        }
        else
        {
            g_hallSelfLearnFinishFlag_M2 = 1;
        }
    }
    else
    {
    }
#endif
}

/*!
* @brief The first part of the FOC interrupt to obtain the electrical angle.
*
* @param[in] stc: pointer to STC_PARAM structure
* @return none
*/
void Foc_Execution_AngleProcess(STC_PARAM *stc)
{
#if (defined MOTOR_ONE_TYPE)
    if (Get_McStatus() != IDENTIFY)
    {
#if (defined FOC_PMSM)
    #if (defined ENCODER_HALL_SENSORS)
// NULL
    #elif (defined HALL_SENSORS)
//        if (defHallSensor.hallSelfLearnEnable == ENABLE)
        if (Get_McStatus() == HALL_SELFLEARN)
        {
            HallSelfLearnProcess();
        }
        else if ((Get_McStatus() == RUN) || (Get_McStatus() == SLOW_DOWN))
        {
//            HallSensor_AdcCallBack(PWM_DRV_GetHallStatus(PWM_MODULE_HALL_MOTOR1), &g_hall);
            HallSensor_AdcCallBack(GetHallStatus(PWM_MODULE_HALL_MOTOR1), &g_hall, pFocVarsCfg);
        }
        else
        {
        }
    #elif (defined ENCODER_SENSORS)
        Encoder_ElecAngleCalc(&g_encoderSensor, pMotorParam);
        #if ((!defined MT_SPEED_MEASURE) && (defined ELEC_FREQUENCE_HIGH))
        Asr_EstSpeedCalc(g_encoderSensor.encoderElecAngle, &g_rotationalSpeed);
        #endif  /* end of MT_SPEED_MEASURE */

    #elif (defined SENSORLESS)
        if ((Get_McStatus() == RUN) || (Get_McStatus() == SLOW_DOWN))
        {
            #if (defined HIGH_FREQUENCE_INJECTION)
            #elif (defined IF_STARTUP) || (defined DIRECT_STARTUP) || (defined VF_STARTUP)
            StartUp_Ctrl_Foc(&g_STC, &g_startUp, &g_rotationalSpeed, &g_pidPll, &g_pidAsr, &g_startUpCurve);
            #endif  /* end of HIGH_FREQUENCE_INJECTION */
        }
    #else
    #endif  /* end of ENCODER_HALL_SENSORS */

#elif (defined FOC_ACIM)
//    #if (defined SENSOR_ACIM)
//        Acim_SensorSpeedCalc(&g_acimFocCtrl, &g_STC, &g_encoderSensor, &g_rotationalSpeed);
//    #elif (defined SENSORLESS_ACIM)
//        if ((Get_McStatus() == RUN) || (Get_McStatus() == SLOW_DOWN) || (Get_McStatus() == ALIGN))
//        {
//            StartUp_Ctrl_Foc(&g_STC, &g_startUp, &g_rotationalSpeed, &g_pidPll, &g_pidAsr, &g_startUpCurve);
//        }
//    #else
//    #endif
#endif  /* end of FOC_PMSM */
    }
    else
    {
#if (defined FOC_PMSM)
        if ((PMSM_LDQ_METHOD_HFS == g_paramIdentify.pParamIdentifyCfg->pmsmLdqTypeSel) \
            && (IDENTIFY_LDQ_STALLTEST == g_paramIdentify.pParamIdentifyCtrl->mainStep))
        {
//            PmsmId_LdqCalcIsr(&g_paramIdentify, &g_STC, &g_sigPhaseShift);

            PMSMLdqId_IsrTask(&g_paramIdentify, &g_STC);
        }
#elif (defined FOC_ACIM)
//        if (IDENTIFY_RS_DCTEST != g_paramIdentify.pParamIdentifyCtrl->mainStep)
//        {
//            AcimId_IdentificationIsr(&g_paramIdentify, &g_sigPhaseShift, &g_STC);
//        }
#else
#endif  /* end of FOC_PMSM */
    }

    if ((Get_McStatus() == RUN) || (Get_McStatus() == SLOW_DOWN) || (Get_McStatus() == IDENTIFY) || (Get_McStatus() == HALL_SELFLEARN))
    {
        Get_ElecAngle(&g_focVarsCtrl);
    }

#if (defined TAIL_HEAD_WIND)
    if ((Get_McStatus() == SPEED_JUDGE) || (Get_McStatus() == FLY_RUN))
    {
        Speed_Judge_Process(&g_initSpeedJudge, &g_focVars);
        if (g_speedCommand.initStatus == HIGH_FORWARD)
        {
            g_pidPll.pPidOutput->uiPu = g_initSpeedJudge.initSpeed;
            #if (defined FLUX_OBSERVE)
            g_fluxObserverCtrl.estTheta = g_initSpeedJudge.bemfTheta;
            #elif (defined SMC_OBSERVE)
            g_smcEstimator.estTheta = g_initSpeedJudge.bemfTheta;
            #elif (defined MRAS_OBSERVE)
            g_mrasObserver.estFreqBuf = g_initSpeedJudge.initSpeed;
            g_mrasObserver.estTheta = g_initSpeedJudge.bemfTheta;
            #endif  /* end of FLUX_OBSERVE */
            ASR_SetSpeedTargetRamp_Pu(&g_speedCommand, g_speedCommand.initSpeedJudgeSpeed);
            ASR_SetSpeedTarget_Pu(&g_speedCommand, g_speedCommand.initSpeedJudgeSpeed);
            Set_McStatus(RUN);
            StartUp_Motor(PWM_MODULE_MOTOR1, &g_startUpCtrl, &g_focVarsCtrl);
            g_startUpCtrl.startUpFlag = 4;
        }
    }
    else
    {
    }
#endif  /* end of TAIL_HEAD_WIND */

#endif  /* end of MOTOR_ONE_TYPE */
}

/*!
* @brief The first part of the FOC interrupt to obtain the electrical angle.
*
* @param[in] stc: pointer to STC_PARAM structure
* @return none
*/
void Foc_Execution_AngleProcess_M2(STC_PARAM *stc)
{
#if (defined MOTOR_TWO_TYPE)
    if (Get_McStatus_M2() != IDENTIFY)
    {
#if (defined FOC_PMSM_M2)
    #if (defined ENCODER_HALL_SENSORS_M2)
// NULL
    #elif (defined HALL_SENSORS_M2)
        if (Get_McStatus_M2() == HALL_SELFLEARN)
        {
            HallSelfLearnProcess_M2();
        }
        else if ((Get_McStatus_M2() == RUN) || (Get_McStatus_M2() == SLOW_DOWN))
        {
            HallSensor_AdcCallBack(GetHallStatus(PWM_MODULE_HALL_MOTOR2), &g_hall_M2, pFocVarsCfg_M2);
        }
        else
        {
        }
    #elif (defined ENCODER_SENSORS_M2)
        Encoder_ElecAngleCalc_M2(&g_encoderSensor_M2, pMotorParam_M2);
        #if (defined ELEC_FREQUENCE_HIGH_M2)
        Asr_EstSpeedCalc(g_encoderSensor_M2.encoderElecAngle, &g_rotationalSpeed_M2);
        #endif  /* end of ELEC_FREQUENCE_HIGH_M2 */

    #elif (defined SENSORLESS_M2)
        if ((Get_McStatus_M2() == RUN) || (Get_McStatus_M2() == SLOW_DOWN))
        {
            #if (defined HIGH_FREQUENCE_INJECTION_M2)
            #elif (defined IF_STARTUP_M2) || (defined DIRECT_STARTUP_M2) || (defined VF_STARTUP_M2)
            StartUp_Ctrl_Foc_M2(&g_STC_M2, &g_startUp_M2, &g_rotationalSpeed_M2, &g_pidPll_M2, &g_pidAsr_M2, &g_startUpCurve_M2);
            #endif  /* end of HIGH_FREQUENCE_INJECTION_M2 */
        }
    #else
    #endif  /* end of ENCODER_HALL_SENSORS_M2 */

#elif (defined FOC_ACIM_M2)
//    #if (defined SENSOR_ACIM_M2)
//        Acim_SensorSpeedCalc(&g_acimFocCtrl, &g_STC, &g_encoderSensor, &g_rotationalSpeed);
//    #elif (defined SENSORLESS_ACIM_M2)
//        if ((Get_McStatus_M2() == RUN) || (Get_McStatus_M2() == SLOW_DOWN) || (Get_McStatus_M2() == ALIGN))
//        {
//            StartUp_Ctrl_Foc_M2(&g_STC, &g_startUp, &g_rotationalSpeed, &g_pidPll, &g_pidAsr, &g_startUpCurve);
//        }
//    #else
//    #endif
#endif  /* end of FOC_PMSM_M2 */
    }
    else
    {
#if (defined FOC_PMSM_M2)
        if ((PMSM_LDQ_METHOD_HFS == g_paramIdentify_M2.pParamIdentifyCfg->pmsmLdqTypeSel) \
            && (IDENTIFY_LDQ_STALLTEST == g_paramIdentify_M2.pParamIdentifyCtrl->mainStep))
        {
//            PmsmId_LdqCalcIsr(&g_paramIdentify, &g_STC, &g_sigPhaseShift);

            PMSMLdqId_IsrTask(&g_paramIdentify_M2, &g_STC_M2);
        }
#elif (defined FOC_ACIM_M2)
//        if (IDENTIFY_RS_DCTEST != g_paramIdentify.pParamIdentifyCtrl->mainStep)
//        {
//            AcimId_IdentificationIsr(&g_paramIdentify, &g_sigPhaseShift, &g_STC);
//        }
#else
#endif  /* end of FOC_PMSM_M2 */
    }

    if ((Get_McStatus_M2() == RUN) || (Get_McStatus_M2() == SLOW_DOWN) || (Get_McStatus_M2() == IDENTIFY) || (Get_McStatus_M2() == HALL_SELFLEARN))
    {
        Get_ElecAngle_M2(&g_focVarsCtrl_M2);
    }

#if (defined TAIL_HEAD_WIND_M2)
    if ((Get_McStatus_M2() == SPEED_JUDGE) || (Get_McStatus_M2() == FLY_RUN))
    {
        Speed_Judge_Process_M2(&g_initSpeedJudge_M2, &g_focVars_M2);
        if (g_speedCommand_M2.initStatus == HIGH_FORWARD)
        {
            g_pidPll_M2.pPidOutput->uiPu = g_initSpeedJudge_M2.initSpeed;
            #if (defined FLUX_OBSERVE_M2)
            g_fluxObserverCtrl_M2.estTheta = g_initSpeedJudge_M2.bemfTheta;
            #elif (defined SMC_OBSERVE_M2)
            g_smcEstimator_M2.estTheta = g_initSpeedJudge_M2.bemfTheta;
            #elif (defined MRAS_OBSERVE_M2)
            g_mrasObserver_M2.estFreqBuf = g_initSpeedJudge_M2.initSpeed;
            g_mrasObserver_M2.estTheta = g_initSpeedJudge_M2.bemfTheta;
            #endif  /* end of FLUX_OBSERVE_M2 */
            ASR_SetSpeedTargetRamp_Pu(&g_speedCommand_M2, g_speedCommand_M2.initSpeedJudgeSpeed);
            ASR_SetSpeedTarget_Pu(&g_speedCommand_M2, g_speedCommand_M2.initSpeedJudgeSpeed);
            Set_McStatus_M2(RUN);
            StartUp_Motor(PWM_MODULE_MOTOR2, &g_startUpCtrl_M2, &g_focVarsCtrl_M2);
            g_startUpCtrl_M2.startUpFlag = 4;
        }
    }
    else
    {
    }
#endif  /* end of TAIL_HEAD_WIND_M2 */

#endif  /* end of MOTOR_TWO_TYPE */
}

/*!
* @brief The second part of the FOC interrupt to execute current loop calculation and generate SVPWM modulation wave.
*
* @param[in] focVarsCtrl: pointer to FOC_VARS_CTRL structure
* @return none
*/
void Foc_Execution_CurrentLoopProcess(FOC_VARS_CTRL *focVarsCtrl)
{
#if (defined MOTOR_ONE_TYPE)
#if (defined FOC_SINE_WAVE)
    #if (defined FOC_PMSM)
//    #if (defined SENSORLESS) && (defined VOLTAGE_RESTRUCT)
    #if (defined VOLTAGE_RESTRUCT)
    GetThreePhaseVolt(focVarsCtrl, &g_pwmValue);
    #endif  /* end of VOLTAGE_RESTRUCT */
    #elif (defined FOC_ACIM)
//    GetThreePhaseVolt(focVarsCtrl, &g_pwmValue);
    #endif  /* end of FOC_PMSM */
    CurrentLoop_Process(focVarsCtrl);
#elif (defined VF_CONTROL)
    Vf_ISR();
#else
#endif  /* end of FOC_SINE_WAVE */
#endif  /* end of MOTOR_ONE_TYPE */
}

/*!
* @brief The second part of the FOC interrupt to execute current loop calculation and generate SVPWM modulation wave.
*
* @param[in] focVarsCtrl: pointer to FOC_VARS_CTRL structure
* @return none
*/
void Foc_Execution_CurrentLoopProcess_M2(FOC_VARS_CTRL *focVarsCtrl)
{
#if (defined MOTOR_TWO_TYPE)
#if (defined FOC_SINE_WAVE_M2)
    #if (defined FOC_PMSM_M2)
//    #if (defined SENSORLESS_M2) && (defined VOLTAGE_RESTRUCT_M2)
    #if (defined VOLTAGE_RESTRUCT_M2)
    GetThreePhaseVolt(focVarsCtrl, &g_pwmValue_M2);
    #endif  /* end of VOLTAGE_RESTRUCT_M2 */
    #elif (defined FOC_ACIM_M2)
//    GetThreePhaseVolt(focVarsCtrl, &g_pwmValue_M2);
    #endif  /* end of FOC_PMSM_M2 */
    CurrentLoop_Process_M2(focVarsCtrl);
#elif (defined VF_CONTROL_M2)
//    Vf_ISR();
#else
#endif  /* end of FOC_SINE_WAVE_M2 */
#endif  /* end of MOTOR_TWO_TYPE */
}

/*!
* @brief ADC interrupt service for FOC control of Motor_1.
*
* @param[in] info: pointer to adc_interrupt_info_t
* @param[in] parameter: callback parameter
* @return none
*/
void Motor_Ctrl_CallBack(adc_interrupt_info_t *info, void *parameter)
{
    #ifdef USE_GPIO_CALCULATE_FOC_RUNTIME
    GPIO_DRV_WritePin(GPIOC, 5U, 1U);
    #endif

    /* Start adc convert */
    #if (defined MOTOR_ONE_TYPE) && (!defined MOTOR_TWO_TYPE)
    /* only Motor1 */
    ADC_DRV_SoftwareStartRegularConvert(ADC1_MODULE);
    ADC_DRV_SoftwareStartRegularConvert(ADC0_MODULE);
    #elif (!defined MOTOR_ONE_TYPE) && (defined MOTOR_TWO_TYPE)
    /* only Motor2 */
    #elif (defined MOTOR_ONE_TYPE) && (defined MOTOR_TWO_TYPE)
    /* Motor1 and Motor2 */
    ADC_DRV_SoftwareStartRegularConvert(ADC1_MODULE);
    #endif

#if (defined MOTOR_ONE_TYPE)
    if (info->event & ADC_EVENT_EOC)
    {
        if ((Get_McStatus() == FAULT) || (Get_McStatus() == STOP) || (Get_McStatus() == IDLE) || (Get_McStatus() == POWER_ON))
        {
            if (Get_McStatus() == IDLE)
            {
                GetCurrentOffset_M1(&g_adcSampleReal);
            }
//            #if (defined SENSOR_ACIM)
//            Encoder_GetPulseCount(&g_encoderSensor);
//            #endif

    #if ((defined ENCODER_SENSORS) || (defined ENCODER_HALL_SENSORS))
            Encoder_GetPulseCount(&g_encoderSensor);
    #endif  /* end of ENCODER_SENSORS */
            PWM_Disable(PWM_MODULE_MOTOR1);
            return;
        }
        else if (Get_McStatus() == IDENTIFY)
        {
            if (g_paramIdentify.pParamIdentifyCtrl->mainStep == IDENTIFY_END)
            {
                return;
            }
        }
        else if ((Get_McStatus() == START_BRAKE) || (Get_McStatus() == STOP_BRAKE))
        {
            ShortBrake_AdcProcess(PWM_MODULE_MOTOR1, &g_sBrake, &g_STC, &g_adcParam);
            return;
        }
        else if (Get_McStatus() == ANGLE_JUDGE)
        {
            #if (defined PULSE_INJECTION)
            InitPos_AdcSubCallBack(PWM_MODULE_MOTOR1, &g_pulseInject, pPulseInjCfg, &g_focVars);
            return;
            #endif  /* end of PULSE_INJECTION */
        }
        else
        {
        }
        Get_CurrentSample(&g_adcParam);
        AdcResultTransmit(&g_adcTransform, &g_focVarsCtrl);
        Foc_Execution_AngleProcess(&g_STC);
        Foc_Execution_CurrentLoopProcess(&g_focVarsCtrl);


    #if (defined DAC_FUNCTIONALITY)
        Debug_Display_Variables();
    #endif  /* end of DAC_FUNCTIONALITY */
    }
#endif  /* end of MOTOR_ONE_TYPE */

    #ifdef USE_GPIO_CALCULATE_FOC_RUNTIME
    GPIO_DRV_WritePin(GPIOC, 5U, 0U);
    #endif
}

/*!
* @brief ADC interrupt service for FOC control of Motor_2.
*
* @param[in] info: pointer to adc_interrupt_info_t
* @param[in] parameter: callback parameter
* @return none
*/
void Motor_Ctrl_CallBack_M2(adc_interrupt_info_t *info, void *parameter)
{
#ifdef USE_GPIO_CALCULATE_FOC_RUNTIME
    GPIO_DRV_WritePin(GPIOD, 5U, 1U);
#endif

    /* Start adc convert */
    #if (defined MOTOR_ONE_TYPE) && (!defined MOTOR_TWO_TYPE)
    /* only Motor1 */
    #elif (!defined MOTOR_ONE_TYPE) && (defined MOTOR_TWO_TYPE)
    /* only Motor2 */
    ADC_DRV_SoftwareStartRegularConvert(ADC1_MODULE);
    ADC_DRV_SoftwareStartRegularConvert(ADC0_MODULE);
    #elif (defined MOTOR_ONE_TYPE) && (defined MOTOR_TWO_TYPE)
    /* Motor1 and Motor2 */
    ADC_DRV_SoftwareStartRegularConvert(ADC0_MODULE);
    #endif  /* end of MOTOR_ONE_TYPE */

#if (defined MOTOR_TWO_TYPE)
    if (info->event & ADC_EVENT_EOC)
    {
        if ((Get_McStatus_M2() == FAULT) || (Get_McStatus_M2() == STOP) || (Get_McStatus_M2() == IDLE) || (Get_McStatus_M2() == POWER_ON))
        {
            if (Get_McStatus_M2() == IDLE)
            {
                GetCurrentOffset_M2(&g_adcSampleReal_M2);
            }
//            #if (defined SENSOR_ACIM_M2)
//            Encoder_GetPulseCount(&g_encoderSensor_M2);
//            #endif

            #if ((defined ENCODER_SENSORS_M2) || (defined ENCODER_HALL_SENSORS_M2))
            Encoder_GetPulseCount_M2(&g_encoderSensor_M2);
            #endif  /* end of ENCODER_SENSORS_M2 */
            PWM_Disable(PWM_MODULE_MOTOR2);
            return;
        }
        else if (Get_McStatus_M2() == IDENTIFY)
        {
            if (g_paramIdentify_M2.pParamIdentifyCtrl->mainStep == IDENTIFY_END)
            {
                return;
            }
        }
        else if ((Get_McStatus_M2() == START_BRAKE) || (Get_McStatus_M2() == STOP_BRAKE))
        {
            ShortBrake_AdcProcess(PWM_MODULE_MOTOR2, &g_sBrake_M2, &g_STC_M2, &g_adcParam_M2);
            return;
        }
        else if (Get_McStatus_M2() == ANGLE_JUDGE)
        {
            #if (defined PULSE_INJECTION_M2)
            InitPos_AdcSubCallBack_M2(PWM_MODULE_MOTOR2, &g_pulseInject_M2, pPulseInjCfg_M2, &g_focVars_M2);
            return;
            #endif  /* end of PULSE_INJECTION_M2 */
        }
        else
        {
        }

        Get_CurrentSample_M2(&g_adcParam_M2);
        AdcResultTransmit(&g_adcTransform_M2, &g_focVarsCtrl_M2);
        Foc_Execution_AngleProcess_M2(&g_STC_M2);
        Foc_Execution_CurrentLoopProcess_M2(&g_focVarsCtrl_M2);

        #if (defined DAC_FUNCTIONALITY)
        Debug_Display_Variables_M2();
        #endif

    }
#endif  /* end of MOTOR_TWO_TYPE */

#ifdef USE_GPIO_CALCULATE_FOC_RUNTIME
    GPIO_DRV_WritePin(GPIOD, 5U, 0U);
#endif
}

/* =============================================  EOF  ============================================== */
