/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file motorctrl.c
*
* @brief This file provides the FOC current loop control functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pwmdrv.h"
#include "globalvar.h"
#include "motorctrl_api.h"
#include "hfi_api.h"
#include "foc_keys.h"
#include "adc_hw.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Speed loop PI regulator input parameters initialization.
*
* @param[in] pidCfg: pointer to PID_CFG structure
* @param[in] pidAsr: pointer to PID_REGULATOR structure
* @return none
*/
void Asr_SpeedPIInInit(PID_CFG *pidCfg, PID_REGULATOR *pidAsr)
{
    memset(pidAsr->pPidInput, 0, sizeof(*(pidAsr->pPidInput)));
    pidAsr->pPidInput->maxPu = pidCfg->maxSpeed;
    pidAsr->pPidInput->minPu = pidCfg->minSpeed;
}

/*!
* @brief Position loop PI regulator input parameters initialization.
*
* @param[in] pidCfg: pointer to PID_CFG structure
* @param[in] pidApr: pointer to PID_REGULATOR structure
* @return none
*/
void Apr_PositionPIInInit(PID_CFG *pidCfg, PID_REGULATOR *pidApr)
{
    memset(pidApr->pPidInput, 0, sizeof(*(pidApr->pPidInput)));
    pidApr->pPidInput->maxPu = pidCfg->maxPosition;
    pidApr->pPidInput->minPu = pidCfg->minPosition;
}

/*!
* @brief Speed loop PI regulator output parameters initialization.
*
* @param[in] pidAsr: pointer to PID_REGULATOR structure
* @return none
*/
void Asr_SpeedPIOutInit(PID_REGULATOR *pidAsr)
{
    memset(pidAsr->pPidOutput, 0, sizeof(*(pidAsr->pPidOutput)));
}

/*!
* @brief Position loop PI regulator output parameters initialization.
*
* @param[in] pidApr: pointer to PID_REGULATOR structure
* @return none
*/
void Apr_PositionPIOutInit(PID_REGULATOR *pidApr)
{
    memset(pidApr->pPidOutput, 0, sizeof(*(pidApr->pPidOutput)));
}

/*!
* @brief Speed loop PI regulator input and output parameters initialization.
*
* @param[in] pidAsr: pointer to PID_REGULATOR structure
* @param[in] pidCfg: pointer to PID_CFG structure
* @return none
*/
void Asr_SpeedPIInit(PID_REGULATOR *pidAsr, PID_CFG *pidCfg)
{
    Asr_SpeedPIInInit(pidCfg, pidAsr);
    Asr_SpeedPIOutInit(pidAsr);
}

/*!
* @brief Position loop PI regulator input and output parameters initialization.
*
* @param[in] pidApr: pointer to PID_REGULATOR structure
* @param[in] pidCfg: pointer to PID_CFG structure
* @return none
*/
void Apr_PositionPIInit(PID_REGULATOR *pidApr, PID_CFG *pidCfg)
{
    Apr_PositionPIInInit(pidCfg, pidApr);
    Apr_PositionPIOutInit(pidApr);
}

/*!
* @brief Current loop PI regulator input parameters initialization.
*
* @param[in] pidCfg: pointer to PID_CFG structure
* @param[in] pidAcrD: pointer to PID_REGULATOR structure
* @param[in] pidAcrQ: pointer to PID_REGULATOR structure
* @return none
*/
void Acr_CurPIInInit(PID_CFG *pidCfg, PID_REGULATOR *pidAcrD, PID_REGULATOR *pidAcrQ)
{
    memset(pidAcrD->pPidInput, 0, sizeof(*(pidAcrD->pPidInput)));
    memset(pidAcrQ->pPidInput, 0, sizeof(*(pidAcrQ->pPidInput)));
    pidAcrD->pPidInput->maxPu = pidCfg->maxflux;
    pidAcrD->pPidInput->minPu = pidCfg->minflux;
    pidAcrQ->pPidInput->maxPu = pidCfg->maxtorque;
    pidAcrQ->pPidInput->minPu = pidCfg->mintorque;
}

/*!
* @brief Current loop PI regulator output parameters initialization.
*
* @param[in] pidAcrD: pointer to PID_REGULATOR structure
* @param[in] pidAcrQ: pointer to PID_REGULATOR structure
* @return none
*/
void Acr_CurPIOutInit(PID_REGULATOR *pidAcrD, PID_REGULATOR *pidAcrQ)
{
    memset(pidAcrD->pPidOutput, 0, sizeof(*(pidAcrD->pPidOutput)));
    memset(pidAcrQ->pPidOutput, 0, sizeof(*(pidAcrQ->pPidOutput)));
}

/*!
* @brief Current loop PI regulator input and output parameters initialization.
*
* @param[in] pidAcrD: pointer to PID_REGULATOR structure
* @param[in] pidAcrQ: pointer to PID_REGULATOR structure
* @param[in] pidCfg: pointer to PID_CFG structure
* @return none
*/
void Acr_CurPIInit(PID_REGULATOR *pidAcrD, PID_REGULATOR *pidAcrQ, PID_CFG *pidCfg)
{
    Acr_CurPIInInit(pidCfg, pidAcrD, pidAcrQ);
    Acr_CurPIOutInit(pidAcrD, pidAcrQ);
}

/*!
* @brief Feedforward decoupling of current loop voltage control initialization.
*
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] pUdqDcpIn: pointer to ACR_UDQDCP_IN structure
* @param[in] pUdqDcpCof: pointer to ACR_UDQDCP_COF structure
* @param[in] pUdqDcpOut: pointer to ACR_UDQDCP_OUT structure
* @return none
*/
void Acr_UdqDcpInit(MOTOR_PARAM_PU *paramPu, ACR_UDQDCP_IN *pUdqDcpIn, ACR_UDQDCP_COF *pUdqDcpCof, ACR_UDQDCP_OUT *pUdqDcpOut)
{
    /* D_axis inductance of motor */
    pUdqDcpCof->dcpLdPu = paramPu->ldPu;

    /* Q_axis inductance of motor */
    pUdqDcpCof->dcpLqPu = paramPu->lqPu;

    /*!< flux of rotor */
    pUdqDcpCof->dcpFluxFPu = paramPu->fluxPu;

    memset(pUdqDcpIn, 0, sizeof(*(pUdqDcpIn)));
    memset(pUdqDcpOut, 0, sizeof(*(pUdqDcpOut)));
}

/*!
* @brief Calculate the average value of motor stator current vector amplitude, expressed as 'ialphabeta'.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void Acr_GetAvgIs(FOC_VARS_CTRL *focVars)
{
    static uint16_t avgIsCalcCnt = 0;
    static float avgIsSum = 0;
    float iAlfBeta = 0;

    iAlfBeta = MathFloat_Mag(focVars->ialpha, focVars->ibeta);
    avgIsCalcCnt++;

    avgIsSum += iAlfBeta;
    if (avgIsCalcCnt >= 1024)
    {
        focVars->ialphabeta = (focVars->ialphabeta + (avgIsSum / 1024)) * 0.5f;
        avgIsCalcCnt = 0;
        avgIsSum = 0;
    }
}

/*!
* @brief Calculate the average value of input variable.
*
* @param[in] avgVar: input variable before average
* @param[in] avgOrder: power exponent, used to calculated the average times, e.g. avgOrder = 5, calculate the average of (2^5 = 32) values
* @return avgValue: output variable after average
*/
float GetAvgCalc(float avgVar, uint16_t avgOrder)
{
    static uint16_t avgCalcCnt = 0;
    static float avgSum = 0;
    float avgValue = 0;

    avgValue = avgVar;
    avgCalcCnt++;

    avgSum += avgValue;
    if (avgCalcCnt >= (1 << avgOrder))
    {
        avgValue = avgSum / (1 << avgOrder);
        avgCalcCnt = 0;
        avgSum = 0;
    }

    return avgValue;
}

/*!
* @brief Pre-exciting startup process of induction motor.
*
* @param[in] acimFoc: pointer to ACIM_FOC_CTRL structure
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] time: pre-exciting duration time, unit: ms
* @return PreExcit Startup status
*/
//void Acim_FocPreExcitStartup(ACIM_FOC_CTRL *acimFoc, FOC_VARS *focVars, uint16_t time)
//{
////#if (defined FOC_ACIM)
////    static uint16_t dcStartupCnt = 0;
////    if (g_mcStatus == ALIGN)
////    {
////        dcStartupCnt++;

////        acimFoc->cur->curDRef += 20;
////        if (acimFoc->cur->curDRef > acimFoc->mot->excitCur)
////        {
////            acimFoc->cur->curDRef = acimFoc->mot->excitCur;
////        }
////        focVars->pCtrl->iqRef = 0;

////        focVars->pCtrl->elecAngle = 0;

////        /*  */
////        if (dcStartupCnt >= time)
////        {
////            g_mcStatus = RUN;
////            dcStartupCnt = 0;
////            g_startUpCtrl.startUpFlag = 3;
////        }
////    }
////#endif
//}

/*!
* @brief Speed loop parameter, current loop parameter configuration, which can be used to adjust PI parameters.
*
* @param[in] pidCfg: pointer to PID_CFG structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @return none
*/
void MC_SpeedTrqPICofCalc(PID_CFG *pidCfg, FOC_VARS_CFG *focVars, MOTOR_PARAM_PU *paramPu)
{
#if (defined MOTOR_ONE_TYPE)
    float maxCurBandWidth;

    maxCurBandWidth = (PI2 * focVars->curFreq / 20);
    paramPu->curBandWidthPu = pidCfg->curWicPu / paramPu->web;
    if (pidCfg->curWicPu > maxCurBandWidth)
    {
        paramPu->curBandWidthPu = maxCurBandWidth / paramPu->web;
    }
    /* Speed loop bandwidth */
    paramPu->speedBandWidthPu = pidCfg->speedWic;

    if (pidCfg->curPidSelfLearn == (uint16_t)ENABLE)
    {
        /* PI Cof of ACR_D*/
        g_acrCurIdCofIn.curLPu = paramPu->ldPu;
        g_acrCurIdCofIn.curRsPu = paramPu->rsPu;
        g_acrCurIdCofIn.curTCtrPu = paramPu->tCurCtrPu;
        g_acrCurIdCofIn.curWicPu = paramPu->curBandWidthPu;
        Acr_CurPICofCalc(&g_acrCurIdCofIn, &g_acrCurIdCof);

        /* PI Cof of ACR_Q*/
        g_acrCurIqCofIn.curLPu =  paramPu->lqPu;
        g_acrCurIqCofIn.curRsPu = paramPu->rsPu;
        g_acrCurIqCofIn.curTCtrPu = paramPu->tCurCtrPu;
        g_acrCurIqCofIn.curWicPu = paramPu->curBandWidthPu;
        Acr_CurPICofCalc(&g_acrCurIqCofIn, &g_acrCurIqCof);
    }
    else
    {
        Acr_SetCurD_PICof(pidCfg->fluxKp, pidCfg->fluxKi, 0, 0, &g_acrCurIdCof);
        Acr_SetCurQ_PICof(pidCfg->torqueKp, pidCfg->torqueKi, 0, 0, &g_acrCurIqCof);
    }

    if (pidCfg->speedPidSelfLearn == (uint16_t)ENABLE)
    {
        /* PI Cof of ASR*/
        g_asrSpeedIqCofIn.jmPu =  paramPu->jmPu;
        g_asrSpeedIqCofIn.dampPu = pidCfg->damp;
        g_asrSpeedIqCofIn.fluxPu = paramPu->fluxPu;
        g_asrSpeedIqCofIn.speedTCntCtrPu = paramPu->tSpeedCtrPu;
        g_asrSpeedIqCofIn.speedWicPu = paramPu->speedBandWidthPu;
        Asr_SpeedPICofCalc(&g_asrSpeedIqCofIn, &g_motorParamPu, &g_asrSpeedCof);
    }
    else
    {
    #if (defined ASR_VARPI_ENABLE)
        ASR_SpeedVarPIInit(&g_asrSpeedVarPI, &g_asrSpeedCof);
    #else
        Asr_SetSpeed_PICof(pidCfg->speedKp, pidCfg->speedKi, 0, 0, &g_asrSpeedCof);
    #endif
    }

//    g_udqDcpCofIn.dcpLdPu = g_motorParam.ldPu;
//    g_udqDcpCofIn.dcpLqPu = g_motorParam.lqPu;
//    g_udqDcpCofIn.dcpFluxFPu = g_motorParam.fluxFPu;
    //Acr_UdqDcpCof();

#endif
}

/*!
* @brief Speed loop parameter, current loop parameter configuration, which can be used to adjust PI parameters.
*
* @param[in] pidCfg: pointer to PID_CFG structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @return none
*/
void MC_SpeedTrqPICofCalc_M2(PID_CFG *pidCfg, FOC_VARS_CFG *focVars, MOTOR_PARAM_PU *paramPu)
{
#if (defined MOTOR_TWO_TYPE)
    uint32_t maxCurBandWidth;

    maxCurBandWidth = (PI2 * focVars->curFreq / 20);
    paramPu->curBandWidthPu = pidCfg->curWicPu / paramPu->web;
    if (pidCfg->curWicPu > maxCurBandWidth)
    {
        paramPu->curBandWidthPu = maxCurBandWidth / paramPu->web;
    }
    /* Speed loop bandwidth */
    paramPu->speedBandWidthPu = pidCfg->speedWic;

    if (pidCfg->curPidSelfLearn == (uint16_t)ENABLE)
    {
        /* PI Cof of ACR_D*/
        g_acrCurIdCofIn_M2.curLPu = paramPu->ldPu;
        g_acrCurIdCofIn_M2.curRsPu = paramPu->rsPu;
        g_acrCurIdCofIn_M2.curTCtrPu = paramPu->tCurCtrPu;
        g_acrCurIdCofIn_M2.curWicPu = paramPu->curBandWidthPu;
        Acr_CurPICofCalc(&g_acrCurIdCofIn_M2, &g_acrCurIdCof_M2);

        /* PI Cof of ACR_Q*/
        g_acrCurIqCofIn_M2.curLPu =  paramPu->lqPu;
        g_acrCurIqCofIn_M2.curRsPu = paramPu->rsPu;
        g_acrCurIqCofIn_M2.curTCtrPu = paramPu->tCurCtrPu;
        g_acrCurIqCofIn_M2.curWicPu = paramPu->curBandWidthPu;
        Acr_CurPICofCalc(&g_acrCurIqCofIn_M2, &g_acrCurIqCof_M2);
    }
    else
    {
        Acr_SetCurD_PICof(pidCfg->fluxKp, pidCfg->fluxKi, 0, 0, &g_acrCurIdCof_M2);
        Acr_SetCurQ_PICof(pidCfg->torqueKp, pidCfg->torqueKi, 0, 0, &g_acrCurIqCof_M2);
    }

    if (pidCfg->speedPidSelfLearn == (uint16_t)ENABLE)
    {
        /* PI Cof of ASR*/
        g_asrSpeedIqCofIn_M2.jmPu =  paramPu->jmPu;
        g_asrSpeedIqCofIn_M2.dampPu =  pidCfg->damp;
        g_asrSpeedIqCofIn_M2.fluxPu = paramPu->fluxPu;
        g_asrSpeedIqCofIn_M2.speedTCntCtrPu = paramPu->tSpeedCtrPu;
        g_asrSpeedIqCofIn_M2.speedWicPu = paramPu->speedBandWidthPu;
        Asr_SpeedPICofCalc(&g_asrSpeedIqCofIn_M2, &g_motorParamPu_M2, &g_asrSpeedCof_M2);
    }
    else
    {
    #if (defined ASR_VARPI_ENABLE_M2)
        ASR_SpeedVarPIInit_M2(&g_asrSpeedVarPI_M2, &g_asrSpeedCof_M2);
    #else
        Asr_SetSpeed_PICof(pidCfg->speedKp, pidCfg->speedKi, 0, 0, &g_asrSpeedCof_M2);
    #endif
    }

//    g_udqDcpCofIn.dcpLdPu = g_motorParam.ldPu;
//    g_udqDcpCofIn.dcpLqPu = g_motorParam.lqPu;
//    g_udqDcpCofIn.dcpFluxFPu = g_motorParam.fluxFPu;
    //Acr_UdqDcpCof();

#endif
}

/*!
* @brief Position loop parameter, position loop parameter configuration.
*
* @param[in] pidApr: pointer to PID_REGULATOR structure
* @param[in] pidCfg: pointer to PID_CFG structure
* @return none
*/
void MC_PositionPICofInit(PID_REGULATOR *pidApr, PID_CFG *pidCfg)
{
    pidApr->pPidCoef->kpPu = pidCfg->positionKp;
    pidApr->pPidCoef->kiPu = pidCfg->positionKi;
    pidApr->pPidCoef->kdPu = pidCfg->positionKd;
}

/*!
* @brief Motor control loop: Speed loop and Current loop parameters initialization.
*
* @param[in] none
* @return none
*/
void MC_SpeedTrqInit(void)
{
#if (defined MOTOR_ONE_TYPE)
    Acr_CurPIInit(&g_pidAcrD, &g_pidAcrQ, pPidCfg);
    Asr_SpeedPIInit(&g_pidAsr, pPidCfg);
    Acr_UdqDcpInit(&g_motorParamPu, &g_udqDcpIn, &g_udqDcpCof, &g_udqDcpOut);
    MC_SpeedTrqPICofCalc(pPidCfg, pFocVarsCfg, &g_motorParamPu);

    memset(&g_pwmValue, 0, sizeof(g_pwmValue));
    memset(&g_curRestruct, 0, sizeof(g_curRestruct));
    memset(&g_pwmShift, 0, sizeof(g_pwmShift));
#endif
}

/*!
* @brief Motor control loop: Speed loop and Current loop parameters initialization.
*
* @param[in] none
* @return none
*/
void MC_SpeedTrqInit_M2(void)
{
#if (defined MOTOR_TWO_TYPE)
    Acr_CurPIInit(&g_pidAcrD_M2, &g_pidAcrQ_M2, pPidCfg_M2);
    Asr_SpeedPIInit(&g_pidAsr_M2, pPidCfg_M2);
    Acr_UdqDcpInit(&g_motorParamPu_M2, &g_udqDcpIn_M2, &g_udqDcpCof_M2, &g_udqDcpOut_M2);
    MC_SpeedTrqPICofCalc_M2(pPidCfg_M2, pFocVarsCfg_M2, &g_motorParamPu_M2);

    memset(&g_pwmValue_M2, 0, sizeof(g_pwmValue_M2));
    memset(&g_curRestruct_M2, 0, sizeof(g_curRestruct_M2));
    memset(&g_pwmShift_M2, 0, sizeof(g_pwmShift_M2));
#endif
}

/*!
* @brief Motor control loop: Position loop parameters initialization.
*
* @param[in] pid: pointer to PID_REGULATOR structure
* @param[in] pidCfg: pointer to PID_CFG structure
* @return none
*/
void MC_PositionInit(PID_REGULATOR *pid, PID_CFG *pidCfg)
{
    Apr_PositionPIInit(pid, pidCfg);
    MC_PositionPICofInit(pid, pidCfg);
}

/*!
* @brief Current loop output voltage composite vector amplitude limiting,
*        limits the 'vdq' to ensure nornal operation of motor.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void Acr_Vdq_Limit(FOC_VARS_CTRL *focVars)
{
#if (defined MOTOR_ONE_TYPE)
    float s_vdqCoeff = 0;
    float IdOutTemp = focVars->vd;
    float IqOutTemp = focVars->vq;
    float vdqLimit = 0;

    focVars->vdq = MathFloat_Mag(IdOutTemp, IqOutTemp);
    vdqLimit = VDQ_LIMIT_VALUE * focVars->vdcPu;
    if (focVars->vdq > vdqLimit)
    {
        s_vdqCoeff = vdqLimit / focVars->vdq;
        focVars->vd = IdOutTemp * s_vdqCoeff;
        focVars->vq = IqOutTemp * s_vdqCoeff;
    }
    else
    {
        /* Do nothing */
    }
#endif
}

/*!
* @brief Current loop output voltage composite vector amplitude limiting of Motor 2,
*        limits the 'vdq' to ensure nornal operation of motor.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void Acr_Vdq_Limit_M2(FOC_VARS_CTRL *focVars)
{
#if (defined MOTOR_TWO_TYPE)
    float s_vdqCoeff = 0;
    float IdOutTemp = focVars->vd;
    float IqOutTemp = focVars->vq;
    float vdqLimit = 0;

    focVars->vdq = MathFloat_Mag(IdOutTemp, IqOutTemp);
    vdqLimit = VDQ_LIMIT_VALUE_M2 * focVars->vdcPu;
    if (focVars->vdq > vdqLimit)
    {
        s_vdqCoeff = vdqLimit / focVars->vdq;
        focVars->vd = IdOutTemp * s_vdqCoeff;
        focVars->vq = IqOutTemp * s_vdqCoeff;
    }
    else
    {
        /* Do nothing */
    }
#endif
}

/*!
* @brief Over voltage suppression initialization.
*
* @param[in] overVoltSupp: pointer to OVER_VOLTAGE_SUPPRESS structure
* @param[in] pid: pointer to PID_REGULATOR structure
* @param[in] pidCfg: pointer to PID_CFG structure
* @param[in] STC: pointer to STC_PARAM structure
* @return none
*/
void OverVoltageSuppressInit(OVER_VOLTAGE_SUPPRESS *overVoltSupp, PID_REGULATOR *pid, PID_CFG *pidCfg, STC_PARAM *STC)
{
    memset(overVoltSupp, 0, sizeof(*overVoltSupp));

    overVoltSupp->overVoltPoint = defHwBoard.maxVoltage / STC->pMotorPu->ubVt;
    overVoltSupp->outLimit = pidCfg->maxSpeed;
    pid->pPidCoef->kpPu = (0.5f);
    pid->pPidCoef->kiPu = (0.005f);
    pid->pPidInput->maxPu = overVoltSupp->outLimit;
    pid->pPidInput->minPu = 0;
}

/*!
* @brief Over voltage suppression to prevent bus voltage overvoltage when Motor motion status is slow down.
*
* @param[in] udcPu: DC Bus voltage
* @param[in] overVoltSupp: pointer to OVER_VOLTAGE_SUPPRESS structure
* @param[in] pid: pointer to PID_REGULATOR structure
* @param[in] asrIn: pointer to PID_REGULATOR_IN structure
* @return none
*/
void OverVoltageSuppressProcess(float udcPu, OVER_VOLTAGE_SUPPRESS *overVoltSupp, PID_REGULATOR *pid, PID_REGULATOR_IN *asrIn)
{
    float overVoltSuppOutput = 0;

    pid->pPidInput->refPu = overVoltSupp->overVoltPoint;
    pid->pPidInput->fbkPu = udcPu;
    overVoltSuppOutput = PID_SeriesRegulator(pid);
    asrIn->maxPu = overVoltSuppOutput;
    asrIn->minPu = -overVoltSuppOutput;
}

/*!
* @brief Parameter identify run in isr. This function provides Rs Ld Lq Flux identify isr interface.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return temp: indicate whether to end the foc isr
*/
//uint16_t ParamIdentify_AdcSubCallBack(uint8_t pwmModule, FOC_VARS *focVars, PARAM_IDENTIFY_STRUCT *paramId, ADC_PARAM *adc)
//{
//    uint16_t temp = 0;
//    PARAM_IDENTIFY_MAIN_CTRL *parIdCtrl = paramId->pParamIdentifyCtrl;

//    if (parIdCtrl->mainStep == (uint8_t)IDENTIFY_LDQ_STALLTEST)
//    {
//        if (paramId->pParamIdentifyCfg->pmsmLdqTypeSel == 1)
//        {
//            if ((paramId->pParamLdqIdentify->step != 0) && (paramId->pParamLdqIdentify->step != 5))
//            {
//                adc->adTrans->iaPu = adc->adCoef->kCurSamplePu * (adc->adSample->iaOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_0));
//                adc->adTrans->ibPu = adc->adCoef->kCurSamplePu * (adc->adSample->ibOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_1));
//                adc->adTrans->icPu = adc->adCoef->kCurSamplePu * (adc->adSample->icOffset - ADC_GetInjectData(ADC0, ADC_ISEQ_2));
//
//                temp = 1;
//            }
//            else
//            {
//                Get_CurrentSample(adc);
//                temp = 0;
//            }
//            AdcResultTransmit(adc->adTrans, focVars->pCtrl);
//            ParamIdentify_LdqCallBack(pwmModule, paramId, paramId->pParamLdqIdentify, focVars);
//            }
//        else
//        {
//            temp = 0;
//        }

//    }
//    else if (parIdCtrl->mainStep == IDENTIFY_FLUX_ROTATIONTEST)
//    {
//        if (paramId->pParamFluxIdentify->subStep != 0)
//        {
//            parIdCtrl->setElecAngle += paramId->pParamFluxIdentify->fluxIdThetaDelta;
//        }

//        paramId->pParamFluxIdentify->fluxIdOutVolPu = Math_Q15Mag(focVars->pCtrl->vd, focVars->pCtrl->vd);

//        paramId->pParamFluxIdentify->fluxIdOutCurPu = focVars->pCtrl->idRef;
//        temp = 0;
//    }
//    else
//    {
//        temp = 0;
//    }

//    return (temp);
//}

/*!
* @brief Load torque compensation function, feedforward compensation to the Q axis current loop.
*
* @param[in] loadTorque: pointer to TRQ_LOAD_EST structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @param[in] acrQ: pointer to PID_REGULATOR structure
* @param[in] startupCtrl: pointer to STARTUP_CTRL structure
* @return none
*/
void TrqLoad_Comp(TRQ_LOAD_EST *loadTorque, FOC_VARS_CTRL *focVars, PID_REGULATOR *acrQ, STARTUP_CTRL *startupCtrl)
{
#if (defined SENSORLESS)
    if (startupCtrl->startUpFlag == 6)
    {
        Set_PID_Ref(acrQ, (focVars->iqRef + loadTorque->iqCompPu));
    }
    else
    {
        Set_PID_Ref(acrQ, focVars->iqRef);
    }
#elif (defined HALL_SENSORS)
    if (g_hallSensor.swapstate == 1)
    {
        Set_PID_Ref(acrQ, (focVars->iqRef + loadTorque->iqCompPu));
    }
    else
    {
        Set_PID_Ref(acrQ, focVars->iqRef);
    }
#else
#endif
}

/*!
* @brief Load torque compensation function, feedforward compensation to the Q axis current loop.
*
* @param[in] loadTorque: pointer to TRQ_LOAD_EST structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @param[in] acrQ: pointer to PID_REGULATOR structure
* @param[in] startupCtrl: pointer to STARTUP_CTRL structure
* @return none
*/
void TrqLoad_Comp_M2(TRQ_LOAD_EST *loadTorque, FOC_VARS_CTRL *focVars, PID_REGULATOR *acrQ, STARTUP_CTRL *startupCtrl)
{
#if (defined SENSORLESS_M2)
    if (startupCtrl->startUpFlag == 6)
    {
        Set_PID_Ref(acrQ, (focVars->iqRef + loadTorque->iqCompPu));
    }
    else
    {
        Set_PID_Ref(acrQ, focVars->iqRef);
    }
#elif (defined HALL_SENSORS_M2)
    if (g_hallSensor_M2.swapstate == 1)
    {
        Set_PID_Ref(acrQ, (focVars->iqRef + loadTorque->iqCompPu));
    }
    else
    {
        Set_PID_Ref(acrQ, focVars->iqRef);
    }
#else
#endif
}

/*!
* @brief Calculation and modulation of SVPWM.
*
* @param[in] none
* @return none
*/
void Svpwm_Process(void)
{
#if (defined MOTOR_ONE_TYPE)
    Svpwm_Gen(&g_focVars, &g_pwmValue, pSvModulationCfg);

    #ifdef DEAD_BAND_COMP
    DeadBandCompCal(&g_deadCompCtrl, &g_pwmValue, pDeadCompCfg, &g_focVars);
    #endif

    #ifdef NARROW_PULSE_PROCESS
    NarrowPulseProcess(&g_pwmValue, pFocVarsCfg);
    #endif
#endif
}

/*!
* @brief Calculation and modulation of SVPWM.
*
* @param[in] none
* @return none
*/
void Svpwm_Process_M2(void)
{
#if (defined MOTOR_TWO_TYPE)
    Svpwm_Gen(&g_focVars_M2, &g_pwmValue_M2, pSvModulationCfg_M2);

    #ifdef DEAD_BAND_COMP_M2
    DeadBandCompCal(&g_deadCompCtrl_M2, &g_pwmValue_M2, pDeadCompCfg_M2, &g_focVars_M2);
    #endif

    #ifdef NARROW_PULSE_PROCESS_M2
    NarrowPulseProcess(&g_pwmValue_M2, pFocVarsCfg_M2);
    #endif
#endif
}

/*!
* @brief Load the duty cycle calculated by SVPWM algorithm into the PWM module.
*
* @param[in] none
* @return none
*/
void LoadToPwmModule(void)
{
#if (defined MOTOR_ONE_TYPE)
    #if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    CurRestruct_PwmShift(&g_pwmValue, &g_curRestruct, &g_pwmShift, pCurRestructCfg, pFocVarsCfg);
    PWM_SingleShuntUpdate(&g_curRestruct, &g_focVars);

    #elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    PWM_Update(g_pwmValue.pwmTa, g_pwmValue.pwmTb, g_pwmValue.pwmTc, &g_focVars);
    #elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)

    #endif
#endif
}

/*!
* @brief Load the duty cycle calculated by SVPWM algorithm into the PWM module.
*
* @param[in] none
* @return none
*/
void LoadToPwmModule_M2(void)
{
#if (defined MOTOR_TWO_TYPE)
    #if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    CurRestruct_PwmShift(&g_pwmValue_M2, &g_curRestruct_M2, &g_pwmShift_M2, pCurRestructCfg_M2, pFocVarsCfg_M2);
    PWM_SingleShuntUpdate(&g_curRestruct_M2, &g_focVars_M2);

    #elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    PWM_Update(g_pwmValue_M2.pwmTa, g_pwmValue_M2.pwmTb, g_pwmValue_M2.pwmTc, &g_focVars_M2);
    #elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)

    #endif
#endif
}

/*!
* @brief FOC current loop processing. According to the Vector Control principle,
*        the conduction time of each switch tube in the inverter is calculated
*        to drive the motor.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void CurrentLoop_Process(FOC_VARS_CTRL *focVars)
{
#if (defined MOTOR_ONE_TYPE)
    float tempSin, tempCos;

    tempSin = MathFloat_Sin(focVars->elecAngle);
    tempCos = MathFloat_Cos(focVars->elecAngle);

    /* Current Clark transform*/
    focVars->ialpha = MathFloat_AutoSat(focVars->iaPu);
    focVars->ibeta = MathFloat_AutoSat(INV_SQRT3 * (focVars->iaPu + (focVars->ibPu * 2)));

    if (Get_McStatus() != IDENTIFY)
    {
        /* Current Park transform*/
#if (defined HIGH_FREQUENCE_INJECTION)
        focVars->idBeforeLpf = MathFloat_AutoSat(tempCos * focVars->ialpha + tempSin * focVars->ibeta);
        focVars->iqBeforeLpf = MathFloat_AutoSat(-tempSin * focVars->ialpha + tempCos * focVars->ibeta);
        if ((g_hfinj.state >= 4U) && (MathFloat_Abs(g_hfinj.weightFreq) > pHighFreqInjCfg->hfiVdFreqHigh))
        {
            focVars->idFdb = focVars->idBeforeLpf;
            focVars->iqFdb = focVars->iqBeforeLpf;
        }
        else
        {
            focVars->idFdb += CUR_LPF_COEFF * (focVars->idBeforeLpf - focVars->idFdb);
            focVars->iqFdb += CUR_LPF_COEFF * (focVars->iqBeforeLpf - focVars->iqFdb);
        }
#else
        focVars->idFdb = MathFloat_AutoSat(tempCos * focVars->ialpha + tempSin * focVars->ibeta);
        focVars->iqFdb = MathFloat_AutoSat((-tempSin) * focVars->ialpha + tempCos * focVars->ibeta);
#endif  /* end of HIGH_FREQUENCE_INJECTION */

        focVars->idq = MathFloat_Mag(focVars->idFdb, focVars->iqFdb);
    }
    else
    {
    }

    if (focVars->pwmOutMask == SVPWM_OUTPUT_ENABLE)
    {
        if (Get_McStatus() != IDENTIFY)
        {
            if (Get_McStatus() == INIT_POS)
            {
                #if (defined INIT_POS_DETEC)
                InitPosDetec_CommandGen(&g_initPosDetecStruct, &g_STC);
                InitPosDetec_IsrTask(PWM_MODULE_MOTOR1, &g_initPosDetecStruct, &g_initPosPolar, pInitPosDetecCfg, &g_STC);
                #endif  /* end of INIT_POS_DETEC */
            }
            else
            {
#if (defined FOC_PMSM)
    #if (defined FEED_FORWARD_DECOUPLE)
            g_udqDcpIn.dcpIdRefPu = focVars->idRef;
            g_udqDcpIn.dcpIqRefPu = focVars->iqRef;
            g_udqDcpIn.dcpWsPu = g_asrSpeedPIIn.fbkPu;
            g_udqDcpIn.dcpULimPu = FEED_FORWARD_LIMIT;
            Acr_UdqDcpCalc(&g_udqDcpIn, &g_udqDcpCof, &g_udqDcpOut);
            g_acrCurIdPIIn.feedForwardPu = g_udqDcpOut.dcpUdPu;
            g_acrCurIqPIIn.feedForwardPu = g_udqDcpOut.dcpUqPu;
    #else
            g_acrCurIdPIIn.feedForwardPu = 0;
            g_acrCurIqPIIn.feedForwardPu = 0;
    #endif  /* end of FEED_FORWARD_DECOUPLE */

            /* ACR_D */
            Set_PID_Ref(&g_pidAcrD, focVars->idRef);
            Set_PID_Fbk(&g_pidAcrD, focVars->idFdb);
    #if (defined COMPLEX_VECTOR_DECOUPLE)
            Current_Decoup_Regulator(&g_pidAcrD, &g_pidAcrQ, &g_motorParamPu, g_asrSpeedPIIn.fbkPu);
            focVars->vd = g_pidAcrD.pPidOutput->outPu;
    #else
            focVars->vd = PID_SeriesRegulator(&g_pidAcrD);
    #endif  /* end of COMPLEX_VECTOR_DECOUPLE */

            /* ACR_Q */
    #if (defined LOAD_TORQUE_ESTIMATE)
            TrqLoadEst_Calc(&g_loadTorqueEst, pLoadTorqueCfg, &g_motorParamPu, g_speedCommand.speedFbk, focVars->iqFdb);
            TrqLoad_Comp(&g_loadTorqueEst, &g_focVarsCtrl, &g_pidAcrQ, &g_startUpCtrl);
    #else
            Set_PID_Ref(&g_pidAcrQ, focVars->iqRef);
    #endif  /* end of LOAD_TORQUE_ESTIMATE */
            Set_PID_Fbk(&g_pidAcrQ, focVars->iqFdb);
    #if (defined COMPLEX_VECTOR_DECOUPLE)
            focVars->vq = g_pidAcrQ.pPidOutput->outPu;
    #else
            focVars->vq = PID_SeriesRegulator(&g_pidAcrQ);
    #endif  /* end of COMPLEX_VECTOR_DECOUPLE */

    #if (defined SAVE_ENERGY)
            EnergySave_CalculateIsVal(&g_energySave, focVars);
    #endif  /* end of SAVE_ENERGY */

    #if (((defined HALL_SENSORS) || (defined ENCODER_SENSORS) || (defined ENCODER_HALL_SENSORS)) && (defined VDQ_CIRCLE_LIMIT_CTRL))
            Acr_Vdq_Limit(focVars);
            focVars->vdqLimit = MathFloat_Mag(focVars->vd, focVars->vq);
            focVars->uOutputPu = focVars->vdqLimit;
        #if (defined FLUX_WEAKENING)
            focVars->vdqFilter = (focVars->vdqFilter * 7 + focVars->vdq) / 8;
        #endif  /* end of FLUX_WEAKENING */
    #endif  /* end of HALL_SENSORS */

    #if (defined SENSORLESS)
        #if (defined IF_STARTUP) && (defined VDQ_CIRCLE_LIMIT_CTRL)
            Acr_Vdq_Limit(focVars);
            focVars->vdqLimit = MathFloat_Mag(focVars->vd, focVars->vq);
            focVars->uOutputPu = focVars->vdqLimit;
            #if (defined FLUX_WEAKENING)
            focVars->vdqFilter = (focVars->vdqFilter * 7 + focVars->vdq) / 8;
            #endif  /* end of FLUX_WEAKENING */

        #elif (defined VF_STARTUP)
            focVars->vd = focVars->vdRef;
            focVars->vq = focVars->vqRef;
            focVars->uOutputPu = MathFloat_Mag(focVars->vd, focVars->vq);

        #elif (defined HIGH_FREQUENCE_INJECTION)
            HighFreqInj_Proc(&g_hfiCtrl, pHighFreqInjCfg, &g_focVars, &g_pidHfi, &g_pidAsr);
            if ((g_hfinj.state == 0U) || (g_hfinj.state == 2U))
            {
                focVars->vd = g_hfinj.vsdRef;
                focVars->vq = 0;
            }
            else if ((g_hfinj.state == 3U) || (g_hfinj.state == 4U))
            {
                focVars->vd += g_hfinj.vsdRef;
            }
            else
            {
                focVars->vd = 0;
                focVars->vq = 0;
            }
            focVars->vd = MathFloat_AutoSat(focVars->vd);
        #endif  /* end of IF_STARTUP */
    #endif  /* end of SENSORLESS */
#elif (defined FOC_ACIM)
//            if (g_mcStatus == START)
//            {
//                focVars->idRef = g_acimFocCtrl.cur->curDRef;
//                focVars->iqRef = 0;
//            }
//            else
//            {
//                #if (defined SENSORLESS_ACIM)
//                if (g_startUp.pStCtrl->startUpFlag > 3)
//                {
////                    focVars->idRef = g_acimFocCtrl.cur->excitCurCalc;
//                    focVars->idRef = g_acimFocCtrl.mot->excitCur;
//                }
//                else
//                {
//                    focVars->idRef = g_acimFocCtrl.mot->excitCur;
//                }
//                #else
//                focVars->idRef = g_acimFocCtrl.mot->excitCur;
//                #endif
//            }

//            Set_PID_Ref(&g_pidAcrD, focVars->idRef);
//            Set_PID_Fbk(&g_pidAcrD, focVars->idFdb);
//            focVars->vd = PID_SeriesRegulator(&g_pidAcrD);

//            Set_PID_Ref(&g_pidAcrQ, focVars->iqRef);
//            Set_PID_Fbk(&g_pidAcrQ, focVars->iqFdb);
//            focVars->vq = PID_SeriesRegulator(&g_pidAcrQ);

//            Acim_CurrentLoopDecouple(&g_acimFocCtrl, focVars);

//            focVars->vd = focVars->vd + g_acimFocCtrl.volt->decoupleVoltD;
//            focVars->vq = focVars->vq + g_acimFocCtrl.volt->decoupleVoltQ;
//            Acr_Vdq_Limit(focVars);
//            focVars->vdqLimit = Math_Q15Mag(focVars->vd, focVars->vq);
//            focVars->uOutputPu = focVars->vdqLimit;
#endif

            focVars->valpha = MathFloat_AutoSat(tempCos * focVars->vd - tempSin * focVars->vq);
            focVars->vbeta = MathFloat_AutoSat(tempSin * focVars->vd + tempCos * focVars->vq);
            }
        }
        else if (Get_McStatus() == IDENTIFY)
        {
            if (g_paramIdentify.pParamIdentifyCtrl->mainStep == IDENTIFY_RS_DCTEST)
            {
                Set_PID_Ref(&g_pidAcrAlfaRsId, g_paramIdentify.pParamRsIdentify->ialfaRef);
                Set_PID_Fbk(&g_pidAcrAlfaRsId, focVars->ialpha);
                focVars->valpha = PID_SeriesRegulator(&g_pidAcrAlfaRsId);

                if (g_paramIdentify.pParamRsIdentify->phaseOrder != 0U)
                {
                    Set_PID_Ref(&g_pidAcrBetaRsId, g_paramIdentify.pParamRsIdentify->ibetaRef);
                    Set_PID_Fbk(&g_pidAcrBetaRsId, focVars->ibeta);
                    focVars->vbeta = PID_SeriesRegulator(&g_pidAcrBetaRsId);
                }
                else
                {
                    focVars->vbeta = 0;
                }
                focVars->uOutputPu = MathFloat_Mag(focVars->valpha, focVars->vbeta);
            }
            else if (g_paramIdentify.pParamIdentifyCtrl->mainStep == IDENTIFY_LDQ_STALLTEST)
            {
                #if (defined FOC_PMSM)
                focVars->idFdb = MathFloat_AutoSat(tempCos * focVars->ialpha + tempSin * focVars->ibeta);
                focVars->iqFdb = MathFloat_AutoSat((-tempSin) * focVars->ialpha + tempCos * focVars->ibeta);

//                PmsmId_LdqIdCommondGen(&g_paramIdentify, &g_STC);
                PMSMLdqId_IsrTask_CommandGen(&g_paramIdentify, &g_STC);

                Set_PID_Ref(&g_pidAcrD, focVars->idRef);
                Set_PID_Fbk(&g_pidAcrD, focVars->idFdb);
                focVars->vd = PID_SeriesRegulator(&g_pidAcrD);
                Set_PID_Ref(&g_pidAcrQ, focVars->iqRef);
                Set_PID_Fbk(&g_pidAcrQ, focVars->iqFdb);
                focVars->vq = PID_SeriesRegulator(&g_pidAcrQ);

                focVars->valpha = MathFloat_AutoSat(tempCos * focVars->vd - tempSin * focVars->vq);
                focVars->vbeta = MathFloat_AutoSat(tempSin * focVars->vd + tempCos * focVars->vq);

                #elif (defined FOC_ACIM)
//                AcimId_RotorIdSignalGeneration(&g_paramIdentify, &g_STC);
//                Set_PID_Ref(&g_pidAcrAlfaRsId, g_paramIdentify.pAcimParamIdentify->curAlfaSet);
//                Set_PID_Fbk(&g_pidAcrAlfaRsId, focVars->ialpha);
//                focVars->valpha = PID_SeriesRegulator(&g_pidAcrAlfaRsId);
//                focVars->vbeta = 0;
//                focVars->uOutputPu = Math_Q15Mag(focVars->valpha, focVars->vbeta);
                #else
                #endif
            }
            else if (g_paramIdentify.pParamIdentifyCtrl->mainStep == IDENTIFY_FLUX_ROTATIONTEST)
            {
                #if (defined FOC_PMSM)
                if (g_paramIdentify.pParamFluxIdentify->subStep != 0)
                {
                    g_paramIdentify.pParamIdentifyCtrl->setElecAngle += g_paramIdentify.pParamFluxIdentify->fluxIdThetaDelta;
                }
                if (g_paramIdentify.pParamIdentifyCtrl->setElecAngle > 1.0f)
                {
                    g_paramIdentify.pParamIdentifyCtrl->setElecAngle -= 1.0f;
                }
                g_paramIdentify.pParamFluxIdentify->fluxIdOutVolPu = MathFloat_Mag(focVars->vd, focVars->vq);
                g_paramIdentify.pParamFluxIdentify->fluxIdOutCurPu = focVars->idFdb;
                
                focVars->idFdb = MathFloat_AutoSat(tempCos * focVars->ialpha + tempSin * focVars->ibeta);
                focVars->iqFdb = MathFloat_AutoSat((-tempSin) * focVars->ialpha + tempCos * focVars->ibeta);
                Set_PID_Ref(&g_pidAcrD, focVars->idRef);
                Set_PID_Fbk(&g_pidAcrD, focVars->idFdb);
                focVars->vd = PID_SeriesRegulator(&g_pidAcrD);
                Set_PID_Ref(&g_pidAcrQ, focVars->iqRef);
                Set_PID_Fbk(&g_pidAcrQ, focVars->iqFdb);
                focVars->vq = PID_SeriesRegulator(&g_pidAcrQ);

                focVars->valpha = MathFloat_AutoSat(tempCos * focVars->vd - tempSin * focVars->vq);
                focVars->vbeta = MathFloat_AutoSat(tempSin * focVars->vd + tempCos * focVars->vq);

                #elif (defined FOC_ACIM)
//                AcimId_RotationVfControl(&g_paramIdentify, &g_STC);
//                focVars->uOutputPu = Math_Q15Mag(focVars->valpha, focVars->vbeta);

                #else
                #endif
            }
            else
            {
            }
        }
        else
        {
        }
        Svpwm_Process();
        LoadToPwmModule();
    }
    else
    {
        focVars->idRef = 0;
        focVars->iqRef = 0;
        focVars->idFdb = 0;
        focVars->iqFdb = 0;
        focVars->vd = 0;
        focVars->vq = 0;
        focVars->valpha = 0;
        focVars->vbeta = 0;
    }
#endif  /* end of MOTOR_ONE_TYPE */
}

/*!
* @brief FOC current loop processing. According to the Vector Control principle,
*        the conduction time of each switch tube in the inverter is calculated
*        to drive the motor.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void CurrentLoop_Process_M2(FOC_VARS_CTRL *focVars)
{
#if (defined MOTOR_TWO_TYPE)
    float tempSin, tempCos;

    tempSin = MathFloat_Sin(focVars->elecAngle);
    tempCos = MathFloat_Cos(focVars->elecAngle);

    /* Current Clark transform*/
    focVars->ialpha = MathFloat_AutoSat(focVars->iaPu);
    focVars->ibeta = MathFloat_AutoSat(INV_SQRT3 * (focVars->iaPu + (focVars->ibPu * 2)));

    if (Get_McStatus_M2() != IDENTIFY)
    {
        /* Current Park transform*/
#if (defined HIGH_FREQUENCE_INJECTION_M2)
        focVars->idBeforeLpf = MathFloat_AutoSat(tempCos * focVars->ialpha + tempSin * focVars->ibeta);
        focVars->iqBeforeLpf = MathFloat_AutoSat(-tempSin * focVars->ialpha + tempCos * focVars->ibeta);
        if ((g_hfinj_M2.state >= 4U) && (MathFloat_Abs(g_hfinj_M2.weightFreq) > pHighFreqInjCfg_M2->hfiVdFreqHigh))
        {
            focVars->idFdb = focVars->idBeforeLpf;
            focVars->iqFdb = focVars->iqBeforeLpf;
        }
        else
        {
            focVars->idFdb += MathFloat_Mpy(CUR_LPF_COEFF, focVars->idBeforeLpf - focVars->idFdb);
            focVars->iqFdb += MathFloat_Mpy(CUR_LPF_COEFF, focVars->iqBeforeLpf - focVars->iqFdb);
        }
#else

        focVars->idFdb = MathFloat_AutoSat(tempCos * focVars->ialpha + tempSin * focVars->ibeta);
        focVars->iqFdb = MathFloat_AutoSat((-tempSin) * focVars->ialpha + tempCos * focVars->ibeta);
#endif  /* end of HIGH_FREQUENCE_INJECTION_M2 */

        focVars->idq = MathFloat_Mag(focVars->idFdb, focVars->iqFdb);
    }
    else
    {
    }

    if (focVars->pwmOutMask == SVPWM_OUTPUT_ENABLE)
    {
        if (Get_McStatus_M2() != IDENTIFY)
        {
            if (Get_McStatus_M2() == INIT_POS)
            {
                #if (defined INIT_POS_DETEC_M2)
                InitPosDetec_CommandGen(&g_initPosDetecStruct_M2, &g_STC_M2);
                InitPosDetec_IsrTask(PWM_MODULE_MOTOR2, &g_initPosDetecStruct_M2, &g_initPosPolar_M2, pInitPosDetecCfg_M2, &g_STC_M2);
                #endif  /* end of INIT_POS_DETEC_M2 */
            }
            else
            {
#if (defined FOC_PMSM_M2)
    #if (defined FEED_FORWARD_DECOUPLE_M2)
            g_udqDcpIn_M2.dcpIdRefPu = focVars->idRef;
            g_udqDcpIn_M2.dcpIqRefPu = focVars->iqRef;
            g_udqDcpIn_M2.dcpWsPu = g_asrSpeedPIIn_M2.fbkPu;
            g_udqDcpIn_M2.dcpULimPu = FEED_FORWARD_LIMIT_M2;
            Acr_UdqDcpCalc(&g_udqDcpIn_M2, &g_udqDcpCof_M2, &g_udqDcpOut_M2);
            g_acrCurIdPIIn_M2.feedForwardPu = g_udqDcpOut_M2.dcpUdPu;
            g_acrCurIqPIIn_M2.feedForwardPu = g_udqDcpOut_M2.dcpUqPu;
    #else
            g_acrCurIdPIIn_M2.feedForwardPu = 0;
            g_acrCurIqPIIn_M2.feedForwardPu = 0;
    #endif  /* end of FEED_FORWARD_DECOUPLE_M2 */

            /* ACR_D */
            Set_PID_Ref(&g_pidAcrD_M2, focVars->idRef);
            Set_PID_Fbk(&g_pidAcrD_M2, focVars->idFdb);
    #if (defined COMPLEX_VECTOR_DECOUPLE_M2)
            Current_Decoup_Regulator(&g_pidAcrD_M2, &g_pidAcrQ_M2, &g_motorParamPu_M2, g_asrSpeedPIIn_M2.fbkPu);
            focVars->vd = g_pidAcrD_M2.pPidOutput->outPu;
    #else
            focVars->vd = PID_SeriesRegulator(&g_pidAcrD_M2);
    #endif  /* end of COMPLEX_VECTOR_DECOUPLE_M2 */

            /* ACR_Q */
    #if (defined LOAD_TORQUE_ESTIMATE_M2)
            TrqLoadEst_Calc(&g_loadTorqueEst_M2, pLoadTorqueCfg_M2, &g_motorParamPu_M2, g_speedCommand_M2.speedFbk, focVars->iqFdb);
            TrqLoad_Comp_M2(&g_loadTorqueEst_M2, &g_focVarsCtrl_M2, &g_pidAcrQ_M2, &g_startUpCtrl_M2);
    #else
            Set_PID_Ref(&g_pidAcrQ_M2, focVars->iqRef);
    #endif  /* end of LOAD_TORQUE_ESTIMATE_M2 */
            Set_PID_Fbk(&g_pidAcrQ_M2, focVars->iqFdb);
    #if (defined COMPLEX_VECTOR_DECOUPLE_M2)
            focVars->vq = g_pidAcrQ_M2.pPidOutput->outPu;
    #else
            focVars->vq = PID_SeriesRegulator(&g_pidAcrQ_M2);
    #endif  /* end of COMPLEX_VECTOR_DECOUPLE_M2 */

    #if (defined SAVE_ENERGY_M2)
            EnergySave_CalculateIsVal(&g_energySave_M2, focVars);
    #endif  /* end of SAVE_ENERGY_M2 */

    #if (((defined HALL_SENSORS_M2) || (defined ENCODER_SENSORS_M2) || (defined ENCODER_HALL_SENSORS_M2)) && (defined VDQ_CIRCLE_LIMIT_CTRL_M2))
            Acr_Vdq_Limit_M2(focVars);
            focVars->vdqLimit = MathFloat_Mag(focVars->vd, focVars->vq);
            focVars->uOutputPu = focVars->vdqLimit;
        #if (defined FLUX_WEAKENING_M2)
            focVars->vdqFilter = (focVars->vdqFilter * 7U + focVars->vdq) / 8;
        #endif  /* end of FLUX_WEAKENING_M2 */
    #endif  /* end of HALL_SENSORS_M2 */

    #if (defined SENSORLESS_M2)
        #if (defined IF_STARTUP_M2) && (defined VDQ_CIRCLE_LIMIT_CTRL_M2)
            Acr_Vdq_Limit_M2(focVars);
            focVars->vdqLimit = MathFloat_Mag(focVars->vd, focVars->vq);
            focVars->uOutputPu = focVars->vdqLimit;
            #if (defined FLUX_WEAKENING_M2)
            focVars->vdqFilter = (focVars->vdqFilter * 7U + focVars->vdq) / 8;
            #endif  /* end of FLUX_WEAKENING_M2 */

        #elif (defined VF_STARTUP_M2)
            focVars->vd = focVars->vdRef;
            focVars->vq = focVars->vqRef;
            focVars->uOutputPu = MathFloat_Mag(focVars->vd, focVars->vq);

        #elif (defined HIGH_FREQUENCE_INJECTION_M2)
            HighFreqInj_Proc_M2(&g_hfiCtrl_M2, pHighFreqInjCfg_M2, &g_focVars_M2, &g_pidHfi_M2, &g_pidAsr_M2);
            if ((g_hfinj_M2.state == 0U) || (g_hfinj_M2.state == 2U))
            {
                focVars->vd = g_hfinj_M2.vsdRef;
                focVars->vq = 0;
            }
            else if ((g_hfinj_M2.state == 3U) || (g_hfinj_M2.state == 4U))
            {
                focVars->vd += g_hfinj_M2.vsdRef;
            }
            else
            {
                focVars->vd = 0;
                focVars->vq = 0;
            }
            focVars->vd = MathFloat_AutoSat(focVars->vd);
        #endif  /* end of IF_STARTUP_M2 */
    #endif  /* end of SENSORLESS_M2 */
#elif (defined FOC_ACIM_M2)
//            if (Get_McStatus_M2() == START)
//            {
//                focVars->idRef = g_acimFocCtrl.cur->curDRef;
//                focVars->iqRef = 0;
//            }
//            else
//            {
//                #if (defined SENSORLESS_ACIM)
//                if (g_startUp.pStCtrl->startUpFlag > 3)
//                {
////                    focVars->idRef = g_acimFocCtrl.cur->excitCurCalc;
//                    focVars->idRef = g_acimFocCtrl.mot->excitCur;
//                }
//                else
//                {
//                    focVars->idRef = g_acimFocCtrl.mot->excitCur;
//                }
//                #else
//                focVars->idRef = g_acimFocCtrl.mot->excitCur;
//                #endif
//            }

//            Set_PID_Ref(&g_pidAcrD, focVars->idRef);
//            Set_PID_Fbk(&g_pidAcrD, focVars->idFdb);
//            focVars->vd = PID_SeriesRegulator(&g_pidAcrD);

//            Set_PID_Ref(&g_pidAcrQ, focVars->iqRef);
//            Set_PID_Fbk(&g_pidAcrQ, focVars->iqFdb);
//            focVars->vq = PID_SeriesRegulator(&g_pidAcrQ);

//            Acim_CurrentLoopDecouple(&g_acimFocCtrl, focVars);

//            focVars->vd = focVars->vd + g_acimFocCtrl.volt->decoupleVoltD;
//            focVars->vq = focVars->vq + g_acimFocCtrl.volt->decoupleVoltQ;
//            Acr_Vdq_Limit_M2(focVars);
//            focVars->vdqLimit = Math_Q15Mag(focVars->vd, focVars->vq);
//            focVars->uOutputPu = focVars->vdqLimit;
#endif

            focVars->valpha = MathFloat_AutoSat(tempCos * focVars->vd - tempSin * focVars->vq);
            focVars->vbeta = MathFloat_AutoSat(tempSin * focVars->vd + tempCos * focVars->vq);
            }
        }
        else if (Get_McStatus_M2() == IDENTIFY)
        {
            if (g_paramIdentify_M2.pParamIdentifyCtrl->mainStep == IDENTIFY_RS_DCTEST)
            {
                Set_PID_Ref(&g_pidAcrAlfaRsId_M2, g_paramIdentify_M2.pParamRsIdentify->ialfaRef);
                Set_PID_Fbk(&g_pidAcrAlfaRsId_M2, focVars->ialpha);
                focVars->valpha = PID_SeriesRegulator(&g_pidAcrAlfaRsId_M2);

                if (g_paramIdentify_M2.pParamRsIdentify->phaseOrder != 0U)
                {
                    Set_PID_Ref(&g_pidAcrBetaRsId_M2, g_paramIdentify_M2.pParamRsIdentify->ibetaRef);
                    Set_PID_Fbk(&g_pidAcrBetaRsId_M2, focVars->ibeta);
                    focVars->vbeta = PID_SeriesRegulator(&g_pidAcrBetaRsId_M2);
                }
                else
                {
                    focVars->vbeta = 0;
                }
                focVars->uOutputPu = MathFloat_Mag(focVars->valpha, focVars->vbeta);
            }
            else if (g_paramIdentify_M2.pParamIdentifyCtrl->mainStep == (uint8_t)IDENTIFY_LDQ_STALLTEST)
            {
                #if (defined FOC_PMSM_M2)
                focVars->idFdb = MathFloat_AutoSat(tempCos * focVars->ialpha + tempSin * focVars->ibeta);
                focVars->iqFdb = MathFloat_AutoSat((-tempSin) * focVars->ialpha + tempCos * focVars->ibeta);
//                PmsmId_LdqIdCommondGen(&g_paramIdentify, &g_STC);
                PMSMLdqId_IsrTask_CommandGen(&g_paramIdentify_M2, &g_STC_M2);

                Set_PID_Ref(&g_pidAcrD_M2, focVars->idRef);
                Set_PID_Fbk(&g_pidAcrD_M2, focVars->idFdb);
                focVars->vd = PID_SeriesRegulator(&g_pidAcrD_M2);
                Set_PID_Ref(&g_pidAcrQ_M2, focVars->iqRef);
                Set_PID_Fbk(&g_pidAcrQ_M2, focVars->iqFdb);
                focVars->vq = PID_SeriesRegulator(&g_pidAcrQ_M2);

                focVars->valpha = MathFloat_AutoSat(tempCos * focVars->vd - tempSin * focVars->vq);
                focVars->vbeta = MathFloat_AutoSat(tempSin * focVars->vd + tempCos * focVars->vq);

                #elif (defined FOC_ACIM_M2)
//                AcimId_RotorIdSignalGeneration(&g_paramIdentify, &g_STC);
//                Set_PID_Ref(&g_pidAcrAlfaRsId, g_paramIdentify.pAcimParamIdentify->curAlfaSet);
//                Set_PID_Fbk(&g_pidAcrAlfaRsId, focVars->ialpha);
//                focVars->valpha = PID_SeriesRegulator(&g_pidAcrAlfaRsId);
//                focVars->vbeta = 0;
//                focVars->uOutputPu = MathFloat_Mag(focVars->valpha, focVars->vbeta);
                #else
                #endif  /* end of FOC_PMSM_M2 */
            }
            else if (g_paramIdentify_M2.pParamIdentifyCtrl->mainStep == IDENTIFY_FLUX_ROTATIONTEST)
            {
                #if (defined FOC_PMSM_M2)
                if (g_paramIdentify_M2.pParamFluxIdentify->subStep != 0)
                {
                    g_paramIdentify_M2.pParamIdentifyCtrl->setElecAngle += g_paramIdentify_M2.pParamFluxIdentify->fluxIdThetaDelta;
                }
                if (g_paramIdentify_M2.pParamIdentifyCtrl->setElecAngle > 1.0f)
                {
                    g_paramIdentify_M2.pParamIdentifyCtrl->setElecAngle -= 1.0f;
                }
                g_paramIdentify_M2.pParamFluxIdentify->fluxIdOutVolPu = MathFloat_Mag(focVars->vd, focVars->vq);
                g_paramIdentify_M2.pParamFluxIdentify->fluxIdOutCurPu = focVars->idFdb;

                focVars->idFdb = MathFloat_AutoSat(tempCos * focVars->ialpha + tempSin * focVars->ibeta);
                focVars->iqFdb = MathFloat_AutoSat((-tempSin) * focVars->ialpha + tempCos * focVars->ibeta);
                Set_PID_Ref(&g_pidAcrD_M2, focVars->idRef);
                Set_PID_Fbk(&g_pidAcrD_M2, focVars->idFdb);
                focVars->vd = PID_SeriesRegulator(&g_pidAcrD_M2);
                Set_PID_Ref(&g_pidAcrQ_M2, focVars->iqRef);
                Set_PID_Fbk(&g_pidAcrQ_M2, focVars->iqFdb);
                focVars->vq = PID_SeriesRegulator(&g_pidAcrQ_M2);

                focVars->valpha = MathFloat_AutoSat(tempCos * focVars->vd - tempSin * focVars->vq);
                focVars->vbeta = MathFloat_AutoSat(tempSin * focVars->vd + tempCos * focVars->vq);

                #elif (defined FOC_ACIM_M2)
//                AcimId_RotationVfControl(&g_paramIdentify, &g_STC);
//                focVars->uOutputPu = MathFloat_Mag(focVars->valpha, focVars->vbeta);

                #else
                #endif  /* end of FOC_PMSM_M2 */
            }
            else
            {
            }
        }
        else
        {
        }
        Svpwm_Process_M2();
        LoadToPwmModule_M2();
    }
    else
    {
        focVars->idRef = 0;
        focVars->iqRef = 0;
        focVars->idFdb = 0;
        focVars->iqFdb = 0;
        focVars->vd = 0;
        focVars->vq = 0;
        focVars->valpha = 0;
        focVars->vbeta = 0;
    }
#endif  /* end of MOTOR_TWO_TYPE */
}

/* =============================================  EOF  ============================================== */
