/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file pwmdrv.c
*
* @brief This file provides PWM module functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pwm_output.h"
#include "pwmdrv.h"
#include "globalvar.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Disable PWM module output pulse.
*
* @param[in] pwmModule: PWM module corresponding to the Motor
* @return none
*/
void PWM_Disable(uint8_t pwmModule)
{
    PWM_Output(pwmModule, PWM_OUTPUT_DISABLE);
}

/*!
* @brief Enable PWM module output pulse.
*
* @param[in] pwmModule: PWM module corresponding to the Motor
* @return none
*/
void PWM_Enable(uint8_t pwmModule)
{
    PWM_Output(pwmModule, PWM_OUTPUT_ENABLE);
}

/*!
* @brief Enable or disable PWM channel output according to the input mark.
*
* @param[in] pwmModule: PWM module corresponding to the Motor
* @param[in] enable: Enable mark, 'enable' = DISABLE = 0, will mask the PWM channel output; 'enable' = ENABLE = 1, will make the PWM channel output.
* @return none
*/
void PWM_Output(uint8_t pwmModule, uint8_t enable)
{
    FOC_VARS_CFG *focCfg;
    uint8_t maskStatus = !enable;
    
    switch (pwmModule)
    {
    case PWM_INDEX(PWM_MODULE):
        focCfg = pFocVarsCfg;
        PWM_DRV_SetChannelOutputMask(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), maskStatus);
        PWM_DRV_SetChannelOutputMask(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), maskStatus);
        PWM_DRV_SetChannelOutputMask(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), maskStatus);
        PWM_DRV_SetChannelOutputMask(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), maskStatus);
        PWM_DRV_SetChannelOutputMask(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), maskStatus);
        PWM_DRV_SetChannelOutputMask(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), maskStatus);
    break;

    case PWM_INDEX(PWM_MODULE_M2):
        focCfg = pFocVarsCfg_M2;
        PWM_DRV_SetChannelOutputMask(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), maskStatus);
        PWM_DRV_SetChannelOutputMask(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), maskStatus);
        PWM_DRV_SetChannelOutputMask(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), maskStatus);
        PWM_DRV_SetChannelOutputMask(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), maskStatus);
        PWM_DRV_SetChannelOutputMask(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), maskStatus);
        PWM_DRV_SetChannelOutputMask(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), maskStatus);
    break;
    
    default:
    break;
    }
}

/*!
* @brief Load the duty cycle of three-phase inverter calculated by SVPWM algorithm into PWM module.
*
* @param[in] Ta: duty of phase A upper bridge arm
* @param[in] Tb: duty of phase B upper bridge arm
* @param[in] Tc: duty of phase C upper bridge arm
* @param[in] focVars: pointer to FOC_VARS structure
* @return none
*/
void PWM_Update(uint16_t Ta, uint16_t Tb, uint16_t Tc, FOC_VARS *focVars)
{
    FOC_VARS_CFG *focCfg = focVars->pCfg;

#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), (focCfg->pwmPeriod - Ta));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), (focCfg->pwmPeriod - Ta));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), (focCfg->pwmPeriod - Tb));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), (focCfg->pwmPeriod - Tb));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), (focCfg->pwmPeriod - Tc));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), (focCfg->pwmPeriod - Tc));

    #else
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), ((focCfg->pwmPeriod - Ta) >> 1));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), ((focCfg->pwmPeriod + Ta) >> 1));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), ((focCfg->pwmPeriod - Tb) >> 1));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), ((focCfg->pwmPeriod + Tb) >> 1));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), ((focCfg->pwmPeriod - Tc) >> 1));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), ((focCfg->pwmPeriod + Tc) >> 1));

    #endif

    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), Ta);
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), Ta);
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), Tb);
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), Tb);
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), Tc);
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), Tc);

    #else
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), (Ta >> 1));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), (focCfg->pwmPeriod - (Ta >> 1));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), (Tb >> 1));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), (focCfg->pwmPeriod - (Tb >> 1));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), (Tc >> 1));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), (focCfg->pwmPeriod - (Tc >> 1));

    #endif
    #endif

#elif (defined GATE_DRIVER_HIGH_LOW)
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), (focCfg->pwmPeriod - Ta));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), (focCfg->pwmPeriod - Ta));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), (focCfg->pwmPeriod - Tb));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), (focCfg->pwmPeriod - Tb));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), (focCfg->pwmPeriod - Tc));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), (focCfg->pwmPeriod - Tc));

#elif (defined GATE_DRIVER_LOW_HIGH)
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), (focCfg->pwmPeriod - Ta));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), (focCfg->pwmPeriod - Ta));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), (focCfg->pwmPeriod - Tb));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), (focCfg->pwmPeriod - Tb));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), (focCfg->pwmPeriod - Tc));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), (focCfg->pwmPeriod - Tc));

#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), (focCfg->pwmPeriod - Ta));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), (focCfg->pwmPeriod - Ta));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), (focCfg->pwmPeriod - Tb));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), (focCfg->pwmPeriod - Tb));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), (focCfg->pwmPeriod - Tc));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), (focCfg->pwmPeriod - Tc));

    #else
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), ((focCfg->pwmPeriod - Ta) >> 1));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), ((focCfg->pwmPeriod + Ta) >> 1));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), ((focCfg->pwmPeriod - Tb) >> 1));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), ((focCfg->pwmPeriod + Tb) >> 1));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), ((focCfg->pwmPeriod - Tc) >> 1));
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), ((focCfg->pwmPeriod + Tc) >> 1));

    #endif
#endif

    PWM_DRV_SetSoftwareTriggerSync(focCfg->pwmId, ENABLE);
}

/*!
* @brief Load the duty cycle of three-phase inverter calculated by SVPWM
*        algorithm into PWM module when single shunt sampling is used.
*
* @param[in] curRestruct: pointer to CUR_RESTRUCT_DATA structure
* @param[in] focVars: pointer to FOC_VARS structure
* @return none
*/
void PWM_SingleShuntUpdate(CUR_RESTRUCT_DATA *curRestruct, FOC_VARS *focVars)
{
    FOC_VARS_CFG *focCfg = focVars->pCfg;
    /* if compare value equle to 1, will resul to pluse abnormal at high speed */
    curRestruct->pwmTaUp = Math_Sat(curRestruct->pwmTaUp, (focCfg->pwmPeriod - 1), 2);
    curRestruct->pwmTbUp = Math_Sat(curRestruct->pwmTbUp, (focCfg->pwmPeriod - 1), 2);
    curRestruct->pwmTcUp = Math_Sat(curRestruct->pwmTcUp, (focCfg->pwmPeriod - 1), 2);
    curRestruct->pwmTaDown = Math_Sat(curRestruct->pwmTaDown, (focCfg->pwmPeriod - 1), 2);
    curRestruct->pwmTbDown = Math_Sat(curRestruct->pwmTbDown, (focCfg->pwmPeriod - 1), 2);
    curRestruct->pwmTcDown = Math_Sat(curRestruct->pwmTcDown, (focCfg->pwmPeriod - 1), 2);
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), curRestruct->pwmTaUp);
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), curRestruct->pwmTaDown);
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), curRestruct->pwmTbUp);
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), curRestruct->pwmTbDown);
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), curRestruct->pwmTcUp);
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), curRestruct->pwmTcDown);
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelT), curRestruct->maxMidTrig);
    PWM_DRV_SetChannelCountValue(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelT + 1), curRestruct->midMinTrig);

    #else

    #endif
    PWM_DRV_SetSoftwareTriggerSync(focCfg->pwmId, ENABLE);
}

/*!
* @brief Turn on three phase lower MOSFET or IGBT to make motor winding in short circuit control.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @return none
*/
void PWM_ShortCircuitOn(FOC_VARS *focVars)
{
    PWM_DRV_SetChannelOutputMask(focVars->pCfg->pwmId, (pwm_channel_type_t)(2 * focVars->pCfg->pwmChannelU), ENABLE);
    PWM_DRV_SetChannelOutputMask(focVars->pCfg->pwmId, (pwm_channel_type_t)(2 * focVars->pCfg->pwmChannelU + 1), DISABLE);
    PWM_DRV_SetChannelOutputMask(focVars->pCfg->pwmId, (pwm_channel_type_t)(2 * focVars->pCfg->pwmChannelV), ENABLE);
    PWM_DRV_SetChannelOutputMask(focVars->pCfg->pwmId, (pwm_channel_type_t)(2 * focVars->pCfg->pwmChannelV + 1), DISABLE);
    PWM_DRV_SetChannelOutputMask(focVars->pCfg->pwmId, (pwm_channel_type_t)(2 * focVars->pCfg->pwmChannelW), ENABLE);
    PWM_DRV_SetChannelOutputMask(focVars->pCfg->pwmId, (pwm_channel_type_t)(2 * focVars->pCfg->pwmChannelW + 1), DISABLE);
}

/*!
* @brief Turn off three phase lower MOSFET or IGBT.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @return none
*/
void PWM_ShortCircuitOff(FOC_VARS *focVars)
{
    PWM_DRV_SetChannelOutputMask(focVars->pCfg->pwmId, (pwm_channel_type_t)(2 * focVars->pCfg->pwmChannelU), ENABLE);
    PWM_DRV_SetChannelOutputMask(focVars->pCfg->pwmId, (pwm_channel_type_t)(2 * focVars->pCfg->pwmChannelU + 1), ENABLE);
    PWM_DRV_SetChannelOutputMask(focVars->pCfg->pwmId, (pwm_channel_type_t)(2 * focVars->pCfg->pwmChannelV), ENABLE);
    PWM_DRV_SetChannelOutputMask(focVars->pCfg->pwmId, (pwm_channel_type_t)(2 * focVars->pCfg->pwmChannelV + 1), ENABLE);
    PWM_DRV_SetChannelOutputMask(focVars->pCfg->pwmId, (pwm_channel_type_t)(2 * focVars->pCfg->pwmChannelW), ENABLE);
    PWM_DRV_SetChannelOutputMask(focVars->pCfg->pwmId, (pwm_channel_type_t)(2 * focVars->pCfg->pwmChannelW + 1), ENABLE);
}

/* =============================================  EOF  ============================================== */
