/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file startup.c
*
* @brief This file provides sensorless foc open loop startup functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "globalvar.h"
#include "foc_app.h"
#include "motorctrl_api.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
/*!
* @brief Open loop startup curve time parameters.
*/
//CURVE_TIME_DURATION  g_startUpCurveTime;

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief StartUp parameter initialization.
*
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void InitStartUpParam(START_UP *startUp)
{
    startUp->pStCtrl->curStartOffset = startUp->pStCfg->curStartOffset + MathFloat_Mpy(0.01, startUp->pStCtrl->startUpTimes);

    return;
}

/*!
* @brief Clear startUp control counting parameter.
*
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void ClearStartUpParam(START_UP *startUp)
{
    startUp->pStCtrl->countStartUpST1 = 0;
    startUp->pStCtrl->countStartUpST1a = 0;
    startUp->pStCtrl->countStartUpST2 = 0;
    startUp->pStCtrl->countStartUpST3 = 0;
    startUp->pStCtrl->countStartUpST4 = 0;
    startUp->pStCtrl->countStartUpST5 = 0;

    return;
}

/*!
* @brief Open loop curve time variables deal, processing when time value is 0.
*
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUpCurveTimeInit(CURVE_TIME_DURATION *time, START_UP *startUp)
{
    uint16_t temp_timeValueUnit;

    temp_timeValueUnit = 1000 / 1000;
    if (startUp->pStCfg->curveTime1 >= temp_timeValueUnit)
    {
        time->timePoint1 = startUp->pStCfg->curveTime1;
    }
    else
    {
        time->timePoint1 = temp_timeValueUnit;
    }
    if (startUp->pStCfg->curveTime2 >= temp_timeValueUnit)
    {
        time->timePoint2 = time->timePoint1 + startUp->pStCfg->curveTime2;
    }
    else
    {
        time->timePoint2 = time->timePoint1 + temp_timeValueUnit;
    }
    if (startUp->pStCfg->curveTime3 >= temp_timeValueUnit)
    {
        time->timePoint3 = time->timePoint2 + startUp->pStCfg->curveTime3;
    }
    else
    {
        time->timePoint3 = time->timePoint2 + temp_timeValueUnit;
    }
    if (startUp->pStCfg->curveTime4 >= temp_timeValueUnit)
    {
        time->timePoint4 = time->timePoint3 + startUp->pStCfg->curveTime4;
    }
    else
    {
        time->timePoint4 = time->timePoint3 + temp_timeValueUnit;
    }
    if (startUp->pStCfg->curveTime5 >= temp_timeValueUnit)
    {
        time->timePoint5 = time->timePoint4 + startUp->pStCfg->curveTime5;
    }
    else
    {
        time->timePoint5 = time->timePoint4 + temp_timeValueUnit;
    }
}

/*!
* @brief Set startup curve reference value, contain start and end value, time duration of each curve segment.
*
* @param[in] stCurveIn: pointer to CURVE_IN structure
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @param[in] startUp: pointer to START_UP structure
* @param[in] stc: pointer to STC_PARAM structure
* @return none
*/
void StartUpCurveRefInit(CURVE_IN *stCurveIn, CURVE_TIME_DURATION *time, START_UP *startUp, STC_PARAM *stc)
{
    CURVE_REFVALUE *stCurveSeg1 = stCurveIn->curveSeg1;
    CURVE_REFVALUE *stCurveSeg2 = stCurveIn->curveSeg2;
    CURVE_REFVALUE *stCurveSeg3 = stCurveIn->curveSeg3;
    CURVE_REFVALUE *stCurveSeg4 = stCurveIn->curveSeg4;
    CURVE_REFVALUE *stCurveSeg5 = stCurveIn->curveSeg5;

    stCurveSeg1->curRefStart = startUp->pStCfg->curveCurStart;
    stCurveSeg1->freqRefStart = startUp->pStCfg->curveFreqStart;
    if (startUp->pStCfg->curveType == CURVE_TYPE_STRAIGHT)
    {
        stCurveSeg1->freqRefEnd = (float)startUp->pStCfg->synSpd * stc->pMotor->polePairs / stc->pFOCVars->pCfg->baseFreq;
        stCurveSeg1->curRefEnd = startUp->pStCfg->curStartOffset;
    }
    else if (startUp->pStCfg->curveType == CURVE_TYPE_CUSTOMER)
    {
        if (startUp->pStCfg->curveSegNum > 1)
        {
            stCurveSeg1->freqRefEnd = startUp->pStCfg->curveFreqSet1;
            stCurveSeg1->curRefEnd = startUp->pStCfg->curveCurSet1;
        }
        else
        {
            stCurveSeg1->freqRefEnd = (float)startUp->pStCfg->synSpd * stc->pMotor->polePairs / stc->pFOCVars->pCfg->baseFreq;
            stCurveSeg1->curRefEnd = startUp->pStCfg->curStartOffset;
        }
    }
    else
    {
    }
    stCurveSeg1->voltRefStart = startUp->pStCfg->curveVoltStart;
    stCurveSeg1->voltRefEnd = startUp->pStCfg->curveVoltSet1;
    stCurveSeg1->timeStart = 0;
    stCurveSeg1->timeEnd = time->timePoint1;

    stCurveSeg2->curRefStart = startUp->pStCfg->curveCurSet1;
    stCurveSeg2->curRefEnd = startUp->pStCfg->curveCurSet2;
    stCurveSeg2->freqRefStart = startUp->pStCfg->curveFreqSet1;
    stCurveSeg2->freqRefEnd = startUp->pStCfg->curveFreqSet2;
    stCurveSeg2->voltRefStart = startUp->pStCfg->curveVoltSet1;
    stCurveSeg2->voltRefEnd = startUp->pStCfg->curveVoltSet2;
    stCurveSeg2->timeStart = time->timePoint1;
    stCurveSeg2->timeEnd = time->timePoint2;

    stCurveSeg3->curRefStart = startUp->pStCfg->curveCurSet2;
    stCurveSeg3->curRefEnd = startUp->pStCfg->curveCurSet3;
    stCurveSeg3->freqRefStart = startUp->pStCfg->curveFreqSet2;
    stCurveSeg3->freqRefEnd = startUp->pStCfg->curveFreqSet3;
    stCurveSeg3->voltRefStart = startUp->pStCfg->curveVoltSet2;
    stCurveSeg3->voltRefEnd = startUp->pStCfg->curveVoltSet3;
    stCurveSeg3->timeStart = time->timePoint2;
    stCurveSeg3->timeEnd = time->timePoint3;

    stCurveSeg4->curRefStart = startUp->pStCfg->curveCurSet3;
    stCurveSeg4->curRefEnd = startUp->pStCfg->curveCurSet4;
    stCurveSeg4->freqRefStart = startUp->pStCfg->curveFreqSet3;
    stCurveSeg4->freqRefEnd = startUp->pStCfg->curveFreqSet4;
    stCurveSeg4->voltRefStart = startUp->pStCfg->curveVoltSet3;
    stCurveSeg4->voltRefEnd = startUp->pStCfg->curveVoltSet4;
    stCurveSeg4->timeStart = time->timePoint3;
    stCurveSeg4->timeEnd = time->timePoint4;

    stCurveSeg5->curRefStart = startUp->pStCfg->curveCurSet4;
    stCurveSeg5->curRefEnd = startUp->pStCfg->curveCurSet5;
    stCurveSeg5->freqRefStart = startUp->pStCfg->curveFreqSet4;
    stCurveSeg5->freqRefEnd = startUp->pStCfg->curveFreqSet5;
    stCurveSeg5->voltRefStart = startUp->pStCfg->curveVoltSet4;
    stCurveSeg5->voltRefEnd = startUp->pStCfg->curveVoltSet5;
    stCurveSeg5->timeStart = time->timePoint4;
    stCurveSeg5->timeEnd = time->timePoint5;
}

/*!
* @brief Startup curve parameters initilization, contain start and end value, time duration of each curve segment.
*
* @param[in] stCurveIn: pointer to CURVE_IN structure
* @param[in] stCurveOut: pointer to CURVE_OUT structure
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @param[in] startUp: pointer to START_UP structure
* @param[in] stc: pointer to STC_PARAM structure
* @return none
*/
void StartUpCurveParamInit(CURVE_IN *stCurveIn, CURVE_OUT *stCurveOut, CURVE_TIME_DURATION *time, START_UP *startUp, STC_PARAM *stc)
{
    stCurveIn->groupNum = 0;
    stCurveIn->refValue1 = 0;
    stCurveIn->refValue2 = 0;
    stCurveIn->timeDuration = 0;
    stCurveIn->timeEndValue = 0;
    stCurveIn->timeStartValue = 0;
    stCurveIn->timeValue = 0;

    memset(stCurveOut, 0, sizeof(CURVE_OUT));
    StartUpCurveTimeInit(time, startUp);

    StartUpCurveRefInit(stCurveIn, time, startUp, stc);
}

/*!
* @brief Motor startup control initialization.
*
* @param[in] startUp: pointer to START_UP structure
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] stc: pointer to STC_PARAM structure
* @return none
*/
void StartUp_Ctrl_Init(START_UP *startUp, STARTUP_CURVE *curve, STC_PARAM *stc)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_OUT *out = curve->curveOut;
    CURVE_TIME_DURATION *time = curve->curveTime;

    startUp->pStCtrl->startUpElecTheta = 0;
    startUp->pStCtrl->elecThetaEst = 0;

    startUp->pStCtrl->startUpTimes = 0;

    startUp->pStCtrl->synSpd = startUp->pStCfg->synSpd;
    startUp->pStCtrl->startUpFlag = 0;
    startUp->pStCtrl->synTime = startUp->pStCfg->synTime;
    startUp->pStCtrl->synFreReach = 0;
    startUp->pStCtrl->synCount = 0;
    startUp->pStCtrl->curStartSub = startUp->pStCfg->curStartSub;
    startUp->pStCtrl->curAddSlope1 = MathFloat_Div(startUp->pStCfg->alignCur - startUp->pStCfg->curRamp1Init, startUp->pStCfg->slopeDuration1);
    startUp->pStCtrl->curAddSlope2 = MathFloat_Div(startUp->pStCfg->alignCurTwice - startUp->pStCfg->curRamp2Init, startUp->pStCfg->slopeDuration2);
#if (defined SENSORLESS)
    startUp->pStCtrl->alignState = 0;
#elif (defined ENCODER_SENSORS)
    if(startUp->pStCtrl->alignState)
    {
        startUp->pStCtrl->alignState = 1;
    }
    else
    {
        startUp->pStCtrl->alignState = 0;
    }
#else
#endif

    ClearStartUpParam(startUp);
    InitStartUpParam(startUp);

    StartUpCurveParamInit(in, out, time, startUp, stc);
}

/*!
* @brief Motor 2 startup control initialization.
*
* @param[in] startUp: pointer to START_UP structure
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] stc: pointer to STC_PARAM structure
* @return none
*/
void StartUp_Ctrl_Init_M2(START_UP *startUp, STARTUP_CURVE *curve, STC_PARAM *stc)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_OUT *out = curve->curveOut;
    CURVE_TIME_DURATION *time = curve->curveTime;

    startUp->pStCtrl->startUpElecTheta = 0;
    startUp->pStCtrl->elecThetaEst = 0;

    startUp->pStCtrl->startUpTimes = 0;

    startUp->pStCtrl->synSpd = startUp->pStCfg->synSpd;
    startUp->pStCtrl->startUpFlag = 0;
    startUp->pStCtrl->synTime = startUp->pStCfg->synTime;
    startUp->pStCtrl->synFreReach = 0;
    startUp->pStCtrl->synCount = 0;
    startUp->pStCtrl->curStartSub = startUp->pStCfg->curStartSub;
    startUp->pStCtrl->curAddSlope1 = MathFloat_Div(startUp->pStCfg->alignCur - startUp->pStCfg->curRamp1Init, startUp->pStCfg->slopeDuration1);
    startUp->pStCtrl->curAddSlope2 = MathFloat_Div(startUp->pStCfg->alignCurTwice - startUp->pStCfg->curRamp2Init, startUp->pStCfg->slopeDuration2);
#if (defined SENSORLESS_M2)
    startUp->pStCtrl->alignState = 0;
#elif (defined ENCODER_SENSORS_M2)
    if(startUp->pStCtrl->alignState)
    {
        startUp->pStCtrl->alignState = 1;
    }
    else
    {
        startUp->pStCtrl->alignState = 0;
    }
#else
#endif

    ClearStartUpParam(startUp);
    InitStartUpParam(startUp);

    StartUpCurveParamInit(in, out, time, startUp, stc);
}

/*!
* @brief Bootstrap capacitor charging process.
*
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return none
*/
void Bootstrap_Charging_Process(STARTUP_CTRL *startUp)
{
    startUp->countStartUpST1 ++;

    if (startUp->countStartUpST1 <= 1)
    {
        PWM_Disable(PWM_MODULE_MOTOR1);
    }
    else if (startUp->countStartUpST1 <= 2)
    {
#if PHASE_UVW_POLARITY == ACTIVE_HIGH
        #if (defined MOTOR_ONE_TYPE)
        PWM_Update(0, 0, 0, &g_focVars);
        #endif
        #if (defined MOTOR_TWO_TYPE)
        PWM_Update(0, 0, 0, &g_focVars_M2);
        #endif
#elif PHASE_UVW_POLARITY == ACTIVE_LOW
        PWM_Update(PWM_PERIOD_VALUE, PWM_PERIOD_VALUE, PWM_PERIOD_VALUE, &g_focVars);
#endif
        PWM_Enable(PWM_MODULE_MOTOR1);
    }
    else if (startUp->countStartUpST1 <= 4)
    {

    }
    else
    {
        startUp->countStartUpST1 = 1;
        startUp->countStartUpST1a ++;
        if (startUp->countStartUpST1 > 20 * MS_UINT_INT)
        {
            startUp->countStartUpST1 = 0;
            startUp->countStartUpST1a = 0;

            PWM_Enable(PWM_MODULE_MOTOR1);
            startUp->startUpFlag = 2;
        }
    }
}

/*!
* @brief Tail wind and head wind state processes.
*
* @param[in] startUpCtrl: pointer to STARTUP_CTRL structure
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] speedHandle: pointer to SPEED_RAMP_HANDLE structure
* @return status: Motor operation status
*/
MC_STATUS Tail_Head_Wind_StateProcess(STARTUP_CTRL *startUpCtrl, FOC_VARS *focVars, SPEED_RAMP_HANDLE *speedHandle)
{
    MC_STATUS status;
#if 1
    switch (speedHandle->initStatus)
    {
    case UNKNOWN:
    status = SPEED_JUDGE;
    break;

    case HIGH_REVERSE:
    break;

    case LOW_REVERSE:
    status = START_BRAKE;
    Stop_Motor(startUpCtrl, focVars);
    break;

    case STATIC:
    status = START_BRAKE;//START;
    break;

    case LOW_FORWARD:
    status = START_BRAKE;
    Stop_Motor(startUpCtrl, focVars);
    break;

    case HIGH_FORWARD:
    break;

    default:
    status = START_BRAKE;
    Stop_Motor(startUpCtrl, focVars);
    break;
    }

    return status;
#endif
}

/*!
* @brief Tail wind and head wind processes.
         When the motor is in the HIGH_REVERSE state, it will enter the STOP state;
         when the motor is in the LOW_REVERSE state or LOW_FORWARD state, it will enter the START_BRAKE state;
         when the motor is in the STATIC state, it will enter the START state;
         when the motor is in the HIGH_FORWARD state, it will enter the RUN state;
*
* @param[in] startUpCtrl: pointer to STARTUP_CTRL structure
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] speedHandle: pointer to SPEED_RAMP_HANDLE structure
* @param[in] judgeHandle: pointer to INITSPEED_JUDGE structure
* @return status: Motor operation status
*/
MC_STATUS Tail_Head_Wind_Process(STARTUP_CTRL *startUpCtrl, FOC_VARS *focVars, SPEED_RAMP_HANDLE *speedHandle, INITSPEED_JUDGE *judgeHandle)
{
    MC_STATUS status;
#if (defined MOTOR_ONE_TYPE)
    static uint16_t s_flyRunCount = 0;
    static uint16_t s_highReverseCount = 0;

    speedHandle->initSpeedJudgeBemfAmp = Get_InitSpeed_Judge_BemfAmp(judgeHandle);
    speedHandle->initSpeedJudgeSpeed = Get_InitSpeed_Judge_Speed(judgeHandle);

    s_flyRunCount++;

    if (speedHandle->initStatus == HIGH_REVERSE)
    {
        s_flyRunCount = 0;
        s_highReverseCount++;

        /* 3000ms, if it is judged as HIGH_REVERSE, it will be judged again after 3000ms time delay */
        if (s_highReverseCount <= 500)
        {
            speedHandle->initStatus = HIGH_REVERSE;
        }
        else
        {
            s_highReverseCount = 0;
            speedHandle->initStatus = UNKNOWN;
            status = Tail_Head_Wind_StateProcess(startUpCtrl, focVars, speedHandle);
        }
    }
    else
    {
        if (s_flyRunCount == 1)
        {
            if (speedHandle->initSpeedJudgeBemfAmp > MOTOR_MOTION_BEMF_AMP_THRESHOLD)
            {
                if (speedHandle->initSpeedJudgeSpeed < 0)
                {
                    if (speedHandle->initSpeedJudgeSpeed <= HIGH_REVERSE_THRESHOLD)
                    {
                        speedHandle->initStatus = HIGH_REVERSE;
                    }
                    else if ((speedHandle->initSpeedJudgeSpeed > HIGH_REVERSE_THRESHOLD) \
                        && (speedHandle->initSpeedJudgeSpeed < -STATIC_THRESHOLD))
                    {
                        speedHandle->initStatus  = LOW_REVERSE;
                    }
                    else
                    {
                        speedHandle->initStatus  = STATIC;
                    }
                }
                else
                {
                    if ((speedHandle->initSpeedJudgeSpeed > STATIC_THRESHOLD) \
                        && (speedHandle->initSpeedJudgeSpeed < HIGH_FORWARD_THRESHOLD))
                    {
                        speedHandle->initStatus  = LOW_FORWARD;
                    }
                    else if (speedHandle->initSpeedJudgeSpeed >= HIGH_FORWARD_THRESHOLD)
                    {
                        speedHandle->initStatus  = HIGH_FORWARD;
                    }
                    else
                    {
                        speedHandle->initStatus = STATIC;
                    }
                }
            }
            else
            {
                speedHandle->initStatus = STATIC;
            }

            status = Tail_Head_Wind_StateProcess(startUpCtrl, focVars, speedHandle);
            s_flyRunCount = 0;

        }
        else
        {
        }
    }
#endif
    return status;
}

/*!
* @brief Pre_positioning motor to an expected angle.
*
* @param[in] focVarsCtrl: pointer to FOC_VARS_CTRL structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void Angle_Prepositioning_Process(FOC_VARS *focVars, START_UP *startUp)
{
    startUp->pStCtrl->alignCnt++;
    if (startUp->pStCtrl->alignCnt <= startUp->pStCfg->alignTime)
    {
        startUp->pStCtrl->startUpElecTheta = startUp->pStCfg->alignAngle;
        focVars->pCtrl->elecAngle = startUp->pStCtrl->startUpElecTheta;
        focVars->pCtrl->idRef = 0;
        if (startUp->pStCtrl->alignCnt <= startUp->pStCfg->slopeDuration1)
        {
            focVars->pCtrl->iqRef = (startUp->pStCfg->curRamp1Init + MathFloat_Mpy(startUp->pStCtrl->alignCnt, startUp->pStCtrl->curAddSlope1)) * focVars->pCfg->motorDir;
        }
        else
        {
            focVars->pCtrl->iqRef = startUp->pStCfg->alignCur * focVars->pCfg->motorDir;
        }
    }
    else
    {
#if (defined MOTOR_POLES_OBTAIN)
        startUp->pStCtrl->startUpFlag = 2;
#else
        startUp->pStCtrl->alignCnt = 0;
        startUp->pStCtrl->alignState = 1;
        startUp->pStCtrl->startUpFlag = 3;
#endif
    }
}

/*!
* @brief Pre_positioning motor 2 to an expected angle.
*
* @param[in] focVarsCtrl: pointer to FOC_VARS structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void Angle_Prepositioning_Process_M2(FOC_VARS *focVars, START_UP *startUp)
{
    startUp->pStCtrl->alignCnt++;
    if (startUp->pStCtrl->alignCnt <= startUp->pStCfg->alignTime)
    {
        startUp->pStCtrl->startUpElecTheta = startUp->pStCfg->alignAngle;
        focVars->pCtrl->elecAngle = startUp->pStCtrl->startUpElecTheta;
        focVars->pCtrl->idRef = 0;
        if (startUp->pStCtrl->alignCnt <= startUp->pStCfg->slopeDuration1)
        {
            focVars->pCtrl->iqRef = (startUp->pStCfg->curRamp1Init + MathFloat_Mpy(startUp->pStCtrl->alignCnt, startUp->pStCtrl->curAddSlope1)) * focVars->pCfg->motorDir;
        }
        else
        {
            focVars->pCtrl->iqRef = startUp->pStCfg->alignCur * focVars->pCfg->motorDir;
        }
    }
    else
    {
#if (defined MOTOR_POLES_OBTAIN_M2)
        startUp->pStCtrl->startUpFlag = 2;
#else
        startUp->pStCtrl->alignCnt = 0;
        startUp->pStCtrl->alignState = 1;
        startUp->pStCtrl->startUpFlag = 3;
#endif
    }
}

/*!
* @brief Angle alignment process to prepare for motor open loop operation.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void Angle_Alignment_Process(FOC_VARS *focVars, START_UP *startUp)
{
    startUp->pStCtrl->alignCnt++;
    if (startUp->pStCtrl->alignCnt <= startUp->pStCfg->alignTime)
    {
        startUp->pStCtrl->startUpElecTheta = F_180_DEGREE;
        focVars->pCtrl->elecAngle = startUp->pStCtrl->startUpElecTheta;
        focVars->pCtrl->idRef = 0;
        if (startUp->pStCtrl->alignCnt <= startUp->pStCfg->slopeDuration1)
        {
            focVars->pCtrl->iqRef = (startUp->pStCfg->curRamp1Init + MathFloat_Mpy(startUp->pStCtrl->alignCnt, startUp->pStCtrl->curAddSlope1)) * focVars->pCfg->motorDir;
        }
        else
        {
            focVars->pCtrl->iqRef = startUp->pStCfg->alignCur * focVars->pCfg->motorDir;
        }
    }
    else if (startUp->pStCtrl->alignCnt <= startUp->pStCfg->alignTime + startUp->pStCfg->alignTimeTwice)
    {
        startUp->pStCtrl->startUpElecTheta = F_270_DEGREE;
        focVars->pCtrl->elecAngle = startUp->pStCtrl->startUpElecTheta;
        focVars->pCtrl->idRef = 0;
        if (startUp->pStCtrl->alignCnt <= startUp->pStCfg->alignTime + startUp->pStCfg->slopeDuration2)
        {
            focVars->pCtrl->iqRef = (startUp->pStCfg->curRamp2Init + MathFloat_Mpy(startUp->pStCtrl->alignCnt - startUp->pStCfg->alignTime, startUp->pStCtrl->curAddSlope2)) * focVars->pCfg->motorDir;
        }
        else
        {
            focVars->pCtrl->iqRef = startUp->pStCfg->alignCurTwice * focVars->pCfg->motorDir;
        }
    }
    else
    {
#if (defined MOTOR_POLES_OBTAIN)
        startUp->pStCtrl->startUpFlag = 2;
#else
        startUp->pStCtrl->alignCnt = 0;
        startUp->pStCtrl->alignState = 1;
        startUp->pStCtrl->startUpFlag = 3;
#endif
    }
}

/*!
* @brief Angle alignment process to prepare for motor 2 open loop operation.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void Angle_Alignment_Process_M2(FOC_VARS *focVars, START_UP *startUp)
{
    startUp->pStCtrl->alignCnt++;
    if (startUp->pStCtrl->alignCnt <= startUp->pStCfg->alignTime)
    {
        startUp->pStCtrl->startUpElecTheta = F_180_DEGREE;
        focVars->pCtrl->elecAngle = startUp->pStCtrl->startUpElecTheta;
        focVars->pCtrl->idRef = 0;
        if (startUp->pStCtrl->alignCnt <= startUp->pStCfg->slopeDuration1)
        {
            focVars->pCtrl->iqRef = (startUp->pStCfg->curRamp1Init + MathFloat_Mpy(startUp->pStCtrl->alignCnt, startUp->pStCtrl->curAddSlope1)) * focVars->pCfg->motorDir;
        }
        else
        {
            focVars->pCtrl->iqRef = startUp->pStCfg->alignCur * focVars->pCfg->motorDir;
        }
    }
    else if (startUp->pStCtrl->alignCnt <= startUp->pStCfg->alignTime + startUp->pStCfg->alignTimeTwice)
    {
        startUp->pStCtrl->startUpElecTheta = F_270_DEGREE;
        focVars->pCtrl->elecAngle = startUp->pStCtrl->startUpElecTheta;
        focVars->pCtrl->idRef = 0;
        if (startUp->pStCtrl->alignCnt <= startUp->pStCfg->alignTime + startUp->pStCfg->slopeDuration2)
        {
            focVars->pCtrl->iqRef = (startUp->pStCfg->curRamp2Init + MathFloat_Mpy(startUp->pStCtrl->alignCnt - startUp->pStCfg->alignTime, startUp->pStCtrl->curAddSlope2)) * focVars->pCfg->motorDir;
        }
        else
        {
            focVars->pCtrl->iqRef = startUp->pStCfg->alignCurTwice * focVars->pCfg->motorDir;
        }
    }
    else
    {
#if (defined MOTOR_POLES_OBTAIN_M2)
        startUp->pStCtrl->startUpFlag = 2;
#else
        startUp->pStCtrl->alignCnt = 0;
        startUp->pStCtrl->alignState = 1;
        startUp->pStCtrl->startUpFlag = 3;
#endif
    }
}

/*!
* @brief Make the open angle of the operation close to the observer angle.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return none
*/
void Angle_Switching_Process(FOC_VARS *focVars, STARTUP_CTRL *startUp)
{
    float elecThetaErr = 0;

    if (focVars->pCfg->motorDir == 1U)
    {
        elecThetaErr = startUp->elecThetaEst - startUp->startUpElecTheta;
    }
    else if (focVars->pCfg->motorDir == -1)
    {
        elecThetaErr = startUp->startUpElecTheta - startUp->elecThetaEst;
    }
    else
    {
    }

    if (1 == startUp->synFreReach)
    {
        if (elecThetaErr >= F_1_DEGREE)
        {
            if (focVars->pCfg->motorDir == 1U)
            {
                if (startUp->startUpElecTheta < startUp->elecThetaEst)
                {
                    if (++(startUp->thetaCompTimes) <= 1024U)
                    {
                        startUp->startUpElecTheta = ((float)(startUp->elecThetaEst * startUp->thetaCompTimes \
                                                  + startUp->startUpElecTheta * (1024U - (startUp->thetaCompTimes))) / 1024.0f);
                        if (startUp->startUpElecTheta > 1.0f)
                        {
                            startUp->startUpElecTheta -= 1.0f;
                        }
                        else if (startUp->startUpElecTheta < 0)
                        {
                            startUp->startUpElecTheta += 1.0f;
                        }
                        else
                        {
                        }
                    }
                    else
                    {
                        startUp->countStartUpST3 = 0U;
                    #if (defined OPEN_LOOP)
                        startUp->startUpFlag = 3U;
                    #elif (defined CLOSE_LOOP)
                        startUp->startUpFlag = 4U;
                    #endif
                        startUp->startUpElecTheta = startUp->elecThetaEst;
                        startUp->thetaCompTimes = 0U;
                    }
                }
                else
                {
                }
            }
            else if (focVars->pCfg->motorDir == -1)
            {
                if (startUp->startUpElecTheta > startUp->elecThetaEst)
                {
                    if (++(startUp->thetaCompTimes) <= 1024U)
                    {
                        startUp->startUpElecTheta = ((float)(startUp->elecThetaEst * startUp->thetaCompTimes \
                                                  + startUp->startUpElecTheta * (1024U - (startUp->thetaCompTimes))) / 1024.0f);
                        if (startUp->startUpElecTheta > 1.0f)
                        {
                            startUp->startUpElecTheta -= 1.0f;
                        }
                        else if (startUp->startUpElecTheta < 0)
                        {
                            startUp->startUpElecTheta += 1.0f;
                        }
                        else
                        {
                        }
                    }
                    else
                    {
                        startUp->countStartUpST3 = 0U;
                    #if (defined OPEN_LOOP)
                        startUp->startUpFlag = 3U;
                    #elif (defined CLOSE_LOOP)
                        startUp->startUpFlag = 4U;
                    #endif

                        startUp->startUpElecTheta = startUp->elecThetaEst;
                        startUp->thetaCompTimes = 0U;
                    }
                }
            }
            else
            {
            }
        }
        else if ((elecThetaErr < F_1_DEGREE) && (elecThetaErr > 0))
        {
            startUp->countStartUpST3 = 0;
#if (defined OPEN_LOOP)
            startUp->startUpFlag = 3;           /*!< If runs in OPEN_LOOP, do not run the follow-up process */
#elif (defined CLOSE_LOOP)
            startUp->startUpFlag = 4;
#endif

            startUp->startUpElecTheta = startUp->elecThetaEst;
            startUp->thetaCompTimes = 0;
        }
        else
        {
        }
    }
}

/*!
* @brief Make the Motor 2 open angle of the operation close to the observer angle.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return none
*/
void Angle_Switching_Process_M2(FOC_VARS *focVars, STARTUP_CTRL *startUp)
{
    float elecThetaErr = 0;

    if (focVars->pCfg->motorDir == 1U)
    {
        elecThetaErr = startUp->elecThetaEst - startUp->startUpElecTheta;
    }
    else if (focVars->pCfg->motorDir == -1)
    {
        elecThetaErr = startUp->startUpElecTheta - startUp->elecThetaEst;
    }
    else
    {
    }

    if (1 == startUp->synFreReach)
    {
        if (elecThetaErr >= F_1_DEGREE)
        {
            if (focVars->pCfg->motorDir == 1U)
            {
                if (startUp->startUpElecTheta < startUp->elecThetaEst)
                {
                    if (++(startUp->thetaCompTimes) <= 1024U)
                    {
                        startUp->startUpElecTheta = ((float)(startUp->elecThetaEst * startUp->thetaCompTimes \
                                                  + startUp->startUpElecTheta * (1024U - (startUp->thetaCompTimes))) / 1024.0f);
                        if (startUp->startUpElecTheta > 1.0f)
                        {
                            startUp->startUpElecTheta -= 1.0f;
                        }
                        else if (startUp->startUpElecTheta < 0)
                        {
                            startUp->startUpElecTheta += 1.0f;
                        }
                        else
                        {
                        }
                    }
                    else
                    {
                        startUp->countStartUpST3 = 0U;
                    #if (defined OPEN_LOOP)
                        startUp->startUpFlag = 3U;
                    #elif (defined CLOSE_LOOP)
                        startUp->startUpFlag = 4U;
                    #endif
                        startUp->startUpElecTheta = startUp->elecThetaEst;
                        startUp->thetaCompTimes = 0U;
                    }
                }
                else
                {
                }
            }
            else if (focVars->pCfg->motorDir == -1)
            {
                if (startUp->startUpElecTheta > startUp->elecThetaEst)
                {
                    if (++(startUp->thetaCompTimes) <= 1024U)
                    {
                        startUp->startUpElecTheta = ((float)(startUp->elecThetaEst * startUp->thetaCompTimes \
                                                  + startUp->startUpElecTheta * (1024U - (startUp->thetaCompTimes))) / 1024.0f);
                        if (startUp->startUpElecTheta > 1.0f)
                        {
                            startUp->startUpElecTheta -= 1.0f;
                        }
                        else if (startUp->startUpElecTheta < 0)
                        {
                            startUp->startUpElecTheta += 1.0f;
                        }
                        else
                        {
                        }
                    }
                    else
                    {
                        startUp->countStartUpST3 = 0U;
                    #if (defined OPEN_LOOP)
                        startUp->startUpFlag = 3U;
                    #elif (defined CLOSE_LOOP)
                        startUp->startUpFlag = 4U;
                    #endif

                        startUp->startUpElecTheta = startUp->elecThetaEst;
                        startUp->thetaCompTimes = 0U;
                    }
                }
            }
            else
            {
            }
        }
        else if ((elecThetaErr < F_1_DEGREE) && (elecThetaErr > 0))
        {
            startUp->countStartUpST3 = 0;
#if (defined OPEN_LOOP_M2)
            startUp->startUpFlag = 3;           /*!< If runs in OPEN_LOOP, do not run the follow-up process */
#elif (defined CLOSE_LOOP_M2)
            startUp->startUpFlag = 4;
#endif

            startUp->startUpElecTheta = startUp->elecThetaEst;
            startUp->thetaCompTimes = 0;
        }
        else
        {
        }
    }
}

/*!
* @brief Open loop acceleration process to make the motor run to a certain speed.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @param[in] speed: pointer to ESTIMATED_SPEED structure
* @param[in] pll: pointer to PID_REGULATOR structure
* @param[in] stCurve: pointer to STARTUP_CURVE structure
* @return none
*/
void Openloop_Acceleration_Process(STC_PARAM *stc, START_UP *startUp, ESTIMATED_SPEED *speed, PID_REGULATOR *pll, STARTUP_CURVE *stCurve)
{
#if (defined FOC_PMSM)
    /* Normal StartUp Mode */
#if (defined DIRECT_STARTUP)
    FOC_VARS *focVars = stc->pFOCVars;
    STARTUP_CTRL *startUpCtrl = startUp->pStCtrl;

    #if (defined SMC_OBSERVE)
    startUp->elecThetaEst = Smc_PositionEstimator(stc, speed, pll);
    #elif (defined FLUX_OBSERVE)
    startUp->elecThetaEst = FluxObserver_PosEst(stc, speed, pll);
    #elif (defined MRAS_OBSERVE)
    startUp->elecThetaEst = Mras_PosEst(stc, pll, startUpCtrl->startUpFlag);
    #else
    /* Do nothing */
    #endif
    startUpCtrl->startUpElecTheta = startUpCtrl->elecThetaEst;
    startUpCtrl->countStartUpST3 ++;

    if (startUpCtrl->countStartUpST3 < 100 * MS_UINT_INT)
    {
        focVars->pCtrl->idRef = 0;
        focVars->pCtrl->iqRef = startUp->pStCfg->curStartOffset;
    }
    else
    {
        startUpCtrl->countStartUpST3 = 0;
        #if (defined OPEN_LOOP)
        startUpCtrl->startUpFlag = 3;
        #elif (defined CLOSE_LOOP)
        startUpCtrl->startUpFlag = 4;
        #endif
    }

    /* If StartUp Mode */
#elif (defined IF_STARTUP)
    #if (defined CLOSE_LOOP)
    FOC_VARS *focVars = stc->pFOCVars;
    #endif
    STARTUP_CTRL *startUpCtrl = startUp->pStCtrl;

    #if (defined SMC_OBSERVE)
    startUpCtrl->elecThetaEst = Smc_PositionEstimator(stc, speed, pll);
    #elif (defined FLUX_OBSERVE)
    startUpCtrl->elecThetaEst = FluxObserver_PosEst(stc, speed, pll);
    #elif (defined MRAS_OBSERVE)
    startUpCtrl->elecThetaEst = Mras_PosEst(stc, pll, startUpCtrl->startUpFlag);
    #else
    /* Do nothing */
    #endif

    #if (defined CLOSE_LOOP)
    Angle_Switching_Process(focVars, startUpCtrl);
    #endif

    /* Vf StartUp Mode */
#elif (defined VF_STARTUP)
    FOC_VARS *focVars = stc->pFOCVars;
    STARTUP_CTRL *startUpCtrl = startUp->pStCtrl;

    #if (defined SMC_OBSERVE)
    startUpCtrl->elecThetaEst = Smc_PositionEstimator(stc, speed, pll);
    #elif (defined FLUX_OBSERVE)
    startUpCtrl->elecThetaEst = FluxObserver_PosEst(stc, speed, pll);
    #elif (defined MRAS_OBSERVE)
    startUpCtrl->elecThetaEst = Mras_PosEst(stc, pll, startUpCtrl->startUpFlag);
    #else
    #endif
    focVars->pCtrl->idRef = 0;

    focVars->pCtrl->vdRef = 0;
    focVars->pCtrl->vqRef = (int32_t)stCurve->curveOut->voltRefValue * focVars->pCfg->motorDir;

    #if (defined CLOSE_LOOP)
    Angle_Switching_Process(focVars, startUpCtrl);
    #endif
#endif

#elif (defined FOC_ACIM)
//    Acim_FluxObserverCalc(&g_acimFocCtrl, focVars, &g_pidPll, ACIM_FO_METHOD_2);
//    Acim_FluxObserverAngleCalc(&g_acimFocCtrl, &g_STC);

//    startUp->elecThetaEst = g_acimFocCtrl.flux->rotorFluxAngle;

//    #if (defined CLOSE_LOOP)
//    Angle_Switching_Process(focVars, startUp);
//    #endif
#endif
}

/*!
* @brief Open loop acceleration process to make the motor 2 run to a certain speed.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @param[in] speed: pointer to ESTIMATED_SPEED structure
* @param[in] pll: pointer to PID_REGULATOR structure
* @param[in] stCurve: pointer to STARTUP_CURVE structure
* @return none
*/
void Openloop_Acceleration_Process_M2(STC_PARAM *stc, START_UP *startUp, ESTIMATED_SPEED *speed, PID_REGULATOR *pll, STARTUP_CURVE *stCurve)
{
#if (defined FOC_PMSM_M2)
    /* Normal StartUp Mode */
#if (defined DIRECT_STARTUP_M2)
    FOC_VARS *focVars = stc->pFOCVars;
    STARTUP_CTRL *startUpCtrl = startUp->pStCtrl;

    #if (defined SMC_OBSERVE_M2)
    startUp->elecThetaEst = Smc_PositionEstimator(stc, speed, pll);
    #elif (defined FLUX_OBSERVE_M2)
    startUp->elecThetaEst = FluxObserver_PosEst(stc, speed, pll);
    #elif (defined MRAS_OBSERVE_M2)
    startUp->elecThetaEst = Mras_PosEst(stc, pll, startUpCtrl->startUpFlag);
    #else
    /* Do nothing */
    #endif
    startUpCtrl->startUpElecTheta = startUpCtrl->elecThetaEst;
    startUpCtrl->countStartUpST3 ++;

    if (startUpCtrl->countStartUpST3 < 100 * MS_UINT_INT)
    {
        focVars->pCtrl->idRef = 0;
        focVars->pCtrl->iqRef = startUp->pStCfg->curStartOffset;
    }
    else
    {
        startUpCtrl->countStartUpST3 = 0;
        #if (defined OPEN_LOOP_M2)
        startUpCtrl->startUpFlag = 3;
        #elif (defined CLOSE_LOOP_M2)
        startUpCtrl->startUpFlag = 4;
        #endif
    }

    /* If StartUp Mode */
#elif (defined IF_STARTUP_M2)
    #if (defined CLOSE_LOOP_M2)
    FOC_VARS *focVars = stc->pFOCVars;
    #endif
    STARTUP_CTRL *startUpCtrl = startUp->pStCtrl;

    #if (defined SMC_OBSERVE_M2)
    startUpCtrl->elecThetaEst = Smc_PositionEstimator(stc, speed, pll);
    #elif (defined FLUX_OBSERVE_M2)
    startUpCtrl->elecThetaEst = FluxObserver_PosEst(stc, speed, pll);
    #elif (defined MRAS_OBSERVE_M2)
    startUpCtrl->elecThetaEst = Mras_PosEst(stc, pll, startUpCtrl->startUpFlag);
    #else
    /* Do nothing */
    #endif

    #if (defined CLOSE_LOOP_M2)
    Angle_Switching_Process_M2(focVars, startUpCtrl);
    #endif

    /* Vf StartUp Mode */
#elif (defined VF_STARTUP_M2)
    FOC_VARS *focVars = stc->pFOCVars;
    STARTUP_CTRL *startUpCtrl = startUp->pStCtrl;

    #if (defined SMC_OBSERVE_M2)
    startUpCtrl->elecThetaEst = Smc_PositionEstimator(stc, speed, pll);
    #elif (defined FLUX_OBSERVE_M2)
    startUpCtrl->elecThetaEst = FluxObserver_PosEst(stc, speed, pll);
    #elif (defined MRAS_OBSERVE_M2)
    startUpCtrl->elecThetaEst = Mras_PosEst(stc, pll, startUpCtrl->startUpFlag);
    #else
    #endif
    focVars->pCtrl->idRef = 0;

    focVars->pCtrl->vdRef = 0;
    focVars->pCtrl->vqRef = (int32_t)stCurve->curveOut->voltRefValue * focVars->pCfg->motorDir;

    #if (defined CLOSE_LOOP_M2)
    Angle_Switching_Process_M2(focVars, startUpCtrl);
    #endif
#endif

#elif (defined FOC_ACIM_M2)
//    Acim_FluxObserverCalc(&g_acimFocCtrl, focVars, &g_pidPll, ACIM_FO_METHOD_2);
//    Acim_FluxObserverAngleCalc(&g_acimFocCtrl, &g_STC);

//    startUp->elecThetaEst = g_acimFocCtrl.flux->rotorFluxAngle;

//    #if (defined CLOSE_LOOP_M2)
//    Angle_Switching_Process_M2(focVars, startUp);
//    #endif
#endif
}

/*!
* @brief The closed-loop deceleration process allows the motor to smoothly cut from the open loop to the closed loop state.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @param[in] speed: pointer to ESTIMATED_SPEED structure
* @param[in] pll: pointer to PID_REGULATOR structure
* @param[in] asr: pointer to PID_REGULATOR structure
* @return none
*/
void Closedloop_Deceleration_Process(STC_PARAM *stc, START_UP *startUp, ESTIMATED_SPEED *speed, PID_REGULATOR *pll, PID_REGULATOR *asr)
{
#if (defined FOC_PMSM)
    FOC_VARS *focVars = stc->pFOCVars;
    PID_REGULATOR_OUT *asrOut = asr->pPidOutput;

    #if (defined SMC_OBSERVE)
    startUp->pStCtrl->elecThetaEst = Smc_PositionEstimator(stc, speed, pll);
    #elif (defined FLUX_OBSERVE)
    startUp->pStCtrl->elecThetaEst = FluxObserver_PosEst(stc, speed, pll);
    #elif (defined MRAS_OBSERVE)
    startUp->pStCtrl->elecThetaEst = Mras_PosEst(stc, pll, startUp->pStCtrl->startUpFlag);
    #else
    /* Do nothing */
    #endif

    startUp->pStCtrl->startUpElecTheta = startUp->pStCtrl->elecThetaEst;
    startUp->pStCtrl->countStartUpST4++;
    startUp->pStCtrl->countStartUpST4 = 10 * MS_UINT_INT;
    if (startUp->pStCtrl->countStartUpST4 < 10 * MS_UINT_INT)       /*!< 10ms */
    {
        if (focVars->pCtrl->iqRef > 0)
        {
            focVars->pCtrl->iqRef -= startUp->pStCtrl->curStartSub;
        }
        else
        {
            focVars->pCtrl->iqRef = 0;
        }
    }
    else
    {
        startUp->pStCtrl->startUpFlag = 5;
        startUp->pStCtrl->countStartUpST4 = 0;

        /* smoothly cut from the open loop to the closed loop state */
        Asr_SetSpeedPID_Integral(asrOut, (focVars->pCfg->motorDir * startUp->pStCfg->spdSmoothCur));
    }
#elif (defined FOC_ACIM)
//    Acim_FluxObserverCalc(&g_acimFocCtrl, focVars, &g_pidPll, ACIM_FO_METHOD_2);
//    Acim_FluxObserverAngleCalc(&g_acimFocCtrl, &g_STC);

//    startUp->pStCtrl->elecThetaEst = g_acimFocCtrl.flux->rotorFluxAngle;

//    startUp->pStCtrl->startUpElecTheta = startUp->pStCtrl->elecThetaEst;
//    startUp->pStCtrl->countStartUpST4++;
//    if (startUp->pStCtrl->countStartUpST4 < 10 * MS_UINT_INT)       /*!< 10ms */
//    {
//        if (focVars->pCfg->motorDir == 1)
//        {
//            if (focVars->pCtrl->iqRef > g_pidAsr.pPidOutput->outPu)
//            {
//                focVars->pCtrl->iqRef -= startUp->pStCtrl->curStartSub;
//            }
//            else
//            {
//                focVars->pCtrl->iqRef = g_pidAsr.pPidOutput->outPu;
//            }
//        }
//        else
//        {
//            if (focVars->pCtrl->iqRef < g_pidAsr.pPidOutput->outPu)
//            {
//                focVars->pCtrl->iqRef += startUp->pStCtrl->curStartSub;
//            }
//            else
//            {
//                focVars->pCtrl->iqRef = g_pidAsr.pPidOutput->outPu;
//            }
//        }
//    }
//    else
//    {
//        startUp->pStCtrl->startUpFlag = 5;
//        startUp->pStCtrl->countStartUpST4 = 0;
//    }
#endif
}

/*!
* @brief The closed-loop deceleration process allows the motor 2 to smoothly cut from the open loop to the closed loop state.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @param[in] speed: pointer to ESTIMATED_SPEED structure
* @param[in] pll: pointer to PID_REGULATOR structure
* @param[in] asr: pointer to PID_REGULATOR structure
* @return none
*/
void Closedloop_Deceleration_Process_M2(STC_PARAM *stc, START_UP *startUp, ESTIMATED_SPEED *speed, PID_REGULATOR *pll, PID_REGULATOR *asr)
{
#if (defined FOC_PMSM_M2)
    FOC_VARS *focVars = stc->pFOCVars;
    PID_REGULATOR_OUT *asrOut = asr->pPidOutput;

    #if (defined SMC_OBSERVE_M2)
    startUp->pStCtrl->elecThetaEst = Smc_PositionEstimator(stc, speed, pll);
    #elif (defined FLUX_OBSERVE_M2)
    startUp->pStCtrl->elecThetaEst = FluxObserver_PosEst(stc, speed, pll);
    #elif (defined MRAS_OBSERVE_M2)
    startUp->pStCtrl->elecThetaEst = Mras_PosEst(stc, pll, startUp->pStCtrl->startUpFlag);
    #else
    /* Do nothing */
    #endif

    startUp->pStCtrl->startUpElecTheta = startUp->pStCtrl->elecThetaEst;
    startUp->pStCtrl->countStartUpST4++;
    startUp->pStCtrl->countStartUpST4 = 10 * MS_UINT_INT;
    if (startUp->pStCtrl->countStartUpST4 < 10 * MS_UINT_INT)       /*!< 10ms */
    {
        if (focVars->pCtrl->iqRef > 0)
        {
            focVars->pCtrl->iqRef -= startUp->pStCtrl->curStartSub;
        }
        else
        {
            focVars->pCtrl->iqRef = 0;
        }
    }
    else
    {
        startUp->pStCtrl->startUpFlag = 5;
        startUp->pStCtrl->countStartUpST4 = 0;

        /* smoothly cut from the open loop to the closed loop state */
        Asr_SetSpeedPID_Integral(asrOut, (focVars->pCfg->motorDir * startUp->pStCfg->spdSmoothCur));
    }
#elif (defined FOC_ACIM_M2)
//    Acim_FluxObserverCalc(&g_acimFocCtrl, focVars, &g_pidPll, ACIM_FO_METHOD_2);
//    Acim_FluxObserverAngleCalc(&g_acimFocCtrl, &g_STC);

//    startUp->pStCtrl->elecThetaEst = g_acimFocCtrl.flux->rotorFluxAngle;

//    startUp->pStCtrl->startUpElecTheta = startUp->pStCtrl->elecThetaEst;
//    startUp->pStCtrl->countStartUpST4++;
//    if (startUp->pStCtrl->countStartUpST4 < 10 * MS_UINT_INT)       /*!< 10ms */
//    {
//        if (focVars->pCfg->motorDir == 1)
//        {
//            if (focVars->pCtrl->iqRef > g_pidAsr.pPidOutput->outPu)
//            {
//                focVars->pCtrl->iqRef -= startUp->pStCtrl->curStartSub;
//            }
//            else
//            {
//                focVars->pCtrl->iqRef = g_pidAsr.pPidOutput->outPu;
//            }
//        }
//        else
//        {
//            if (focVars->pCtrl->iqRef < g_pidAsr.pPidOutput->outPu)
//            {
//                focVars->pCtrl->iqRef += startUp->pStCtrl->curStartSub;
//            }
//            else
//            {
//                focVars->pCtrl->iqRef = g_pidAsr.pPidOutput->outPu;
//            }
//        }
//    }
//    else
//    {
//        startUp->pStCtrl->startUpFlag = 5;
//        startUp->pStCtrl->countStartUpST4 = 0;
//    }
#endif
}

/*!
* @brief The closed loop process runs the motor using the observer angle.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @param[in] speed: pointer to ESTIMATED_SPEED structure
* @param[in] pll: pointer to PID_REGULATOR structure
* @return none
*/
void Closedloop_Process(STC_PARAM *stc, START_UP *startUp, ESTIMATED_SPEED *speed, PID_REGULATOR *pll)
{
    STARTUP_CTRL *startUpCtrl = startUp->pStCtrl;

#if (defined FOC_PMSM)
    #if (defined SMC_OBSERVE)
    startUpCtrl->elecThetaEst = Smc_PositionEstimator(stc, speed, pll);
    #elif (defined FLUX_OBSERVE)
    startUpCtrl->elecThetaEst = FluxObserver_PosEst(stc, speed, pll);
    #elif (defined MRAS_OBSERVE)
    startUpCtrl->elecThetaEst = Mras_PosEst(stc, pll, startUpCtrl->startUpFlag);
    #else
    /* Do nothing */
#endif
#elif (defined FOC_ACIM)
//    Acim_FluxObserverCalc(&g_acimFocCtrl, focVars, &g_pidPll, ACIM_FO_METHOD_2);
//    Acim_FluxObserverAngleCalc(&g_acimFocCtrl, &g_STC);

//    startUpCtrl->elecThetaEst = g_acimFocCtrl.flux->rotorFluxAngle;
#endif

    startUpCtrl->startUpElecTheta = startUpCtrl->elecThetaEst;
}

/*!
* @brief The closed loop process runs the motor 2 using the observer angle.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @param[in] speed: pointer to ESTIMATED_SPEED structure
* @param[in] pll: pointer to PID_REGULATOR structure
* @return none
*/
void Closedloop_Process_M2(STC_PARAM *stc, START_UP *startUp, ESTIMATED_SPEED *speed, PID_REGULATOR *pll)
{
    STARTUP_CTRL *startUpCtrl = startUp->pStCtrl;

#if (defined FOC_PMSM_M2)
    #if (defined SMC_OBSERVE_M2)
    startUpCtrl->elecThetaEst = Smc_PositionEstimator(stc, speed, pll);
    #elif (defined FLUX_OBSERVE_M2)
    startUpCtrl->elecThetaEst = FluxObserver_PosEst(stc, speed, pll);
    #elif (defined MRAS_OBSERVE_M2)
    startUpCtrl->elecThetaEst = Mras_PosEst(stc, pll, startUpCtrl->startUpFlag);
    #else
    /* Do nothing */
#endif
#elif (defined FOC_ACIM_M2)
//    Acim_FluxObserverCalc(&g_acimFocCtrl, focVars, &g_pidPll, ACIM_FO_METHOD_2);
//    Acim_FluxObserverAngleCalc(&g_acimFocCtrl, &g_STC);

//    startUpCtrl->elecThetaEst = g_acimFocCtrl.flux->rotorFluxAngle;
#endif

    startUpCtrl->startUpElecTheta = startUpCtrl->elecThetaEst;
}

/*!
* @brief The closed loop process switch to open loop.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @param[in] speed: pointer to ESTIMATED_SPEED structure
* @param[in] pll: pointer to PID_REGULATOR structure
* @return none
*/
void CloseToOpenloop_Process(STC_PARAM *stc, START_UP *startUp, ESTIMATED_SPEED *speed, PID_REGULATOR *pll)
{
#if (defined MOTOR_ONE_TYPE)
    STARTUP_CTRL *startUpCtrl = startUp->pStCtrl;

    #if (defined SMC_OBSERVE)
    startUpCtrl->elecThetaEst = Smc_PositionEstimator(stc, speed, pll);
    #elif (defined FLUX_OBSERVE)
    startUpCtrl->elecThetaEst = FluxObserver_PosEst(stc, speed, pll);
    #elif (defined MRAS_OBSERVE)
    startUpCtrl->elecThetaEst = Mras_PosEst(stc, pll, startUpCtrl->startUpFlag);
    #else
    /* Do nothing */
    #endif
#endif
}

/*!
* @brief The closed loop process switch to open loop of Motor 2.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @param[in] speed: pointer to ESTIMATED_SPEED structure
* @param[in] pll: pointer to PID_REGULATOR structure
* @return none
*/
void CloseToOpenloop_Process_M2(STC_PARAM *stc, START_UP *startUp, ESTIMATED_SPEED *speed, PID_REGULATOR *pll)
{
#if (defined MOTOR_TWO_TYPE)
    STARTUP_CTRL *startUpCtrl = startUp->pStCtrl;

    #if (defined SMC_OBSERVE_M2)
    startUpCtrl->elecThetaEst = Smc_PositionEstimator(stc, speed, pll);
    #elif (defined FLUX_OBSERVE_M2)
    startUpCtrl->elecThetaEst = FluxObserver_PosEst(stc, speed, pll);
    #elif (defined MRAS_OBSERVE_M2)
    startUpCtrl->elecThetaEst = Mras_PosEst(stc, pll, startUpCtrl->startUpFlag);
    #else
    /* Do nothing */
    #endif
#endif
}

/*!
* @brief Open loop process switch to close loop process.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @param[in] speed: pointer to ESTIMATED_SPEED structure
* @param[in] pll: pointer to PID_REGULATOR structure
* @return none
*/
void OpenToCloseloop_Process(STC_PARAM *stc, START_UP *startUp, ESTIMATED_SPEED *speed, PID_REGULATOR *pll)
{
    /* If StartUp Mode */
#if (defined IF_STARTUP)
    #if (defined CLOSE_LOOP)
    FOC_VARS *focVars = stc->pFOCVars;
    #endif
    STARTUP_CTRL *startUpCtrl = startUp->pStCtrl;

    #if (defined SMC_OBSERVE)
    startUpCtrl->elecThetaEst = Smc_PositionEstimator(stc, speed, pll);
    #elif (defined FLUX_OBSERVE)
    startUpCtrl->elecThetaEst = FluxObserver_PosEst(stc, speed, pll);
    #elif (defined MRAS_OBSERVE)
    startUpCtrl->elecThetaEst = Mras_PosEst(stc, pll, startUpCtrl->startUpFlag);
    #else
    /* Do nothing */
    #endif

    #if (defined CLOSE_LOOP)
    Angle_Switching_Process(focVars, startUpCtrl);
    #endif
#endif
}

/*!
* @brief Open loop process switch to close loop process of Motor 2.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @param[in] speed: pointer to ESTIMATED_SPEED structure
* @param[in] pll: pointer to PID_REGULATOR structure
* @return none
*/
void OpenToCloseloop_Process_M2(STC_PARAM *stc, START_UP *startUp, ESTIMATED_SPEED *speed, PID_REGULATOR *pll)
{
    /* If StartUp Mode */
#if (defined IF_STARTUP_M2)
    #if (defined CLOSE_LOOP_M2)
    FOC_VARS *focVars = stc->pFOCVars;
    #endif
    STARTUP_CTRL *startUpCtrl = startUp->pStCtrl;

    #if (defined SMC_OBSERVE_M2)
    startUpCtrl->elecThetaEst = Smc_PositionEstimator(stc, speed, pll);
    #elif (defined FLUX_OBSERVE_M2)
    startUpCtrl->elecThetaEst = FluxObserver_PosEst(stc, speed, pll);
    #elif (defined MRAS_OBSERVE_M2)
    startUpCtrl->elecThetaEst = Mras_PosEst(stc, pll, startUpCtrl->startUpFlag);
    #else
    /* Do nothing */
    #endif

    #if (defined CLOSE_LOOP_M2)
    Angle_Switching_Process(focVars, startUpCtrl);
    #endif
#endif
}

/*!
* @brief Calculate curve output with unsigned type parameters.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @return none
*/
void GetCurveRefValue(STARTUP_CURVE *curve)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_OUT *out = curve->curveOut;

    in->timeDuration = in->timeEndValue - in->timeStartValue;
    if (0 != in->timeDuration)
    {
        if (in->refValue2 > in->refValue1)
        {
            out->refCalcValue = ((float)(in->refValue2 - in->refValue1) * in->timeValue \
                                + ((float)in->refValue1 * in->timeEndValue) \
                                - ((float)in->refValue2 * in->timeStartValue)) / in->timeDuration;
        }
        else
        {
            out->refCalcValue = (((float)in->refValue1 * in->timeEndValue) \
                                - ((float)(in->refValue1 - in->refValue2) * in->timeValue) \
                                - ((float)in->refValue2 * in->timeStartValue)) / in->timeDuration;
        }
    }
    else
    {
        out->refCalcValue = in->refValue2;
    }
}

/*!
* @brief Open loop startup curve straight(default) mode.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_Straight(STARTUP_CURVE *curve, START_UP *startUp)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_OUT *out = curve->curveOut;
    CURVE_TIME_DURATION *time = curve->curveTime;

    if (0 == out->calcEndFlag)
    {
        if (++(in->timeValue) <= time->timePoint1)
        {
            in->timeStartValue = 0;
            in->timeEndValue = time->timePoint1;

            in->refValue1 = startUp->pStCfg->curveFreqStart;
            in->refValue2 = in->curveSeg1->freqRefEnd;

            GetCurveRefValue(curve);
            out->frqRefValue = out->refCalcValue;

            #if (defined IF_STARTUP)
            in->refValue1 = startUp->pStCfg->curveCurStart;
            in->refValue2 = startUp->pStCfg->curveCurSet1;

            GetCurveRefValue(curve);
            out->curRefValue = out->refCalcValue;
            #elif (defined VF_STARTUP)
            in->refValue1 = startUp->pStCfg->curveVoltStart;
            in->refValue2 = startUp->pStCfg->curveVoltSet1;

            GetCurveRefValue(curve);
            out->voltRefValue = out->refCalcValue;
            #endif

            out->calcEndFlag = 0;
        }
        else
        {
            in->timeValue = 0;
            out->calcEndFlag = 1;
        }
    }
    else
    {
    }
}

/*!
* @brief Open loop startup curve straight(default) mode of Motor 2.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_Straight_M2(STARTUP_CURVE *curve, START_UP *startUp)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_OUT *out = curve->curveOut;
    CURVE_TIME_DURATION *time = curve->curveTime;

    if (0 == out->calcEndFlag)
    {
        if (++(in->timeValue) <= time->timePoint1)
        {
            in->timeStartValue = 0;
            in->timeEndValue = time->timePoint1;

            in->refValue1 = startUp->pStCfg->curveFreqStart;
            in->refValue2 = in->curveSeg1->freqRefEnd;

            GetCurveRefValue(curve);
            out->frqRefValue = out->refCalcValue;

            #if (defined IF_STARTUP_M2)
            in->refValue1 = startUp->pStCfg->curveCurStart;
            in->refValue2 = startUp->pStCfg->curveCurSet1;

            GetCurveRefValue(curve);
            out->curRefValue = out->refCalcValue;
            #elif (defined VF_STARTUP_M2)
            in->refValue1 = startUp->pStCfg->curveVoltStart;
            in->refValue2 = startUp->pStCfg->curveVoltSet1;

            GetCurveRefValue(curve);
            out->voltRefValue = out->refCalcValue;
            #endif

            out->calcEndFlag = 0;
        }
        else
        {
            in->timeValue = 0;
            out->calcEndFlag = 1;
        }
    }
    else
    {
    }
}

/*!
* @brief Open loop startup curve straight(default) mode, used in close loop to open loop switch.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_Straight1(STARTUP_CURVE *curve, START_UP *startUp)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_OUT *out = curve->curveOut;
    CURVE_TIME_DURATION *time = curve->curveTime;

    if (0 == out->calcEndFlag1)
    {
        if (++(in->timeValue) <= (time->timePoint1))
        {
            in->timeStartValue = 0;
            in->timeEndValue = time->timePoint1;

            in->refValue1 = startUp->pStCfg->curveFreqStart1;
            in->refValue2 = in->curveSeg1->freqRefEnd;

            GetCurveRefValue(curve);
            out->frqRefValue = out->refCalcValue;

            #if (defined IF_STARTUP)
            out->curRefValue = startUp->pStCfg->curStartOffset;
            #elif (defined VF_STARTUP)
            in->refValue1 = startUp->pStCfg->curveVoltStart;
            in->refValue2 = startUp->pStCfg->curveVoltSet1;

            GetCurveRefValue(curve);
            out->voltRefValue = out->refCalcValue;
            #endif

            out->calcEndFlag1 = 0;
        }
        else
        {
            in->timeValue = 0;
            out->calcEndFlag1 = 1;
        }
    }
    else
    {
    }
}

/*!
* @brief Open loop startup curve straight(default) mode of Motor 2, used in close loop to open loop switch.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_Straight1_M2(STARTUP_CURVE *curve, START_UP *startUp)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_OUT *out = curve->curveOut;
    CURVE_TIME_DURATION *time = curve->curveTime;

    if (0 == out->calcEndFlag1)
    {
        if (++(in->timeValue) <= (time->timePoint1))
        {
            in->timeStartValue = 0;
            in->timeEndValue = time->timePoint1;

            in->refValue1 = startUp->pStCfg->curveFreqStart1;
            in->refValue2 = in->curveSeg1->freqRefEnd;

            GetCurveRefValue(curve);
            out->frqRefValue = out->refCalcValue;

            #if (defined IF_STARTUP_M2)
            out->curRefValue = startUp->pStCfg->curStartOffset;
            #elif (defined VF_STARTUP_M2)
            in->refValue1 = startUp->pStCfg->curveVoltStart;
            in->refValue2 = startUp->pStCfg->curveVoltSet1;

            GetCurveRefValue(curve);
            out->voltRefValue = out->refCalcValue;
            #endif

            out->calcEndFlag1 = 0;
        }
        else
        {
            in->timeValue = 0;
            out->calcEndFlag1 = 1;
        }
    }
    else
    {
    }
}

/*!
* @brief Open loop startup curve straight(default) mode, used in open loop to close loop switch.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] startUp: pointer to START_UP structure
* @param[in] focVars: pointer to FOC_VARS structure
* @return none
*/
void StartUp_Straight2(STARTUP_CURVE *curve, START_UP *startUp, FOC_VARS *focVars)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_OUT *out = curve->curveOut;
    CURVE_TIME_DURATION *time = curve->curveTime;

    if (0 == out->calcEndFlag2)
    {
        if (++(in->timeValue) <= time->timePoint1)
        {
            in->timeStartValue = 0;
            in->timeEndValue = time->timePoint1;

            in->refValue1 = focVars->pCfg->speedMinLimit;
            in->refValue2 = in->curveSeg1->freqRefEnd;

            GetCurveRefValue(curve);
            out->frqRefValue = out->refCalcValue;
            out->curRefValue = startUp->pStCfg->curveCurSet1;
            out->calcEndFlag2 = 0;
        }
        else
        {
            in->timeValue = 0;
            out->calcEndFlag2 = 1;
        }
    }
    else
    {
    }
}

/*!
* @brief Segment curve calculation unit block.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] ref: pointer to CURVE_REFVALUE structure
* @return none
*/
void StartUp_SegmentCurve_Calc(STARTUP_CURVE *curve, CURVE_REFVALUE *ref)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_OUT *out = curve->curveOut;

    in->timeStartValue = ref->timeStart;
    in->timeEndValue = ref->timeEnd;

    in->refValue1 = ref->freqRefStart;
    in->refValue2 = ref->freqRefEnd;

    GetCurveRefValue(curve);
    out->frqRefValue = out->refCalcValue;

    #if (defined IF_STARTUP)
    in->refValue1 = ref->curRefStart;
    in->refValue2 = ref->curRefEnd;

    GetCurveRefValue(curve);
    out->curRefValue = out->refCalcValue;
    #elif (defined VF_STARTUP)
    in->refValue1 = ref->voltRefStart;
    in->refValue2 = ref->voltRefEnd;

    GetCurveRefValue(curve);
    out->voltRefValue = out->refCalcValue;
    #endif

    out->calcEndFlag = 0;
}

/*!
* @brief Segment curve calculation unit block of Motor 2.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] ref: pointer to CURVE_REFVALUE structure
* @return none
*/
void StartUp_SegmentCurve_Calc_M2(STARTUP_CURVE *curve, CURVE_REFVALUE *ref)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_OUT *out = curve->curveOut;

    in->timeStartValue = ref->timeStart;
    in->timeEndValue = ref->timeEnd;

    in->refValue1 = ref->freqRefStart;
    in->refValue2 = ref->freqRefEnd;

    GetCurveRefValue(curve);
    out->frqRefValue = out->refCalcValue;

    #if (defined IF_STARTUP_M2)
    in->refValue1 = ref->curRefStart;
    in->refValue2 = ref->curRefEnd;

    GetCurveRefValue(curve);
    out->curRefValue = out->refCalcValue;
    #elif (defined VF_STARTUP_M2)
    in->refValue1 = ref->voltRefStart;
    in->refValue2 = ref->voltRefEnd;

    GetCurveRefValue(curve);
    out->voltRefValue = out->refCalcValue;
    #endif

    out->calcEndFlag = 0;
}

/*!
* @brief When segment curve calculation finished, call this function.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] ref: pointer to CURVE_REFVALUE structure
* @return none
*/
void StartUp_SegmentCurve_CalcEnd(STARTUP_CURVE *curve, CURVE_REFVALUE *ref)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_OUT *out = curve->curveOut;

    out->frqRefValue = ref->freqRefEnd;
    #if (defined IF_STARTUP)
    out->curRefValue = ref->curRefEnd;
    #elif (defined VF_STARTUP)
    out->voltRefValue = ref->voltRefEnd;
    #endif
    in->timeValue = 0;
    out->calcEndFlag = 1;
}

/*!
* @brief When segment curve calculation finished, call this function.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] ref: pointer to CURVE_REFVALUE structure
* @return none
*/
void StartUp_SegmentCurve_CalcEnd_M2(STARTUP_CURVE *curve, CURVE_REFVALUE *ref)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_OUT *out = curve->curveOut;

    out->frqRefValue = ref->freqRefEnd;
    #if (defined IF_STARTUP_M2)
    out->curRefValue = ref->curRefEnd;
    #elif (defined VF_STARTUP_M2)
    out->voltRefValue = ref->voltRefEnd;
    #endif
    in->timeValue = 0;
    out->calcEndFlag = 1;
}

/*!
* @brief The default output of curve calculation.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_SegmentCurve_Default(STARTUP_CURVE *curve, START_UP *startUp)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_OUT *out = curve->curveOut;

    out->frqRefValue = startUp->pStCfg->curveFreqStart;
    #if (defined IF_STARTUP)
    out->curRefValue = startUp->pStCfg->curveCurStart;
    #elif (defined VF_STARTUP)
    out->voltRefValue = startUp->pStCfg->curveVoltStart;
    #endif
    in->timeValue = 0;
    out->calcEndFlag = 1;
}

/*!
* @brief The default output of curve calculation of Motor 2.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_SegmentCurve_Default_M2(STARTUP_CURVE *curve, START_UP *startUp)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_OUT *out = curve->curveOut;

    out->frqRefValue = startUp->pStCfg->curveFreqStart;
    #if (defined IF_STARTUP_M2)
    out->curRefValue = startUp->pStCfg->curveCurStart;
    #elif (defined VF_STARTUP_M2)
    out->voltRefValue = startUp->pStCfg->curveVoltStart;
    #endif
    in->timeValue = 0;
    out->calcEndFlag = 1;
}

/*!
* @brief When segment curve has 1 point, call this function.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @return none
*/
void StartUp_SegmentCurve_Part1(STARTUP_CURVE *curve)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_TIME_DURATION *time = curve->curveTime;

    in->timeValue++;
    if (in->timeValue <= time->timePoint1)
    {
        StartUp_SegmentCurve_Calc(curve, in->curveSeg1);
    }
    else
    {
        StartUp_SegmentCurve_CalcEnd(curve, in->curveSeg1);
    }
}

/*!
* @brief When segment curve has 2 point, call this function.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @return none
*/
void StartUp_SegmentCurve_Part2(STARTUP_CURVE *curve)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_TIME_DURATION *time = curve->curveTime;

    in->timeValue++;
    if (in->timeValue <= time->timePoint1)
    {
        StartUp_SegmentCurve_Calc(curve, in->curveSeg1);
    }
    else if ((in->timeValue > time->timePoint1) && (in->timeValue <= time->timePoint2))
    {
        StartUp_SegmentCurve_Calc(curve, in->curveSeg2);
    }
    else
    {
        StartUp_SegmentCurve_CalcEnd(curve, in->curveSeg2);
    }
}

/*!
* @brief When segment curve has 3 point, call this function.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @return none
*/
void StartUp_SegmentCurve_Part3(STARTUP_CURVE *curve)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_TIME_DURATION *time = curve->curveTime;

    in->timeValue++;
    if (in->timeValue <= time->timePoint1)
    {
        StartUp_SegmentCurve_Calc(curve, in->curveSeg1);
    }
    else if ((in->timeValue > time->timePoint1) && (in->timeValue <= time->timePoint2))
    {
        StartUp_SegmentCurve_Calc(curve, in->curveSeg2);
    }
    else if ((in->timeValue > time->timePoint2) && (in->timeValue <= time->timePoint3))
    {
        StartUp_SegmentCurve_Calc(curve, in->curveSeg3);
    }
    else
    {
        StartUp_SegmentCurve_CalcEnd(curve, in->curveSeg3);
    }
}

/*!
* @brief When segment curve has 4 point, call this function.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @return none
*/
void StartUp_SegmentCurve_Part4(STARTUP_CURVE *curve)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_TIME_DURATION *time = curve->curveTime;

    in->timeValue++;
    if (in->timeValue <= time->timePoint1)
    {
        StartUp_SegmentCurve_Calc(curve, in->curveSeg1);
    }
    else if ((in->timeValue > time->timePoint1) && (in->timeValue <= time->timePoint2))
    {
        StartUp_SegmentCurve_Calc(curve, in->curveSeg2);
    }
    else if ((in->timeValue > time->timePoint2) && (in->timeValue <= time->timePoint3))
    {
        StartUp_SegmentCurve_Calc(curve, in->curveSeg3);
    }
    else if ((in->timeValue > time->timePoint3) && (in->timeValue <= time->timePoint4))
    {
        StartUp_SegmentCurve_Calc(curve, in->curveSeg4);
    }
    else
    {
        StartUp_SegmentCurve_CalcEnd(curve, in->curveSeg4);
    }
}

/*!
* @brief When segment curve has 5 point, call this function.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @return none
*/
void StartUp_SegmentCurve_Part5(STARTUP_CURVE *curve)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_TIME_DURATION *time = curve->curveTime;

    in->timeValue++;
    if (in->timeValue <= time->timePoint1)
    {
        StartUp_SegmentCurve_Calc(curve, in->curveSeg1);
    }
    else if ((in->timeValue > time->timePoint1) && (in->timeValue <= time->timePoint2))
    {
        StartUp_SegmentCurve_Calc(curve, in->curveSeg2);
    }
    else if ((in->timeValue > time->timePoint2) && (in->timeValue <= time->timePoint3))
    {
        StartUp_SegmentCurve_Calc(curve, in->curveSeg3);
    }
    else if ((in->timeValue > time->timePoint3) && (in->timeValue <= time->timePoint4))
    {
        StartUp_SegmentCurve_Calc(curve, in->curveSeg4);
    }
    else if ((in->timeValue > time->timePoint4) && (in->timeValue <= time->timePoint5))
    {
        StartUp_SegmentCurve_Calc(curve, in->curveSeg5);
    }
    else
    {
        StartUp_SegmentCurve_CalcEnd(curve, in->curveSeg5);
    }
}

/*!
* @brief Open loop startup curve customer mode.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_Customer(STARTUP_CURVE *curve, START_UP *startUp)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_OUT *out = curve->curveOut;

    /* curve segment number, configed by host tool */
    in->groupNum = startUp->pStCfg->curveSegNum;

    if (0 == out->calcEndFlag)
    {
        switch (in->groupNum)
        {
        case 0:
            StartUp_SegmentCurve_Default(curve, startUp);
        break;

        case 1:
            StartUp_SegmentCurve_Part1(curve);
        break;

        case 2:
            StartUp_SegmentCurve_Part2(curve);
        break;

        case 3:
            StartUp_SegmentCurve_Part3(curve);
        break;

        case 4:
            StartUp_SegmentCurve_Part4(curve);
        break;

        case 5:
            StartUp_SegmentCurve_Part5(curve);
        break;

        default:
            StartUp_SegmentCurve_Default(curve, startUp);
        break;
        }
    }
    else
    {
    }
}

/*!
* @brief When segment curve has 1 point, call this function.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @return none
*/
void StartUp_SegmentCurve_Part1_M2(STARTUP_CURVE *curve)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_TIME_DURATION *time = curve->curveTime;

    in->timeValue++;
    if (in->timeValue <= time->timePoint1)
    {
        StartUp_SegmentCurve_Calc_M2(curve, in->curveSeg1);
    }
    else
    {
        StartUp_SegmentCurve_CalcEnd_M2(curve, in->curveSeg1);
    }
}

/*!
* @brief When segment curve has 2 point, call this function.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @return none
*/
void StartUp_SegmentCurve_Part2_M2(STARTUP_CURVE *curve)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_TIME_DURATION *time = curve->curveTime;

    in->timeValue++;
    if (in->timeValue <= time->timePoint1)
    {
        StartUp_SegmentCurve_Calc_M2(curve, in->curveSeg1);
    }
    else if ((in->timeValue > time->timePoint1) && (in->timeValue <= time->timePoint2))
    {
        StartUp_SegmentCurve_Calc_M2(curve, in->curveSeg2);
    }
    else
    {
        StartUp_SegmentCurve_CalcEnd_M2(curve, in->curveSeg2);
    }
}

/*!
* @brief When segment curve has 3 point, call this function.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @return none
*/
void StartUp_SegmentCurve_Part3_M2(STARTUP_CURVE *curve)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_TIME_DURATION *time = curve->curveTime;

    in->timeValue++;
    if (in->timeValue <= time->timePoint1)
    {
        StartUp_SegmentCurve_Calc_M2(curve, in->curveSeg1);
    }
    else if ((in->timeValue > time->timePoint1) && (in->timeValue <= time->timePoint2))
    {
        StartUp_SegmentCurve_Calc_M2(curve, in->curveSeg2);
    }
    else if ((in->timeValue > time->timePoint2) && (in->timeValue <= time->timePoint3))
    {
        StartUp_SegmentCurve_Calc_M2(curve, in->curveSeg3);
    }
    else
    {
        StartUp_SegmentCurve_CalcEnd_M2(curve, in->curveSeg3);
    }
}

/*!
* @brief When segment curve has 4 point, call this function.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @return none
*/
void StartUp_SegmentCurve_Part4_M2(STARTUP_CURVE *curve)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_TIME_DURATION *time = curve->curveTime;

    in->timeValue++;
    if (in->timeValue <= time->timePoint1)
    {
        StartUp_SegmentCurve_Calc_M2(curve, in->curveSeg1);
    }
    else if ((in->timeValue > time->timePoint1) && (in->timeValue <= time->timePoint2))
    {
        StartUp_SegmentCurve_Calc_M2(curve, in->curveSeg2);
    }
    else if ((in->timeValue > time->timePoint2) && (in->timeValue <= time->timePoint3))
    {
        StartUp_SegmentCurve_Calc_M2(curve, in->curveSeg3);
    }
    else if ((in->timeValue > time->timePoint3) && (in->timeValue <= time->timePoint4))
    {
        StartUp_SegmentCurve_Calc_M2(curve, in->curveSeg4);
    }
    else
    {
        StartUp_SegmentCurve_CalcEnd_M2(curve, in->curveSeg4);
    }
}

/*!
* @brief When segment curve has 5 point, call this function.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @return none
*/
void StartUp_SegmentCurve_Part5_M2(STARTUP_CURVE *curve)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_TIME_DURATION *time = curve->curveTime;

    in->timeValue++;
    if (in->timeValue <= time->timePoint1)
    {
        StartUp_SegmentCurve_Calc_M2(curve, in->curveSeg1);
    }
    else if ((in->timeValue > time->timePoint1) && (in->timeValue <= time->timePoint2))
    {
        StartUp_SegmentCurve_Calc_M2(curve, in->curveSeg2);
    }
    else if ((in->timeValue > time->timePoint2) && (in->timeValue <= time->timePoint3))
    {
        StartUp_SegmentCurve_Calc_M2(curve, in->curveSeg3);
    }
    else if ((in->timeValue > time->timePoint3) && (in->timeValue <= time->timePoint4))
    {
        StartUp_SegmentCurve_Calc_M2(curve, in->curveSeg4);
    }
    else if ((in->timeValue > time->timePoint4) && (in->timeValue <= time->timePoint5))
    {
        StartUp_SegmentCurve_Calc_M2(curve, in->curveSeg5);
    }
    else
    {
        StartUp_SegmentCurve_CalcEnd_M2(curve, in->curveSeg5);
    }
}

/*!
* @brief Open loop startup curve customer mode of Motor 2.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_Customer_M2(STARTUP_CURVE *curve, START_UP *startUp)
{
    CURVE_IN *in = curve->curveIn;
    CURVE_OUT *out = curve->curveOut;

    /* curve segment number, configed by host tool */
    in->groupNum = startUp->pStCfg->curveSegNum;

    if (0 == out->calcEndFlag)
    {
        switch (in->groupNum)
        {
        case 0:
            StartUp_SegmentCurve_Default_M2(curve, startUp);
        break;

        case 1:
            StartUp_SegmentCurve_Part1_M2(curve);
        break;

        case 2:
            StartUp_SegmentCurve_Part2_M2(curve);
        break;

        case 3:
            StartUp_SegmentCurve_Part3_M2(curve);
        break;

        case 4:
            StartUp_SegmentCurve_Part4_M2(curve);
        break;

        case 5:
            StartUp_SegmentCurve_Part5_M2(curve);
        break;

        default:
            StartUp_SegmentCurve_Default_M2(curve, startUp);
        break;
        }
    }
    else
    {
    }
}

/*!
* @brief Calculation of electric angle in open loop by integral.
*
* @param[in] freqPu, target frequency
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return none
*/
void OpenLoopElecThetaCalc(float freqPu, FOC_VARS *focVars, STARTUP_CTRL *startUp)
{
    float deltaTheta = 0;
    float freqAbs = 0;
    float operatingFreq = 0;
    float freqBase = 0;

    freqAbs = (float)(MathFloat_Abs(freqPu));
    operatingFreq = focVars->pCfg->speedFreq;

    freqBase = focVars->pCfg->baseFreq;

    deltaTheta = (float)freqAbs * freqBase / operatingFreq;
    if (1 == focVars->pCfg->motorDir)
    {
        startUp->startUpElecTheta += deltaTheta;
    }
    else if (-1 == focVars->pCfg->motorDir)
    {
        startUp->startUpElecTheta -= deltaTheta;
    }
    else
    {
        startUp->startUpElecTheta += deltaTheta;
    }

    if (startUp->startUpElecTheta > 1.0f)
    {
        startUp->startUpElecTheta -= 1.0f;
    }
    else if (startUp->startUpElecTheta < 0)
    {
        startUp->startUpElecTheta += 1.0f;
    }
    else
    {
    }

}

/*!
* @brief Open loop startup curve reference value calculation main process.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUpCurve_Process(STARTUP_CURVE *curve, START_UP *startUp)
{
    if (startUp->pStCfg->curveType == CURVE_TYPE_STRAIGHT)
    {
        StartUp_Straight(curve, startUp);
    }
    else if (startUp->pStCfg->curveType == CURVE_TYPE_CUSTOMER)
    {
        StartUp_Customer(curve, startUp);
    }
    else
    {
    }
}

/*!
* @brief Open loop startup curve reference value calculation main process of Motor 2.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUpCurve_Process_M2(STARTUP_CURVE *curve, START_UP *startUp)
{
    if (startUp->pStCfg->curveType == CURVE_TYPE_STRAIGHT)
    {
        StartUp_Straight_M2(curve, startUp);
    }
    else if (startUp->pStCfg->curveType == CURVE_TYPE_CUSTOMER)
    {
        StartUp_Customer_M2(curve, startUp);
    }
    else
    {
    }
}

/*!
* @brief Open loop startup curve reference value calculation main process, used in close loop to open loop switch..
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUpCurve_Process1(STARTUP_CURVE *curve, START_UP *startUp)
{
    StartUp_Straight1(curve, startUp);
}

/*!
* @brief Open loop startup curve reference value calculation main process of Motor 2, used in close loop to open loop switch..
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUpCurve_Process1_M2(STARTUP_CURVE *curve, START_UP *startUp)
{
    StartUp_Straight1_M2(curve, startUp);
}

/*!
* @brief Open loop startup curve reference value calculation main process, used in open loop to close loop switch.
*
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] startUp: pointer to START_UP structure
* @param[in] focVars: pointer to FOC_VARS structure
* @return none
*/
void StartUpCurve_Process2(STARTUP_CURVE *curve, START_UP *startUp, FOC_VARS *focVars)
{
    StartUp_Straight2(curve, startUp, focVars);
}

/*!
* @brief Open loop startup curve angle calculation process.
*
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return none
*/
void StartUpCurve_Angle_Process(CURVE_OUT *out, FOC_VARS *focVars, STARTUP_CTRL *startUp)
{
    float freqSyn;

    freqSyn = (float)(out->frqRefValue) * focVars->pCfg->motorDir;     /*!< need run direction to set negative value */

    OpenLoopElecThetaCalc(freqSyn, focVars, startUp);
}

/*!
* @brief Open loop estimate frequency judgement.
*
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] pidParam: pointer to PID_CFG structure
* @return none
*/
void StartUpFreqJudge(STARTUP_CTRL *startUp, SPEED_RAMP_HANDLE *speed, MOTOR_PARAM_PU *paramPu, PID_CFG *pidParam)
{
#if (defined MOTOR_ONE_TYPE)
    float estFreqUpper = 0;
    float estFreqLower = 0;
    float tempFreq = 0;
    float estFreqAbs = 0;

    if (speed->speedFbk < 0)
    {
        estFreqAbs = 0 - (speed->speedFbk);
    }
    else
    {
        estFreqAbs = speed->speedFbk;
    }
    tempFreq = (float)(startUp->synSpd) * paramPu->pairsb;
    estFreqUpper = (float)(tempFreq * MathFloat_Mpy(ONE_BASE_FREQ, MAX_SYNC_SPEED_THRES));
    estFreqLower = (float)(tempFreq * MathFloat_Mpy(ONE_BASE_FREQ, MIN_SYNC_SPEED_THRES));
    if ((estFreqAbs > estFreqLower) && (estFreqAbs < estFreqUpper))
    {
        startUp->synCount++;
        if (startUp->synCount > startUp->synTime)
        {
            startUp->synFreReach = 1;
            if (pidParam->curPidSelfLearn == (uint16_t)DISABLE)
            {
                Acr_SetCurD_PICof(pidParam->fluxKp, pidParam->fluxKi, 0, 0, &g_acrCurIdCof);
                Acr_SetCurQ_PICof(pidParam->torqueKp, pidParam->torqueKi, 0, 0, &g_acrCurIqCof);
            }
            else
            {
            }
        }
        else
        {
            startUp->synFreReach = 0;
        }
    }
    else
    {
    }
#endif
}

/*!
* @brief Open loop estimate frequency judgement of Motor 2.
*
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] pidParam: pointer to PID_CFG structure
* @return none
*/
void StartUpFreqJudge_M2(STARTUP_CTRL *startUp, SPEED_RAMP_HANDLE *speed, MOTOR_PARAM_PU *paramPu, PID_CFG *pidParam)
{
#if (defined MOTOR_TWO_TYPE)
    float estFreqUpper = 0;
    float estFreqLower = 0;
    float tempFreq = 0;
    float estFreqAbs = 0;

    if (speed->speedFbk < 0)
    {
        estFreqAbs = 0 - (speed->speedFbk);
    }
    else
    {
        estFreqAbs = speed->speedFbk;
    }
    tempFreq = (float)(startUp->synSpd) * paramPu->pairsb;
    estFreqUpper = (float)(tempFreq * MathFloat_Mpy(ONE_BASE_FREQ_M2, MAX_SYNC_SPEED_THRES_M2));
    estFreqLower = (float)(tempFreq * MathFloat_Mpy(ONE_BASE_FREQ_M2, MIN_SYNC_SPEED_THRES_M2));
    if ((estFreqAbs > estFreqLower) && (estFreqAbs < estFreqUpper))
    {
        startUp->synCount++;
        if (startUp->synCount > startUp->synTime)
        {
            startUp->synFreReach = 1;
            if (pidParam->curPidSelfLearn == (uint16_t)DISABLE)
            {
                Acr_SetCurD_PICof(pidParam->fluxKp, pidParam->fluxKi, 0, 0, &g_acrCurIdCof_M2);
                Acr_SetCurQ_PICof(pidParam->torqueKp, pidParam->torqueKi, 0, 0, &g_acrCurIqCof_M2);
            }
            else
            {
            }
        }
        else
        {
            startUp->synFreReach = 0;
        }
    }
    else
    {
    }
#endif
}

/*!
* @brief StartUp control task in foc process.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to START_UP structure
* @param[in] speed: pointer to ESTIMATED_SPEED structure
* @param[in] pll: pointer to PID_REGULATOR structure
* @param[in] asr: pointer to PID_REGULATOR structure
* @param[in] stCurve: pointer to STARTUP_CURVE structure
* @return none
*/
void StartUp_Ctrl_Foc(STC_PARAM *stc, START_UP *startUp, ESTIMATED_SPEED *speed, PID_REGULATOR *pll, PID_REGULATOR *asr, STARTUP_CURVE *stCurve)
{
//    FOC_VARS *focVars = stc->pFOCVars;

//#if (defined PLL_ANGLE_ESTIMATION)
//    g_rotationalSpeed.estSpeed = g_asrSpeedEstPIOut.outPuFliter;
//#else
//#endif

    switch (startUp->pStCtrl->startUpFlag)
    {
    case 1:
    //Bootstrap_Charging_Process();
//    PWM_Enable();
    startUp->pStCtrl->startUpFlag = 2;
    break;

    case 2:
#if (defined FOC_PMSM)
    #if (defined PULSE_INJECTION)
    startUp->pStCtrl->startUpFlag = 3;
    #elif (defined INIT_POS_DETEC)
    startUp->pStCtrl->startUpFlag = 3;
    #endif
#elif (defined FOC_ACIM)
//    startUp->pStCtrl->startUpFlag = 3;
#endif
    break;

    case 3:
    Openloop_Acceleration_Process(stc, startUp, speed, pll, stCurve);
    break;

    case 4:
    Closedloop_Deceleration_Process(stc, startUp, speed, pll, asr);
    break;

    case 5:
    case 6:
    Closedloop_Process(stc, startUp, speed, pll);
    break;

#if (defined CLOSE_TO_OPEN_SWITCH)
    case 7:
    CloseToOpenloop_Process(stc, startUp, speed, pll);
    break;

    case 8:
    OpenToCloseloop_Process(stc, startUp, speed, pll);
    break;
#endif

    default:
    break;
    }
#if (defined FOC_PMSM)
    #if (defined ELEC_FREQUENCE_HIGH)
    Asr_EstSpeedCalc(startUp->pStCtrl->elecThetaEst, speed);
    #endif
#else
#endif
}

/*!
* @brief StartUp control task in foc process of Motor 2.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to START_UP structure
* @param[in] speed: pointer to ESTIMATED_SPEED structure
* @param[in] pll: pointer to PID_REGULATOR structure
* @param[in] asr: pointer to PID_REGULATOR structure
* @param[in] stCurve: pointer to STARTUP_CURVE structure
* @return none
*/
void StartUp_Ctrl_Foc_M2(STC_PARAM *stc, START_UP *startUp, ESTIMATED_SPEED *speed, PID_REGULATOR *pll, PID_REGULATOR *asr, STARTUP_CURVE *stCurve)
{
    switch (startUp->pStCtrl->startUpFlag)
    {
    case 1:
    startUp->pStCtrl->startUpFlag = 2;
    break;

    case 2:
#if (defined FOC_PMSM_M2)
    #if (defined PULSE_INJECTION_M2)
    startUp->pStCtrl->startUpFlag = 3;
    #elif (defined INIT_POS_DETEC_M2)
    startUp->pStCtrl->startUpFlag = 3;
    #endif
#elif (defined FOC_ACIM_M2)
//    startUp->pStCtrl->startUpFlag = 3;
#endif
    break;

    case 3:
    Openloop_Acceleration_Process_M2(stc, startUp, speed, pll, stCurve);
    break;

    case 4:
    Closedloop_Deceleration_Process_M2(stc, startUp, speed, pll, asr);
    break;

    case 5:
    case 6:
    Closedloop_Process_M2(stc, startUp, speed, pll);
    break;

#if (defined CLOSE_TO_OPEN_SWITCH_M2)
    case 7:
    CloseToOpenloop_Process_M2(stc, startUp, speed, pll);
    break;

    case 8:
    OpenToCloseloop_Process_M2(stc, startUp, speed, pll);
    break;
#endif

    default:
    break;
    }
#if (defined FOC_PMSM_M2)
    #if (defined ELEC_FREQUENCE_HIGH_M2)
    Asr_EstSpeedCalc(startUp->pStCtrl->elecThetaEst, speed);
    #endif
#else
#endif
}

/*!
* @brief StartUp control task in 1ms period.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to START_UP structure
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] asr: pointer to PID_REGULATOR structure
* @param[in] acr: pointer to PID_REGULATOR structure
* @param[in] pidcfg: pointer to PID_CFG structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void StartUp_Ctrl_1ms(STC_PARAM *stc, START_UP *startUp, STARTUP_CURVE *curve, PID_REGULATOR *asr, PID_REGULATOR *acr, PID_CFG *pidcfg, SPEED_RAMP_HANDLE *speed)
{
    FOC_VARS *focVars = stc->pFOCVars;
#if (defined IF_STARTUP) || (defined VF_STARTUP)
    CURVE_OUT *out = curve->curveOut;
#endif

    if (startUp->pStCtrl->startUpFlag == 3)
    {
        #if (defined IF_STARTUP) || (defined VF_STARTUP)

        StartUpCurve_Process(curve, startUp);
        StartUpCurve_Angle_Process(out, focVars, startUp->pStCtrl);

        focVars->pCtrl->idRef = 0;
        focVars->pCtrl->iqRef = (float)out->curRefValue * focVars->pCfg->motorDir;

        if (1 == curve->curveOut->calcEndFlag)
        {
            StartUpFreqJudge(startUp->pStCtrl, speed, stc->pMotorPu, pidcfg);
        }
        else
        {
        }

        Set_PID_Ref(asr, (float)out->frqRefValue * focVars->pCfg->motorDir);
        #endif
    }

    else if (startUp->pStCtrl->startUpFlag == 4)
    {
    }
    else if (startUp->pStCtrl->startUpFlag == 5)
    {
        #if (defined TAIL_HEAD_WIND)
        startUp->pStCtrl->countStartUpST5++;
        if (startUp->pStCtrl->countStartUpST5 < 2)
        {
            focVars->pCtrl->idRef = 0;
            focVars->pCtrl->iqRef = 0;
        }
        else
        {
            ASR_SpeedLoop_Process(speed, asr, focVars->pCtrl);
            startUp->pStCtrl->countStartUpST5 = 0;
            startUp->pStCtrl->startUpFlag = 6;
        }
        #else
        ASR_SpeedLoop_Process(speed, asr, focVars->pCtrl);
        startUp->pStCtrl->startUpFlag = 6;
        #endif
        /* update smc parameter */
        #if (defined SMC_OBSERVE)
        Smc_SetParam(stc->smc->smoCfg->maxErr1, stc->smc->smoCfg->kslide1, stc->smc->smoCfg->kslf1, stc->smc->smoEst);
        #endif
    }
    else if (startUp->pStCtrl->startUpFlag == 6)
    {
        ASR_SpeedLoop_Process(speed, asr, focVars->pCtrl);

        #if (defined CLOSE_TO_OPEN_SWITCH)
        startUp->pStCtrl->synFreReach = 0;
        startUp->pStCtrl->synCount = 0;
//        g_startUpCurveOut.calcEndFlag1 = 0;
//        g_startUpCurveOut.calcEndFlag2 = 0;
        out->calcEndFlag1 = 0;
        out->calcEndFlag2 = 0;
        if (MathFloat_Abs(speed->speedFbk) <= (focVars->pCfg->speedMinLimit - startUp->pStCfg->speedDelayZone))
        {
            startUp->pStCtrl->startUpFlag = 7;
            startUp->pStCfg->curveCurStart1 = MathFloat_Abs(acr->pPidInput->refPu);

            startUp->pStCfg->curveFreqStart1 = MathFloat_Abs(asr->pPidInput->fbkPu);
            startUp->pStCtrl->startUpElecTheta = startUp->pStCtrl->elecThetaEst;

            Asr_SpeedPIInit(asr, pPidCfg);
            speed->speedTargetRamp = focVars->pCfg->speedMinLimit * focVars->pCfg->motorDir;
        }
        #endif

        #if (defined QUICK_DESATURATION)
        ASR_Quick_Desaturation(asr, &g_acrCurIqPIOut);
        #endif
    }
#if (defined CLOSE_TO_OPEN_SWITCH)
    else if (startUp->pStCtrl->startUpFlag == 7)
    {
        #if (defined IF_STARTUP) || (defined VF_STARTUP)
        StartUpCurve_Process1(curve, startUp);
        StartUpCurve_Angle_Process(out, focVars, startUp->pStCtrl);

        focVars->pCtrl->idRef = 0;
        focVars->pCtrl->iqRef = (float)out->curRefValue * focVars->pCfg->motorDir;

        if (MathFloat_Abs(speed.speedTarget) >= (focVars->pCfg->speedMinLimit + startUp->pStCfg->speedDelayZone))
        {
            startUp->pStCtrl->startUpFlag = 8;
        }

        #endif
    }
    else if (startUp->pStCtrl->startUpFlag == 8)
    {
        #if (defined IF_STARTUP) || (defined VF_STARTUP)
        StartUpCurve_Process2(curve, startUp, focVars);
        StartUpCurve_Angle_Process(out, focVars, startUp->pStCtrl);

        focVars->pCtrl->idRef = 0;
        focVars->pCtrl->iqRef = (float)out->curRefValue * focVars->pCfg->motorDir;

        if (1 == g_startUpCurveOut.calcEndFlag2)
        {
            StartUpFreqJudge(startUp->pStCtrl, speed, stc->pMotorPu, pPidCfg);
        }
        else
        {
        }
        #endif
    }
#endif
    else
    {
    }

    return;
}

/*!
* @brief StartUp control task in 1ms period of Motor 2.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] startUp: pointer to START_UP structure
* @param[in] curve: pointer to STARTUP_CURVE structure
* @param[in] asr: pointer to PID_REGULATOR structure
* @param[in] acr: pointer to PID_REGULATOR structure
* @param[in] pidcfg: pointer to PID_CFG structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void StartUp_Ctrl_1ms_M2(STC_PARAM *stc, START_UP *startUp, STARTUP_CURVE *curve, PID_REGULATOR *asr, PID_REGULATOR *acr, PID_CFG *pidcfg, SPEED_RAMP_HANDLE *speed)
{
    FOC_VARS *focVars = stc->pFOCVars;
#if (defined IF_STARTUP_M2) || (defined VF_STARTUP_M2)
    CURVE_OUT *out = curve->curveOut;
#endif

    if (startUp->pStCtrl->startUpFlag == 3)
    {
        #if (defined IF_STARTUP_M2) || (defined VF_STARTUP_M2)

        StartUpCurve_Process_M2(curve, startUp);
        StartUpCurve_Angle_Process(out, focVars, startUp->pStCtrl);

        focVars->pCtrl->idRef = 0;
        focVars->pCtrl->iqRef = (float)out->curRefValue * focVars->pCfg->motorDir;

        if (1 == curve->curveOut->calcEndFlag)
        {
            StartUpFreqJudge_M2(startUp->pStCtrl, speed, stc->pMotorPu, pidcfg);
        }
        else
        {
        }

        Set_PID_Ref(asr, (float)out->frqRefValue * focVars->pCfg->motorDir);
        #endif
    }

    else if (startUp->pStCtrl->startUpFlag == 4)
    {
    }
    else if (startUp->pStCtrl->startUpFlag == 5)
    {
        #if (defined TAIL_HEAD_WIND_M2)
        startUp->pStCtrl->countStartUpST5++;
        if (startUp->pStCtrl->countStartUpST5 < 2)
        {
            focVars->pCtrl->idRef = 0;
            focVars->pCtrl->iqRef = 0;
        }
        else
        {
            ASR_SpeedLoop_Process(speed, asr, focVars->pCtrl);
            startUp->pStCtrl->countStartUpST5 = 0;
            startUp->pStCtrl->startUpFlag = 6;
        }
        #else
        ASR_SpeedLoop_Process(speed, asr, focVars->pCtrl);
        startUp->pStCtrl->startUpFlag = 6;
        #endif

        /* update smc parameter */
        #if (defined SMC_OBSERVE_M2)
        Smc_SetParam(stc->smc->smoCfg->maxErr1, stc->smc->smoCfg->kslide1, stc->smc->smoCfg->kslf1, stc->smc->smoEst);
        #endif
    }
    else if (startUp->pStCtrl->startUpFlag == 6)
    {
        ASR_SpeedLoop_Process(speed, asr, focVars->pCtrl);

        #if (defined CLOSE_TO_OPEN_SWITCH_M2)
        startUp->pStCtrl->synFreReach = 0;
        startUp->pStCtrl->synCount = 0;
//        g_startUpCurveOut.calcEndFlag1 = 0;
//        g_startUpCurveOut.calcEndFlag2 = 0;
        out->calcEndFlag1 = 0;
        out->calcEndFlag2 = 0;
        if (MathFloat_Abs(speed->speedFbk) <= (focVars->pCfg->speedMinLimit - startUp->pStCfg->speedDelayZone))
        {
            startUp->pStCtrl->startUpFlag = 7;
            startUp->pStCfg->curveCurStart1 = MathFloat_Abs(acr->pPidInput->refPu);

            startUp->pStCfg->curveFreqStart1 = MathFloat_Abs(asr->pPidInput->fbkPu);
            startUp->pStCtrl->startUpElecTheta = startUp->pStCtrl->elecThetaEst;

            Asr_SpeedPIInit(asr, pPidCfg);
            speed->speedTargetRamp = focVars->pCfg->speedMinLimit * focVars->pCfg->motorDir;
        }
        #endif

        #if (defined QUICK_DESATURATION_M2)
        ASR_Quick_Desaturation_M2(asr, &g_acrCurIqPIOut);
        #endif
    }
#if (defined CLOSE_TO_OPEN_SWITCH_M2)
    else if (startUp->pStCtrl->startUpFlag == 7)
    {
        #if (defined IF_STARTUP_M2) || (defined VF_STARTUP_M2)
        StartUpCurve_Process1_M2(curve, startUp);
        StartUpCurve_Angle_Process(out, focVars, startUp->pStCtrl);

        focVars->pCtrl->idRef = 0;
        focVars->pCtrl->iqRef = (float)out->curRefValue * focVars->pCfg->motorDir;

        if (MathFloat_Abs(speed.speedTarget) >= (focVars->pCfg->speedMinLimit + startUp->pStCfg->speedDelayZone))
        {
            startUp->pStCtrl->startUpFlag = 8;
        }

        #endif
    }
    else if (startUp->pStCtrl->startUpFlag == 8)
    {
        #if (defined IF_STARTUP_M2) || (defined VF_STARTUP_M2)
        StartUpCurve_Process2(curve, startUp, focVars);
        StartUpCurve_Angle_Process(out, focVars, startUp->pStCtrl);

        focVars->pCtrl->idRef = 0;
        focVars->pCtrl->iqRef = (float)out->curRefValue * focVars->pCfg->motorDir;

        if (1 == g_startUpCurveOut.calcEndFlag2)
        {
            StartUpFreqJudge_M2(startUp->pStCtrl, speed, stc->pMotorPu, pPidCfg);
        }
        else
        {
        }
        #endif
    }
#endif
    else
    {
    }

    return;
}

/*!
* @brief Transfer the electrical angle of the motor during the startup phase.
*
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return startUpElecTheta: electric angle of startup process
*/
float Get_Sensorless_ElecAngle(STARTUP_CTRL *startUp)
{
    return startUp->startUpElecTheta;
}

/*!
* @brief Asr coefficients adaptive calculation.
*
* @param[in] none
* @return none
*/
//void Acim_AsrPiCoefUpdate(void)
//{
//#if (defined FOC_ACIM)
//    int32_t tempAsrKp, tempAsrKi;
//    int16_t temp_AsrKpSetFrq1 = Math_IQ(0.1);
//    int16_t temp_AsrKpSetFrq2 = Math_IQ(0.8);

//#if (defined SENSORLESS_ACIM)
//    int32_t temp_AsrKp1 = Math_IQ(0.6);
//    int32_t temp_AsrKi1 = Math_IQ(0.005);
//    int32_t temp_AsrKp2 = Math_IQ(0.9);
//    int32_t temp_AsrKi2 = Math_IQ(0.005);
//#elif (defined SENSOR_ACIM)
//    int32_t temp_AsrKp1 = Math_IQ(0.2);
//    int32_t temp_AsrKi1 = Math_IQ(0.002);
//    int32_t temp_AsrKp2 = Math_IQ(0.6);
//    int32_t temp_AsrKi2 = Math_IQ(0.005);
//#else
//    int32_t temp_AsrKp1 = Math_IQ(0.6);
//    int32_t temp_AsrKi1 = Math_IQ(0.005);
//    int32_t temp_AsrKp2 = Math_IQ(0.9);
//    int32_t temp_AsrKi2 = Math_IQ(0.005);
//#endif

//    if (g_speedCommand.speedTargetRamp < temp_AsrKpSetFrq1)
//    {
//        tempAsrKp = temp_AsrKp1;
//        tempAsrKi = temp_AsrKi1;
//    }
//    else if (g_speedCommand.speedTargetRamp > temp_AsrKpSetFrq1)
//    {
//        tempAsrKp = temp_AsrKp2;
//        tempAsrKi = temp_AsrKi2;
//    }
//    else
//    {
//        tempAsrKp = (g_speedCommand.speedTargetRamp - temp_AsrKpSetFrq2) * (temp_AsrKp2 - temp_AsrKp1) \
//                    / (temp_AsrKpSetFrq2 - temp_AsrKpSetFrq1) + temp_AsrKp2;
//        tempAsrKi = (g_speedCommand.speedTargetRamp - temp_AsrKpSetFrq2) * (temp_AsrKi2 - temp_AsrKi1) \
//                    / (temp_AsrKpSetFrq2 - temp_AsrKpSetFrq1) + temp_AsrKi2;
//    }
//    Asr_SetSpeed_PICof(tempAsrKp, tempAsrKi, 0, 0, g_pidAsr.pPidCoef);
//#endif
//}

/*!
* @brief StartUp control task in 1ms period of ACIM SLVC.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
//void Acim_Startup_1ms(FOC_VARS *focVars, START_UP *startUp)
//{
//#if (defined FOC_ACIM)
//    if (startUp->pStCtrl->startUpFlag == 3)
//    {
//        StartUpCurve_Process(&g_startUpCurveIn, &g_startUpCurveOut, &g_startUpCurveTime, startUp);
//        StartUpCurve_Angle_Process(&g_startUpCurveOut, focVars, startUp->pStCtrl);

//        focVars->pCtrl->idRef = g_acimFocCtrl.mot->excitCur;

//        if (1 == g_startUpCurveOut.calcEndFlag)
//        {
//            StartUpFreqJudge(startUp->pStCtrl, &g_speedCommand, &g_motorParamPu, pPidCfg);
//        }
//        else
//        {
//        }

//        focVars->pCtrl->iqRef = 1000 * pFocVarsCfg->motorDir;

//        Set_PID_Ref(&g_pidAsr, (int32_t)g_startUpCurveOut.frqRefValue * pFocVarsCfg->motorDir);

//    }

//    else if (startUp->pStCtrl->startUpFlag == 4)
//    {
////        ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, focVars->pCtrl);
//    }
//    else if (startUp->pStCtrl->startUpFlag == 5)
//    {
//        ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, focVars->pCtrl);
//        startUp->pStCtrl->startUpFlag = 6;
//    }
//    else if (startUp->pStCtrl->startUpFlag >= 6)
//    {
//        ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, focVars->pCtrl);
//    }
//    else
//    {
//    }

//    return;
//#endif
//}

/* =============================================  EOF  ============================================== */
