/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __PARAM_INIT_H__
#define __PARAM_INIT_H__

/*!
* @file param_init.h
*
* @brief This file provides per unit value of motor parameters and calculation of ADC per unit coefficient functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include <string.h>
#include "iqmath.h"

/* ============================================  Define  ============================================ */
#define DELAY_100MS             (100UL)             /*!< Delay time value 100ms */
#define DELAY_500MS             (500UL)             /*!< Delay time value 500ms */
#define DELAY_1000MS            (1000ul)            /*!< Delay time value 1000ms */
#define DELAY_2000MS            (2000UL)            /*!< Delay time value 2000ms */
#define DELAY_5000MS            (5000UL)            /*!< Delay time value 5000ms */

#define USE_SVPWM_VOLT          (0)                 /*!< Use the SVPWM input voltage */
#define USE_VOLT_RESTRUCT       (1)                 /*!< Use the reconstruction voltage */

#define ATAN_WAY                (0)                 /*!< Use the arctan method */
#define PLL_WAY                 (1)                 /*!< Use the PLL method */

#define FORWARD_RUN             ((int16_t)1)        /*!< Motor run direction is forward */
#define REVERSE_RUN             (-(int16_t)1)       /*!< Motor run direction is reverse */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Motor parameter structure.
*/
typedef struct
{
    uint16_t motorType;             /*!< Motor type */
    uint32_t encoderLines;          /*!< Encoder pulse number per Motor (constant value) */
    float statorRs;              /*!< Stator resistance (unit:1mohm) */
    float statorLs;              /*!< Stator inductance (unit:1uH) */
    float ld;                    /*!< Motor D-axis inductance (unit:1uH) */
    float lq;                    /*!< Motor q-axis inductance (unit:1uH) */
    float rotorRs;               /*!< Induction motor Rotor resistance (unit:1mohm) */

    float lr;                    /*!< Induction motor rotor inductance (unit:1uH) */
    float lm;                    /*!< Induction motor mutual inductance (unit:1uH) */
    float lleak;                 /*!< Induction motor leakage inductance (unit:1uH) */
    float noLoadExcite;          /*!< Induction motor no load exciting current (unit:1A) */

    float jm;                    /*!< Rotational intertia (unit:ukg*m2) */
    float polePairs;             /*!< Number of pole pairs (constant value) */

    float bEmf;                     /*!< Back emf coefficient (unit:Vrms/Krpm) */
    float maxCur;                /*!< Maximum motor current of the motor (unit:1A,pk) */
    float ratedVoltage;          /*!< rated voltage of the HW board (unit:1V) */
    float maxSpeed;              /*!< Maximum speed of the motor (unit:1rpm) */
    float maxFreq;               /*!< Maximum speed of the motor (unit:HZ) */

    float maxTorque;             /*!< Maximum torque of the motor (unit:1mNm) */
    float maxPower;              /*!< Maximum power of the motor (unit:W) */
} MOTOR_PARAM;

/*!
* @brief Base value and Pu(per unit) value structure of inverter.
*/
typedef struct
{
    float ibAp;                  /*!< 0.1A, Current base */
    float ubVt;                  /*!< 0.1V, Voltage base */
    float fbHz;                  /*!< Hz,Frequency base */
    float pairsb;                /*!< motor pairs base */
    float vbRpm;                 /*!< rpm,Velocity base */
    float tbUs;                  /*!< 0.1us,Time base */
    float pbW;                   /*!< 0.01Kw, Power base */
    float rbOm;                  /*!< 0.01Ohm, Resistance base */
    float web;                   /*!< rad/s, base */
    float lbHm;                  /*!< uH, Inductance base */
    float tbNm;                  /*!< 2e-19Nm, Torque base */
    float jb;                    /*!< Unit:10-10*kg*m2 rotational inertia base */
    float fluxbWb;               /*!< Unit:0.001Wb */

    float tCurCtrPu;             /*!< Q15, Pu value of Current control period */
    float tSpeedCtrPu;           /*!< Q12, Pu value of Speed control period */
    float tPWMPu;                /*!< Q15, Pu value of PWM control period */
    float maxThetaPerCurTcPu;    /*!< Q15, Pu value of maximum operating angle in one current loop cycle */
    float maxThetaPerSpdTcPu;    /*!< Q15, Pu value of maximum operating angle in one speed loop cycle */

    float mTrqMaxPu;             /*!< Pu value of Maximum torque of the motor */
    float mSpeedMaxPu;           /*!< Pu value of Maximum speed of the motor */
    float mSpeedMinPu;           /*!< Pu value of Minimum speed of the motor */
    float mFreqMaxPu;            /*!< Pu value of Maximum electrical frequency of the motor */
    float mFreqMinPu;            /*!< Pu value of Minimum electrical frequency of the motor */
    float mImMaxPu;              /*!< Pu value of Maximum motor current */
    float mImMinPu;              /*!< Pu value of Minimum motor current */
    float mImRatePu;             /*!< Pu value of Motor rated current */
    float ratePowerpu;           /*!< Pu value of Motor rated power */

    float rsPu;                  /*!< Pu value of Motor resistance */
    float ldPu;                  /*!< Pu value of Motor D-axis inductance */
    float lqPu;                  /*!< Pu value of Motor q-axis inductance */
    float rs4Pu;                 /*!< Pu value of Induction motor RS resistance */
    float rr4Pu;                 /*!< Pu value of Induction motor RR resistance */
    float ls4Pu;                 /*!< Pu value of Induction motor LS inductance */
    float lr4Pu;                 /*!< Pu value of Induction motor LR inductance */
    float lm4Pu;                 /*!< Pu value of Induction motor inductance */
    float lleak4Pu;              /*!< Pu value of Induction motor leakage inductance */

    float jmPu;                  /*!< Pu value of motor rotational inertia */
    float pairsPu;               /*!< Pu value of motor Number of pole pairs */
    float fluxPu;                /*!< Pu value of motor magnetic flux */
    float mTrqRatePu;            /*!< Pu value of motor rated torque of the motor */

    float iratePu;               /*!< Pu value of motor Motor rated current */
    float mIMaxPu;               /*!< Pu value of motor Maximum  current */
    float vspCmdMaxSpeedPu;      /*!< Pu value of motor Maximum speed command */
    float vspCmdMinSpeedPu;      /*!< Pu value of motor Minimum speed command */

    float curBandWidthPu;        /*!< Pu value of current bandwidth */
    float speedBandWidthPu;      /*!< Pu value of speed bandwidth */
} MOTOR_PARAM_PU;

/*!
* @brief Hardware board parameter structure.
*/
typedef struct
{
    uint16_t pwmPolarity;           /*!< Pwm active polarity */

    uint16_t hwDeadTime;            /*!< Hardware dead time size,usually refers to the deadband size of the gate driver */
    uint16_t swDeadTime;            /*!< Software dead time size,usually refers to the deadband size set by mcu */

    float maxVoltage;            /*!< Maximum operating voltage of the controller */
    float normalVoltage;         /*!< Normal operating voltage of the controller */
    float minVoltage;            /*!< Minimum operating voltage of the controller */
    float vbusAttenuateVal;         /*!< Voltage attenuation ratio */

    float maxCurrent;            /*!< Maximum operating current of the controller */

    float rShunt;                   /*!< Resistance value for current samping */
    float opGainVal;                /*!< Amplification factor of operational amplifier when sampling current */
    float icsGainVal;               /*!< Amplification factor of ICS when sampling current */
//    uint16_t opGainPolarity;      /*!< Polarity of operational amplifier when sampling current */
    uint16_t adcRange;              /*!< ADC sampling accuracy of MCU */
    float adcRefV;                  /*!< ADC sampling voltage range of MCU  */
    uint16_t overCurrentPolarity;   /*!< Polarity setting of over current protection */

} HW_BOARD;

/*!
* @brief Common configuration parameter structure of FOC.
*/
typedef struct
{
    int16_t motorDir;               /*!< Motor direction */
    uint16_t controlMode;           /*!< Motor Control Mode */
    uint16_t voltObtainWay;         /*!< Sensorless voltage obtain way */
    int16_t angleObtainWay;         /*!< Sensorless angle obtain way */
    int16_t smoWay;                 /*!< Sensorless sliding mode control way, including two order smc and full order smc, 0:two order smo; 1:full order smo */
    int16_t fullOrderSmoFilt;       /*!< Full order smc filter enable switch, 1: means enable filt, 0: means disable filt */
    uint8_t curSampleMode;          /*!< Current sample mode */
    uint8_t pwmId;                  /*!< Motor pwm ID */
    uint8_t pwmChannelU;            /*!< PWM channnel used by phase U */
    uint8_t pwmChannelV;            /*!< PWM channnel used by phase V */
    uint8_t pwmChannelW;            /*!< PWM channnel used by phase W */
    uint8_t pwmChannelT;            /*!< PWM channnel used by Others */

    float speedMaxLimit;          /*!< Max limit of speed command */
    float speedMinLimit;          /*!< Min limit of speed command */
    float stopSpeed;              /*!< Stop speed in slow down stop mode */
    float motorAcceleration;      /*!< Motor speed loop acceleration */
    float motorDeceleration;      /*!< Motor speed loop deacceleration */
    float baseFreq;              /*!< Base frequency */

    uint16_t pwmFreq;               /*!< Motor pwm frequency */
    uint16_t curFreq;               /*!< Motor current loop frequency */
    float speedFreq;             /*!< Motor speed loop frequency */

    uint16_t msUnitInt;             /*!< Ms count value */
    uint16_t halfPwmPeriod;         /*!< Half of pwm period */
    uint16_t pwmPeriod;             /*!< PWM period */
    uint16_t pwmDeadTime;           /*!< PWM dead time */
    uint32_t sysBusClk;             /*!< System bus clock */
    uint16_t pwdtPrescal;           /*!< PWDT prescaler val */

    float atanModeSpeedFilterH;  /*!< Low pass filter coefficient used for ELEC_FREQUENCE_HIGH */
    float atanModeSpeedFilterL;  /*!< Low pass filter coefficient used for ELEC_FREQUENCE_LOW */
} FOC_VARS_CFG;

/*!
* @brief Current loop structure, Q15 means that the value has been converted to Pu value.
*/
typedef struct
{
    uint16_t focExecutionCycleCnt;  /*!< Counter of FOC multi-cycle execution */
    uint16_t pwmOutMask;            /*!< Current loop execution flag */

    float iaPu;                   /*!< float, phase A current value in stator coordinate system */
    float ibPu;                   /*!< float, phase B current value in stator coordinate system */
    float icPu;                   /*!< float, phase C current value in stator coordinate system */
    float ialpha;                 /*!< float, Alfa-axis current in stator coordinate system */
    float ibeta;                  /*!< float, Beta-axis current in stator coordinate system */
    float iqFdb;                  /*!< float, Q-axis feedback current in rotating coordinate system */
    float idFdb;                  /*!< float, D-axis feedback current in rotating coordinate system */
    float iqBeforeLpf;            /*!< float, Q-axis feedback current before lpf in rotating coordinate system */
    float idBeforeLpf;            /*!< float, D-axis feedback current before lpf in rotating coordinate system */
    float iqRef;                  /*!< float, Reference Q-axis feedback current in rotating coordinate system */
    float idRef;                  /*!< float, Reference D-axis feedback current in rotating coordinate system */

    float vq;                     /*!< float, Q-axis voltage */
    float vd;                     /*!< float, D-axis voltage*/
    float vqRef;                  /*!< float, Q-axis target voltage */
    float vdRef;                  /*!< float, D-axis target voltage */
    float valpha;                 /*!< float, Alfa axis voltage */
    float vbeta;                  /*!< float, Beta axis voltage */

    float elecAngle;             /*!< float, Electrical angle (pu) */
    float ialphabeta;            /*!< float, Alfa and Beta axis synthesis current in stator coordinate system */
    float idq;                   /*!< float, D and Q axis synthesis current in rotating coordinate system */
    float vdq;                   /*!< float, D Q-axis synthesis voltage */
    float vdqLimit;              /*!< float, D Q-axis synthesis voltage after limit */

    float uOutputPu;             /*!< float, output voltage pu value */
    float modulationRatio;       /*!< SVPWM Bus voltage modulation ratio/depth, 10000 = 100%, means Udc utilization is 100% */

    float vdcPu;                 /*!< float, Pu value of Vbus voltage */
    float vdqFilter;             /*!< float, D Q-axis synthesis voltage after filter */

    float voltagePhaseA;         /*!< float, A phase voltage */
    float voltagePhaseB;         /*!< float, B phase voltage */
    float voltagePhaseC;         /*!< float, C phase voltage */
    float voltageMax;            /*!< float, phase voltage max value */
    float voltageMin;            /*!< float, phase voltage min value */

    float voltAlphaRestruct;     /*!< float, Rustruct Alpha phase voltage */
    float voltBetaRestruct;      /*!< float, Rustruct Beta phase voltage */
} FOC_VARS_CTRL;

/*!
* @brief Foc parameter structure.
*/
typedef struct
{
    FOC_VARS_CFG *pCfg;             /*!< Common configuration parameter structure of FOC */
    FOC_VARS_CTRL *pCtrl;           /*!< Current loop structure */
} FOC_VARS;

/*!
* @brief The current and voltage per unit coefficient structure, used to convert the sample current to Pu value.
*/
typedef struct {
    float kVdcSamplePu;          /*!< The coefficient of Sample==>Q15 Pu */
    float kCurSamplePu;           /*!< The coefficient of current sample value ==>Q15 Pu */
    float kCurSensorPu;           /*!< The coefficient of current sensor value ==>Q15 Pu */
} ADC_SAMPLE_COF;

/*!
* @brief Sliding mode control configuration parameters structure.
*/
typedef struct
{
    float kslide0;            /*!< Open loop configuration sliding mode gain */
    float maxErr0;            /*!< Open loop configuration sliding mode boundary */
    float kslf0;              /*!< Open loop configuration sliding mode filter coefficient */
    float kslide1;            /*!< Close loop configuration sliding mode gain */
    float maxErr1;            /*!< Close loop configuration sliding mode boundary */
    float kslf1;              /*!< Close loop configuration sliding mode filter coefficient */
    float fullOrderGain;      /*!< Full order smc gain */
} SMC_CFG;

/*!
* @brief Sliding mode control parameters structure.
*/
typedef struct
{
    float kslide;             /*!< Sliding mode gain */
    float maxErr;             /*!< Sliding mode boundary */
    float kslf;               /*!< Sliding mode filter coefficient */
    float kslfa;              /*!< Sliding mode filter coefficient a */
    float kslfb;              /*!< Sliding mode filter coefficient b */
    float gain;               /*!< full order smc gain */

    float fsmopos;            /*!< Sliding mode control fsmopos coefficient */
    float gsmopos;            /*!< Sliding mode control gsmopos coefficient */

    float vAlpha;             /*!< Sliding mode control alpha axis voltage */
    float vBeta;              /*!< Sliding mode control beta axis voltage */

    float zAlpha;             /*!< Sliding mode control alpha axis back emf before filter */
    float zBeta;              /*!< Sliding mode control beta axis back emf before filter */

    float estIalpha;          /*!< Sliding mode control estimated alpha axis current */
    float estIbeta;           /*!< Sliding mode control estimated beta axis current */

    float eAlpha;             /*!< Sliding mode control alpha axis back emf after filter */
    float eBeta;              /*!< Sliding mode control beta axis back emf after filter */
    float bemfAmp;            /*!< Sliding mode control back emf amp */
    float estBemfSq;          /*!< Sliding mode control estimated the square of back emf */

    float estTheta;          /*!< Sliding mode control estimated electrical theta */
    float compCoeff;          /*!< Sliding mode control compensation coefficieng */
    float eAlphaComp;         /*!< Sliding mode control alpha axis back emf compensation value */
    float eBetaComp;          /*!< Sliding mode control beta axis back emf compensation value */
    float estFre;             /*!< Sliding mode control estimated electrical frequency */
    float eAlphaFullOrder;    /*!< Full order sliding mode control alpha axis back emf before filter */
    float eBetaFullOrder;     /*!< Full order sliding mode control beta axis back emf before filter */
    float timeCoeff;          /*!< Sliding mode control time coefficient */
    float cutFreq;           /*!< Low pass filter cut off frequency */
} SMC_ESTIMATOR;

/*!
* @brief Sliding mode control theta compensation structure.
*/
typedef struct
{
    float maxCompTheta;  /*!< Maximum compensation angle */
    float initCompTheta; /*!< Initial compensation angle */
    float cutFreq;       /*!< Cut off frequency */
    float omegaAvg;      /*!< Average speed */
    float theta0Gain;    /*!< Filter compensation angle gain */
    float theta0;        /*!< Filter compensation angle */
} THETA_COMPENSATOR;

typedef struct
{
    SMC_CFG *smoCfg;
    SMC_ESTIMATOR *smoEst;
    THETA_COMPENSATOR *comp;
} SM_OBSERVER;

/*!
* @brief Flux observer sensorless mode control parameters structure.
*/
typedef struct
{
    float estFreq;        /*!< Estimated electrical frequency by flux observer */
    float estTheta;      /*!< Estimated electrical theta by flux observer */
    float cutFreq;       /*!< Stator flux fliter cutoff frequency */
    float timeCoeff;      /*!< Flux observer time coefficient */
    float flitCoeff;      /*!< Stator flux fliter coefficient */

    float emfAlpha;       /*!< Alpha axis back emf */
    float emfBeta;        /*!< Beta axis back emf */
    float emfAmp;         /*!< Back emf amp */
    float emfAlphaComp;   /*!< Alpha axis back emf compensation */
    float emfBetaComp;    /*!< Beta axis back emf compensation */
    float emfCompCoeff;   /*!< Back emf compensation coefficient */

    float staFluxAlpha;   /*!< Alpha axis stator flux */
    float staFluxBeta;    /*!< Beta axis stator flux */
    float rotFluxAlpha;   /*!< Alpha axis rotator flux */
    float rotFluxBeta;    /*!< Beta axis rotator flux */
    float rotFluxAmp;    /*!< Rotator flux amplitide */
} FLUX_OBSERVER_CTRL;

/*!
* @brief Flux observer configuration parameter structure.
*/
typedef struct
{
    float flitCoeff;      /*!< LPF filter coefficient */
    float compCofLimit;  /*!< Back emf compensation uplimit */
} FLUX_OBSERVER_CFG;

/*!
* @brief Flux observer configuration and control parameter structure.
*/
typedef struct
{
    FLUX_OBSERVER_CTRL *pCtrl;      /*!< Flux observer control parameter structure */
    FLUX_OBSERVER_CFG *pCfg;        /*!< Flux observer configuration parameter structure */
} FLUX_OBSERVER;

/*!
* @brief MRAS sensorless mode control parameters structure.
*/
typedef struct
{
    float estTheta;      /*!< Estimated electrical theta by MRAS observer */
    float estFreq;        /*!< Estimated electrical frequency by MRAS observer */
    float estFreqBuf;     /*!< Estimated electrical frequency by MRAS observer */
    float estFreqFilter;  /*!< Estimated electrical frequency by MRAS observer */
    float fMrasPos;       /*!< Model reference adjust system fsmopos coefficient */
    float gMrasPos;       /*!< Model reference adjust system gsmopos coefficient */
    float hMrasPos;       /*!< Model reference adjust system hsmopos coefficient */
    float cofAlpha;       /*!< Model reference adjust system alpha coefficient */
    float cofBeta;        /*!< Model reference adjust system beta coefficient */
    float estId;          /*!< Estimated d axis current */
    float fdbIdFilter;    /*!< Feedback d axis current filter */
    float estIq;          /*!< Estimated q axis current */
    float fdbIqFilter;    /*!< Feedback q axis current filter */
    float vdFilter;       /*!< D axis voltage filter */
    float vqFilter;       /*!< Q axis voltage filter */
} MRAS_OBSERVER;

/*!
* @brief MRAS configuration parameter structure.
*/
typedef struct
{
    float cofAlpha;       /*!< Model reference adjust system alpha coefficient */
    float cofBeta;        /*!< Model reference adjust system beta coefficient */
} MRAS_OBSERVER_CFG;

typedef struct
{
    MRAS_OBSERVER_CFG *mrasCfg;
    MRAS_OBSERVER *mrasCtrl;
} MRAS;

/*!
* @brief Speed and toque control parameter structure.
*/
typedef struct
{
    MOTOR_PARAM *pMotor;            /*!< Motor parameters real vaule */
    MOTOR_PARAM_PU *pMotorPu;       /*!< Motor parameters unit pu */
    FOC_VARS *pFOCVars;             /*!< parameters of FOC control */

    SM_OBSERVER *smc;
    FLUX_OBSERVER *flux;
    MRAS *mras;
} STC_PARAM;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Calculation of the Pu parameters used in speed and torque control.
*
* @param[in] motor:MOTOR_PARAM type pointer
* @param[in] focVarsCfg: variable which use to configure FOC control, pointer to FOC_VARS_CFG structure
* @param[out] param: MOTOR_PARAM_PU type pointer
* @return none
*/
void STC_motorParamPuCalc(MOTOR_PARAM *motor, FOC_VARS_CFG *focVarsCfg, MOTOR_PARAM_PU *paramPu);

/*!
* @brief Current and voltage ADC PU factor coefficient calculation,the calculated coefficient multiplied by the sampling current
*        can be converted current into a Pu value.
*
* @param[in] paramPu: MOTOR_PARAM_PU type pointer
* @param[in] hwBoard: HW_BOARD type pointer
* @param[out] adcPuCof: ADC_SAMPLE_PARAM type pointer
* @return none
*/
void ADC_SampleCofCalc(MOTOR_PARAM_PU *paramPu, HW_BOARD *hwBoard, ADC_SAMPLE_COF *adcPuCof);

/* ======================================  Functions define  ======================================== */

#endif /* __PARAM_INIT_H__ */

/* =============================================  EOF  ============================================== */
